<?php

namespace App\Datatables;
 
use App\Models\Transaction;
use Illuminate\Database\Eloquent\Builder as QueryBuilder;
use \Yajra\DataTables\EloquentDataTable;
use \Yajra\DataTables\Html\Builder as HtmlBuilder;
use \Yajra\DataTables\Html\Button;
use \Yajra\DataTables\Html\Column;
use \Yajra\DataTables\Services\DataTable;
 
class TransactionDataTable extends DataTable
{
    public function dataTable(QueryBuilder $query): EloquentDataTable
    {
        return (new EloquentDataTable($query))->setRowId('id');
    }
 
    public function query(Transaction $model): QueryBuilder
    {
     
        return $model->newQuery();
        
    }
 
    public function html(): HtmlBuilder
    {
     
        return $this->builder()
                    ->setTableId('transaction-table')
                    ->columns($this->getColumns())
                    ->addColumn('coin' , function(){
                        return '<a href="#" class="edit btn btn-primary btn-sm">Edit</a>';
                    })
                    ->minifiedAjax()
                    ->orderBy(1)
                    ->selectStyleSingle()
                    ->buttons([
                        Button::make('add'),
                        Button::make('excel'),
                        Button::make('csv'),
                        Button::make('pdf'),
                        Button::make('print'),
                        Button::make('reset'),
                        Button::make('reload'),
                    ]);
    }
 
    public function getColumns(): array
    {
        return [
            Column::make('txn_id')->title('Reference'),
            Column::make('type')->title('Type'),
            Column::make('created_date')->title('Time'),
            Column::make('coin')->title('Coin'),
            Column::make('amount')->title('Amount'),
            Column::make('status')->title('Status'),
        ];
    }
 
    protected function filename(): string
    {
        return 'Users_'.date('YmdHis');
    }
}