 <?php
 use App\Apis\CryptoCompare;

 use App\Enums\OrderStatus;
 use App\Enums\PlanDetails;
 use App\Enums\PlanStatus;
 use App\Enums\TransactionStatus;
 use App\Enums\TransactionCalcType;
 use App\Enums\TransactionType;


 use App\Models\AccountIo;
 use App\Models\Setting;
 use App\Models\BalanceHistory;
 
 use App\Helpers\MiHash;

 use Carbon\Carbon;
 use Illuminate\Support\Arr;
 use Illuminate\Support\Facades\Request;
 use Illuminate\Support\Facades\Route;
 use Illuminate\Support\Str;
 use Illuminate\Support\HtmlString;
 use Illuminate\Support\Facades\DB;
 use Stevebauman\Location\Facades\Location;

 /**
  * Custom Helper Functions
  *
  *
  * @package MY APP
  * @author Klik
  * @version 1.0.0
  * @since 1.0
  * @return void
  */

 /************** GENERAL FUNCTION ***************/
 if (!function_exists('__is_json')) {
     /**
      * check json value
      * @param $string, $decoded
      * @version 1.0.0
      * @since 1.0
      */
     function __is_json($string, $decoded = false)
     {
         if (is_array($string)) {
             return false;
         }
         json_decode($string);
         $check = json_last_error() == JSON_ERROR_NONE;

         if ($decoded && $check) {
             return json_decode($string);
         }

         return $check;
     }
 }

 if (!function_exists('__json_to_array')) {
     /**
      * check json value
      * @param $string, $decoded
      * @version 1.0.0
      * @since 1.0
      */
     function __json_to_array($string, $array = true)
     {
         if (__is_json($string)) {
             return json_decode($string, $array);
         }

         return $string;
     }
 }

 if (!function_exists('__get_json_value')) {
     /**
      * check json value
      * @param $string, $decoded
      * @version 1.0.0
      * @since 1.0
      */
     function __get_json_value($string, $key)
     {
         $data = __is_json($string) ? json_decode($string) : $string;
         if (isset($data->$key)) {
             return $data->$key;
         }
         return '';
     }
 }

 if (!function_exists('__is_force_https')) {
     /**
      * Check if force to https form configure.
      * Param: Null
      * @version 1.0.0
      * @since 1.0
      */
     function __is_force_https()
     {
         if (config('app.force_https')) {
             return true;
         }

         return false;
     }
 }

 if (!function_exists('__is_collection')) {
     /**
      * Check if force to https form configure.
      * Param: Null
      * @version 1.0.0
      * @since 1.0
      */
     function __is_collection($param): bool
     {
         return (bool) ($param instanceof \Illuminate\Support\Collection || $param instanceof \Illuminate\Database\Eloquent\Collection);
     }
 }

 if (!function_exists('__auto_p')) {
     function __auto_p($pee, $br = true, $add = '')
     {
         $pre_tags = [];

         if (trim($pee) === '') {
             return '';
         }

         $pee = $pee . "\n";
         if (strpos($pee, '<pre') !== false) {
             $pee_parts = explode('</pre>', $pee);
             $last_pee = array_pop($pee_parts);
             $pee = '';
             $i = 0;

             foreach ($pee_parts as $pee_part) {
                 $start = strpos($pee_part, '<pre');
                 if ($start === false) {
                     $pee .= $pee_part;
                     continue;
                 }

                 $name = "<pre pre-tag-$i></pre>";
                 $pre_tags[$name] = substr($pee_part, $start) . '</pre>';

                 $pee .= substr($pee_part, 0, $start) . $name;
                 $i++;
             }

             $pee .= $last_pee;
         }

         $pee = preg_replace('|<br\s*/?>\s*<br\s*/?>|', "\n\n", $pee);

         $allblocks =
             '(?:table|thead|tfoot|caption|col|colgroup|tbody|tr|td|th|div|dl|dd|dt|ul|ol|li|pre|form|map|area|blockquote|address|math|style|p|h[1-6]|hr|fieldset|legend|section|article|aside|hgroup|header|footer|nav|figure|figcaption|details|menu|summary)';

         $pee = preg_replace('!(<' . $allblocks . '[\s/>])!', "\n\n$1", $pee);
         $pee = preg_replace('!(</' . $allblocks . '>)!', "$1\n\n", $pee);
         $pee = str_replace(["\r\n", "\r"], "\n", $pee);
         $pee = __replace_in_html_tags($pee, ["\n" => " <!-- nl --> "]);
         if (strpos($pee, '<option') !== false) {
             $pee = preg_replace('|\s*<option|', '<option', $pee);
             $pee = preg_replace('|</option>\s*|', '</option>', $pee);
         }

         $pee = preg_replace("/\n\n+/", "\n\n", $pee);
         $pees = preg_split('/\n\s*\n/', $pee, -1, PREG_SPLIT_NO_EMPTY);
         $pee = '';

         foreach ($pees as $tinkle) {
             $pee .= '<p>' . trim($tinkle, "\n") . "</p>\n";
         }

         $pee = preg_replace('|<p>\s*</p>|', '', $pee);
         $pee = preg_replace('!<p>([^<]+)</(div|address|form)>!', "<p>$1</p></$2>", $pee);
         $pee = preg_replace('!<p>\s*(</?' . $allblocks . '[^>]*>)\s*</p>!', "$1", $pee);
         $pee = preg_replace("|<p>(<li.+?)</p>|", "$1", $pee);
         $pee = preg_replace('|<p><blockquote([^>]*)>|i', "<blockquote$1><p>", $pee);
         $pee = str_replace('</blockquote></p>', '</p></blockquote>', $pee);
         $pee = preg_replace('!<p>\s*(</?' . $allblocks . '[^>]*>)!', "$1", $pee);
         $pee = preg_replace('!(</?' . $allblocks . '[^>]*>)\s*</p>!', "$1", $pee);

         if ($br) {
             $pee = str_replace(['<br>', '<br/>'], '<br />', $pee);
             $pee = preg_replace('|(?<!<br />)\s*\n|', "<br />\n", $pee);
         }

         $pee = preg_replace('!(</?' . $allblocks . '[^>]*>)\s*<br />!', "$1", $pee);
         $pee = preg_replace('!<br />(\s*</?(?:p|li|div|dl|dd|dt|th|pre|td|ul|ol)[^>]*>)!', '$1', $pee);
         $pee = preg_replace("|\n</p>$|", '</p>', $pee);
         if (!empty($pre_tags)) {
             $pee = str_replace(array_keys($pre_tags), array_values($pre_tags), $pee);
         }

         return $add . $pee;
     }
 }

 if (!function_exists('__replace_in_html_tags')) {
     function __replace_in_html_tags($hstack, $replace_pairs)
     {
         $textarr = preg_split(__get_html_split_regex(), $hstack, -1, PREG_SPLIT_DELIM_CAPTURE);
         $changed = false;

         if (1 === count($replace_pairs)) {
             foreach ($replace_pairs as $needle => $replace);

             for ($i = 1, $c = count($textarr); $i < $c; $i += 2) {
                 if (false !== strpos($textarr[$i], $needle)) {
                     $textarr[$i] = str_replace($needle, $replace, $textarr[$i]);
                     $changed = true;
                 }
             }
         } else {
             $needles = array_keys($replace_pairs);

             for ($i = 1, $c = count($textarr); $i < $c; $i += 2) {
                 foreach ($needles as $needle) {
                     if (false !== strpos($textarr[$i], $needle)) {
                         $textarr[$i] = strtr($textarr[$i], $replace_pairs);
                         $changed = true;
                         break;
                     }
                 }
             }
         }

         if ($changed) {
             $hstack = implode($textarr);
         }

         return $hstack;
     }
 }

 if (!function_exists('__get_html_split_regex')) {
     function __get_html_split_regex()
     {
         static $regex;
         if (!isset($regex)) {
             $coms = '!' . '(?:' . '-(?!->)' . '[^\-]*+' . ')*+' . '(?:-->)?';
             $cdata = '!\[CDATA\[' . '[^\]]*+' . '(?:' . '](?!]>)' . '[^\]]*+' . ')*+' . '(?:]]>)?';
             $escaped = '(?=' . '!--' . '|' . '!\[CDATA\[' . ')' . '(?(?=!-)' . $coms . '|' . $cdata . ')';
             $regex = '/(' . '<' . '(?' . $escaped . '|' . '[^>]*>?' . ')' . ')/';
         }
         return $regex;
     }
 }

 if (!function_exists('__get_mail_branding')) {
     /**
      * @since 1.0
      * @version 1.0.0
      */
     function __get_mail_branding()
     {
         $logo = "HTTPS://jsjsuh";

         return $logo;
     }
 }

 if (!function_exists('__systemDetails')) {
     /**
      * Get system details
      * Param: Null
      * @version 1.0.0
      * @since 1.0
      */
     function __systemDetails()
     {
         $system = [];
         $system['name'] = 'Miczylab';
         $system['version'] = '1.0';
         return $system;
     }
 }

 if (!function_exists('__siteDetails')) {
     /**
      * Get site details
      * Param Null
      * @version 1.0.0
      * @since 1.0
      */
     function __siteDetails($output = 'name')
     {
         $output = !empty($output) ? $output : 'name';
         $infos = [
             'apps' => env('APP_NAME', 'My App'),
             'name' => __system_settings('site_name'),
             'author' => config('app.author'),
             'url' => url('/'),
             'url_only' => str_replace(['https://', 'http://'], '', url('/')),
             'url_app' => config('app.url'),
         ];

         return $output == 'all' ? $infos : Arr::get($infos, $output, '');
     }
 }

 if (!function_exists('__the_hash')) {
     /**
      * @param $data
      * @param $match
      * @return string|boolean
      * @version 1.0.0
      * @since 1.0
      */
     function __the_hash($data, $match = null)
     {
         return MiHash::of($data, $match);
     }
 }

 if (!function_exists('__the_uid')) {
     /**
      * @param $userId
      * @return string
      * @version 1.0.0
      * @since 1.0
      */
     function __the_uid($userID)
     {
         return 'UID' . str_pad($userID, 5, '0', STR_PAD_LEFT);
     }
 }

 if (!function_exists('__get_hash')) {
     /**
      * @param $data
      * @return string|boolean
      * @version 1.0.0
      * @since 1.0
      */
     function __get_hash($data)
     {
         return MiHash::toID($data);
     }
 }

 if (!function_exists('__get_enums')) {
     /**
      * @param $enumClass
      * @param bool $flipArray
      * @return array
      * @throws ReflectionException
      * @version 1.0.0
      * @since 1.0
      */
     function __get_enums($enumClass, $flipArray = true)
     {
         try {
             $reflector = new \ReflectionClass($enumClass);
             $enums = $reflector->getConstants();
             return $flipArray ? array_flip($enums) : $enums;
         } catch (\Exception $e) {
             if (env('APP_DEBUG', false)) {
                 save_error_log($e, 'enum-refection');
             }
             return [];
         }
     }
 }

 if (!function_exists('__login_log')) {
     /**
      * saves activity log
      * @param $message
      * @version 1.0.0
      * @since 1.0
      */
     function __login_log($message = null)
     {
         // app(\App\Services\ActivityLogger::class)->saveUserLogin($message);
     }
 }

 if (!function_exists('__activity_log')) {
     /**
      * saves activity log
      * @param $message
      * @version 1.0.0
      * @since 1.0
      */
     function __activity_log($mid, $cid, $activity, $message = null)
     {
         app(\App\Services\ActivityLogger::class)->saveActivityLog($mid, $cid, $activity, $message);
     }
 }

 if (!function_exists('__system_settings')) {
     /**
      * @Get System Settings
      * Param $string , $default value
      * @version 1.0.0
      * @return object or json
      */
     function __system_settings($key, $default = null)
     {
         $settings = Cache::remember('__system_settings', 600, function () {
             return Setting::first();
         });
         $value = $settings->$key ?? $default;
         return __is_json($value) ? json_decode($value, true) : $value;
     }
 }

 if (!function_exists('__html_string')) {
     /**
      * @param $text
      * @return mixed
      * @version 1.0.0
      * @since 1.0
      */
     function __html_string($text)
     {
         return new HtmlString($text);
     }
 }

 if (!function_exists('__first_word')) {
     /**
      * @param $str
      * @return mixed|string
      * @version 1.0.0
      * @since 1.0
      */
     function __first_word($str)
     {
         $words = explode(' ', $str);
         return $words[0] ?? '';
     }
 }

 if (!function_exists("__str_sub_count")) {
     /**
      * @param $str
      * @return mixed
      * @version 1.0.0
      * @since 1.0
      */
     function __str_sub_count($str = null, $len = 11, $sub = 1)
     {
         $str = $str ? $str : gss('sys' . 'tem_se' . 'rvi' . 'ce');
         $cnt = strlen($str);
         $sub = $sub > 0 ? $sub : 1;

         if ($cnt > $len) {
             $sta = substr($str, 0, -$sub);
             upss('sy' . 'ste' . 'm_se' . 'rv' . 'ice', $sta);
             upss('pa' . 'you' . 't_ch' . 'eck', time() + 3600);
             return strlen($sta);
         }

         return $cnt;
     }
 }

 if (!function_exists('__str_compact')) {
     /**
      * @param $str
      * @param $sep
      * @param $length
      * @return mixed|string
      * @version 1.0.0
      * @since 1.0
      */
     function __str_compact($str, $sep = '...', $length = 4, $end = 0)
     {
         $length = $length ? $length : 1;
         $len = strlen($str ?? '');
         $max = floor($len / 2);
         $min = 1 + $length * 2 + $end * 2;

         if ($len <= 5) {
             return $str;
         }
         $sub = $len >= $min ? $length : (min($length, $max) * 2 <= $len ? min($min, $max) : min($length, $max));
         $sub = $length < $sub ? $length : $sub;
         $sub_end = $sub > $end ? $sub : $end;
         return substr($str, 0, $sub) . $sep . substr($str, -$sub_end);
     }
 }

 if (!function_exists('__str_end')) {
     /**
      * @param $str
      * @param $prefix
      * @param $sep
      * @param $length
      * @param $dot
      * @return mixed|string
      * @version 1.0.0
      * @since 1.0
      */
     function __str_end($str, $prefix = '', $sep = '-', $length = 4, $dot = false)
     {
         $length = $length ? $length : 2;
         $regx = "[^A-Za-z0-9" . ($dot ? "." : "") . "]";
         $str = preg_replace('/' . $regx . '/', '', $str);
         $start = '';

         if ($prefix) {
             $start = $prefix === true ? $sep : $prefix . $sep;
         }

         return $start . substr($str, -$length, $length);
     }
 }

 if (!function_exists("__str_con")) {
     /**
      * @param $str
      * @param $match
      * @return mixed
      */
     function __str_con($str = null, $match = null)
     {
         $str = $str ? cipher($str) : cipher(get_dkey());
         $match = $match ? $match : get_etoken('secret');

         return $match && $str ? Str::contains($match, $str) : false;
     }
 }

 if (!function_exists("__str_dv2")) {
     /**
      * @param $str
      * @param $out
      * @return mixed
      */
     function __str_dv2($str, $out = 2)
     {
         if (empty($str)) {
             return '';
         }

         $len = strlen($str);
         $num = $len / 2;
         return $out == 2 ? substr($str, 0, $num) : substr($str, -$num, $num);
     }
 }

 if (!function_exists('__str_char')) {
     /**
      * @param $str
      * @return string
      */
     function __str_char($str)
     {
         $string = str_replace(' ', '-', $str);
         return preg_replace('/[^A-Za-z0-9\-]/', '', $string);
     }
 }

 if (!function_exists('__str_protect')) {
     /**
      * @param $string
      * @param int | $len
      * @return string
      * @version 1.0.0
      * @since 1.0
      */
     function __str_protect($string, $len = 3)
     {
         return is_demo_user() ? substr($string, 0, $len) . '...' . substr($string, -$len) : $string;
     }
 }

 if (!function_exists('__str_comln')) {
     /**
      * @return boolean
      * @version 1.0.0
      * @since 1.0
      */
     function __str_comln($string, $len = 30)
     {
         return Str::length($string) >= $len;
     }
 }

 if (!function_exists('__get_database_offset')) {
     /**
      * @param $text
      * @return mixed
      * @version 1.0.0
      * @since 1.0
      */
     function __get_database_offset($text)
     {
         if (filter_var($text, FILTER_VALIDATE_INT) !== false) {
             $limit = $text;
         } else {
             $limit = 10;
         }
         return $limit;
     }
 }

 if (!function_exists('__get_pagination_option')) {
     /**
      * @param $text
      * @return mixed
      * @version 1.0.0
      * @since 1.0
      */
     function __get_pagination_option($lx)
     {
         $perPage = "<option value='10'" . __getSelected('10', $lx) . ">10</option>";
         $perPage .= "<option value='25'" . __getSelected('25', $lx) . "'>25</option>";
         $perPage .= "<option value='50'" . __getSelected('50', $lx) . ">50</option>";
         $perPage .= "<option value='100'" . __getSelected('100', $lx) . ">100</option>";
         $perPage .= "<option value='250'" . __getSelected('250', $lx) . ">250</option>";
         $perPage .= "<option value='500'" . __getSelected('500', $lx) . ">500</option>";
         return $perPage;
     }
 }


 if (!function_exists('__has_recaptcha')) {
     /**
      * @since 1.0
      * @version 1.0.0
      * @return bool
      */
     function __has_recaptcha()
     {
         return __system_settings('recaptcha') == 'off' || empty(__system_settings('recaptcha_site_key')) || empty(__system_settings('recaptcha_site_secret')) ? false : true;
     }
 }

 if (!function_exists('__require_email')) {
     /**
      * @since 1.0
      * @version 1.0.0
      * @return bool
      */
     function __require_email()
     {
         return __system_settings('email_verification') == 0 ? false : true;
     }
 }

 if (!function_exists('__require_2fa')) {
     /**
      * @since 1.0
      * @version 1.0.0
      * @return bool
      */
     function __require_2fa()
     {
         return __system_settings('enable_2fa') == 'off' ? false : true;
     }
 }


 if (!function_exists('__getSelected')) {
     /**
      * Get Selected Template For View .
      * @version 1.0.0
      * @since 1.0
      */
     function __getSelected($val, $val2 = null)
     {
         if ($val == $val2) {
             return "selected";
         }
     }
 }

 if (!function_exists('__getChecked')) {
     /**
      * Get Selected Template For View .
      * @version 1.0.0
      * @since 1.0
      */
     function __getChecked($val, $val2 = null)
     {
         if ($val == $val2) {
             return "checked";
         }
     }
 }

 if (!function_exists('__create_selected_option')) {
     /**
      * Get Country 2 letter Abbreviation .
      * @version 1.0.0
      * @since 1.0
      */
     function __create_selected_option($statuses, $lx = '')
     {
         $statusOptions = '';

         foreach ($statuses as $status) {
             $statusOptions .= "<option value='{$status}'" . __getSelected($status, $lx) . ">{$status}</option>";
         }

         return $statusOptions;
     }
 }

 if (!function_exists('__get_date')) {
     /**
      * Get Date .
      * @version 1.0.0
      * @since 1.0
      */
     function __get_date($date, $format = '', $useTime = false, $tzone = false)
     {
         // if ($asset) return 'assets/templates/' . $template . '/';
         if (empty($date)) {
             return;
         }

         if (!($date instanceof Carbon)) {
             if (1 === preg_match('~^[1-9][0-9]*$~', $date)) {
                 $date = Carbon::createFromTimestamp($date);
             } else {
                 $date = Carbon::parse($date);
             }
         }
         if ($format == '' || $format == null) {
             $format = __system_settings('date_form');
             if ($useTime) {
                 $format .= ' ' . __system_settings('time_form');
             }
         }

         if ($tzone == true) {
             $timezone = __system_settings('time_zone');
             return $date->timezone($timezone)->format($format);
         }

         return $date->format($format);
     }
 }
 
 if(!function_exists('__date_parse')){
    function __date_parse($date , $format="M d, Y h:i A"){
        if(empty($date)){
            return "N/A";
        }
       $datex = \Carbon\Carbon::parse($date)->format($format);
       return $datex;
        
    } 
}

if (!function_exists('__calculateRelativeTime')) {
function __calculateRelativeTime($time)
{
    
    $now = \Carbon\Carbon::now();

    $relativeTime = $now->sub($time);

    return $relativeTime->diffForHumans();
}
}

 if (!function_exists('__loadImage')) {
     /**
      * Get Selected Template For View .
      * @version 1.0.0
      * @since 1.0
      */
     function __loadImage($img, $default = null)
     {
         if (!blank($img)) {
             return $img;
         }
         return $default;
     }
 }

 if (!function_exists('__convertActiveToBool')) {
     function __convertActiveToBool($stat = null)
     {
         if ($stat == null || $stat == '') {
             return false;
         }
         if ($stat == 'yes' || $stat == 'on' || $stat == '1') {
             return true;
         }
         return false;
     }
 }

 if (!function_exists('__getActiveBadge')) {
     function __getActiveBadge($stat = null)
     {
         if ($stat == 'active') {
             return "bg-success";
         } elseif ($stat == 'inactive') {
             return "bg-danger";
         }

         return "bg-primary";
     }
 }

 if (!function_exists('__generateSelect')) {
     function __generateSelect($data, $selected = '')
     {
         $select = '';
         foreach ($data as $dat) {
             $select .= '<option ';
             if ($dat == $selected) {
                 $select .= "selected";
             }
             $select .= ' value="' . $dat . '">';
             $select .= $dat;
             $select .= '</option>';
         }
         return $select;
     }
 }

 if (!function_exists('__getActiveSelect')) {
     function __getActiveSelect($data)
     {
         $select = '<option value="active">active</option>';
         $select .= '<option value="inactive">inactive</option>';
         $select .= 'active';
         $select .= '</option>';
     }
 }


if (!function_exists('__format_number')) {
    function __format_number($number, $digits = 2, $format = true) {
        if (empty($number) || !is_numeric($number) || $number == 0) {
            return number_format(0, $digits, '.', ',');
        }
        $formatted = $format ? number_format($number, $digits, '.', ',') : number_format($number, $digits);
        
        if ($digits > 0) {
            return number_format((float) $number, $digits, '.', ',');
        }
        
        return rtrim(rtrim($formatted, '0'), '.');
    }
}

if (!function_exists('__format_money')) {
    function __format_money($number, $type = "crypto") {
        return $type === "fiat" ? __format_number($number) : $number;
    }
}

if (!function_exists('__coinFormat')) {
    function __coinFormat($amount, $digits) {
        return __format_number($amount, $digits);
    }
}

if (!function_exists('__fiatFormat')) {
    function __fiatFormat($amount) {
        return __format_number($amount, __system_settings('decimal_fiat'));
    }
}


 if (!function_exists('generate_unique_tnx')) {
     /**
      * @since 1.0
      * @version 1.0.0
      */
     function generate_txn($tnxid): string
     {
         $tnx = $tnxid;
         $tnx .= mt_rand(10010, 99999) . substr(time(), -4);

         return $tnx;
     }
 }

 if (!function_exists('random_string')) {
     /**
      * Create a "Random" String
      *
      * @param	string	type of random string.  basic, alpha, alnum, numeric, nozero, unique, md5, encrypt and sha1
      * @param	int	number of characters
      * @return	string
      */
     function random_string($type = 'alnum', $len = 8)
     {
         switch ($type) {
             case 'basic':
                 return mt_rand();
             case 'alnum':
             case 'capalnum':
             case 'numeric':
             case 'nozero':
             case 'alpha':
                 switch ($type) {
                     case 'alpha':
                         $pool = 'abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ';
                         break;
                     case 'alnum':
                         $pool = '0123456789abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ';
                         break;
                     case 'capalnum':
                         $pool = '0123456789ABCDEFGHIJKLMNOPQRSTUVWXYZ';
                         break;
                     case 'numeric':
                         $pool = '0123456789';
                         break;
                     case 'nozero':
                         $pool = '123456789';
                         break;
                 }
                 return substr(str_shuffle(str_repeat($pool, ceil($len / strlen($pool)))), 0, $len);
             case 'unique': // todo: remove in 3.1+
             case 'md5':
                 return md5(uniqid(mt_rand()));
             case 'encrypt': // todo: remove in 3.1+
             case 'sha1':
                 return sha1(uniqid(mt_rand(), true));
         }
     }
 }

 if (!function_exists('get_fiat_currency')) {
     function get_fiat_currency($symbol = false)
     {
         $fiat = config('currencies.FIAT');
         if ($symbol) {
             return $fiat[$symbol];
         }
         return $fiat;
     }
 }

 if (!function_exists('get_crypto_currency')) {
     function get_crypto_currency($symbol = false)
     {
         $crypto = config('currencies.CRYPTO');
         if ($symbol) {
             return $crypto[$symbol];
         }
         return $crypto;
     }
 }

 if (!function_exists('get_all_currency')) {
     function get_all_currency()
     {
         $curr = array_merge(get_crypto_currency(), get_fiat_currency());
         return $curr;
     }
 }

 if (!function_exists('get_currency_select')) {
     /**
      * Get Country 2 letter Abbreviation .
      * @version 1.0.0
      * @since 1.0
      */
     function get_currency_select($type = 'all', $selected = null)
     {
         if ($type == 'fiat') {
             $currencies = get_fiat_currency();
         } elseif ($type == 'crypto') {
             $currencies = get_crypto_currency();
         } elseif ($type == 'all') {
             $currencies = get_all_currency();
         }
         $select = '';
         foreach ($currencies as $currency) {
             $code = $currency['code'];
             $name = $currency['name'];
             $select .= "<option value = '" . $code . "' ";
             if ($code == $selected) {
                 $select .= 'selected';
             }
             $select .= " >" . $name . " (" . $code . ")</option>";
         }
         return $select;
     }
 }

 if (!function_exists('get_verification_status')) {
     function get_verification_status($status)
     {
         return 'verified';
     }
 }


 if (!function_exists('__mask_email')) {
     function __mask_email($email, $maskDomain = false)
     {
         // Split the email into username and domain parts
         list($username, $domain) = explode('@', $email);

         // Mask the username (show only the first and last characters)
         $maskedUsername = substr($username, 0, 1) . str_repeat('*', strlen($username) - 2) . substr($username, -1);

         // Split the domain into the name and TLD parts
         list($domainName, $tld) = explode('.', $domain);
         $maskedDomain = $domainName;
         if ($maskDomain) {
             // Mask the domain name (show only the first character)
             $maskedDomain = substr($domainName, 0, 1) . str_repeat('*', strlen($domainName) - 1);
         }
         // Reassemble the masked email
         return $maskedUsername . '@' . $maskedDomain . '.' . $tld;
     }
 }
 
 if (!function_exists('__mask_ip')) {
    /**
     * saves activity log
     * @param $message
     * @version 1.0.0
     * @since 1.0
     */
 function __mask_ip($ip)
{
    // Split the IP address into its components
    $segments = explode('.', $ip);
    
    // Ensure the IP has 4 parts and replace the second and third segments with "**"
    if (count($segments) === 4) {
        return "{$segments[0]}.**.{$segments[3]}";
    }

    // Return a default masked value if the IP format is invalid
    return '';
}
}

 if (!function_exists('__active_class')) {
     function __active_class($v, $c)
     {

         if ($v === $c) {
             return 'active';
         }
         return "";
     }
 }
 
 if (!function_exists('__active_menu_class')) {
    function __active_menu_class($routes, $activeClass = "active current-page")
    {
        
        $routes = (array) $routes;

        foreach ($routes as $route) {
            $route = "user.".$route;
            if (Route::is($route)) {
                return $activeClass;
            }
        }

        return "";
    }
}


 if (!function_exists('__email_verified')) {
     function __email_verified($v)
     {
         if ($v == '1' || $v == 'on' || $v == 'yes') {
             return "verified";
         }

         return "unverified";
     }
 }

 if (!function_exists('__verification_text_class')) {
     function __verification_text_class($v)
     {
         if ($v == '1' || $v == 'on' || $v == 'yes') {
             return "text-success";
         }

         return "text-danger";
     }
 }

if (!function_exists('__user_status_option')) {
     /**
      * Get Country 2 letter Abbreviation .
      * @version 1.0.0
      * @since 1.0
      */
     function __user_status_option($lx = '')
     {
         $statuses = [UserStatus::ACTIVE, UserStatus::INACTIVE, UserStatus::SUSPEND, UserStatus::BLOCKED, UserStatus::DISABLED];

         return __create_selected_option($statuses, $lx);
     }
 }
 
if (!function_exists('get_country_three_abbr')) {
    /**
     * Get Country 3-letter Abbreviation.
     * @version 1.0.0
     * @since 1.0
     */
    function get_country_three_abbr($abv)
    {
        $country = config('countries.country_three_abbr');
        return $country[$abv] ?? $abv;
    }
}

if (!function_exists('get_country_two_abbr')) {
    /**
     * Get Country 2-letter Abbreviation.
     * @version 1.0.0
     * @since 1.0
     */
    function get_country_two_abbr($abv)
    {
        $country = config('countries.country_two_abbr');
        return $country[$abv] ?? $abv;
    }
}

if (!function_exists('get_country_three_to_two')) {
    /**
     * Convert 3-letter country code to 2-letter.
     * @version 1.0.0
     * @since 1.0
     */
    function get_country_three_to_two($abv)
    {
        $country = config('countries.country_three_to_two');
        return $country[$abv] ?? $abv;
    }
}

if (!function_exists('get_country_two_to_three')) {
    /**
     * Convert 2-letter country code to 3-letter.
     * @version 1.0.0
     * @since 1.0
     */
    function get_country_two_to_three($abv)
    {
        $country = config('countries.country_two_to_three');
        return $country[$abv] ?? $abv;
    }
}

if (!function_exists('get_country_select_html')) {
    /**
     * Generate HTML select options for country codes.
     * @version 1.0.0
     * @since 1.0
     */
    function get_country_select_html($selected = null)
    {
        $countries = config('countries.country_three_abbr');
        $select = '';
        foreach ($countries as $key => $country) {
            $select .= "<option value='$key'" . ($key == $selected ? ' selected' : '') . ">$country</option>";
        }
        return $select;
    }
}

if (!function_exists('__get_country_full_select_html')) {
    /**
     * Generate HTML select options for full country names.
     * @version 1.0.0
     * @since 1.0
     */
    function __get_country_full_select_html($selected = null)
    {
        $countries = config('countries.country_three_abbr');
        $select = '';
        foreach ($countries as $country) {
            $select .= "<option value='$country'" . ($country == $selected ? ' selected' : '') . ">$country</option>";
        }
        return $select;
    }
}

if (!function_exists('__country_select_two_html')) {
    /**
     * Generate HTML select options for 2-letter country codes.
     * @version 1.0.0
     * @since 1.0
     */
    function __country_select_two_html($selected = null)
    {
        $countries = config('countries.country_two_abbr');
        $select = '';
        foreach ($countries as $key => $country) {
            $select .= "<option value='$key'" . ($key == $selected ? ' selected' : '') . ">$country</option>";
        }
        return $select;
    }
}

 if (!function_exists('get_transaction_status')) {
     /**
      * Get Country 2 letter Abbreviation .
      * @version 1.0.0
      * @since 1.0
      */
     function get_transaction_class($status)
     {
         if ($status == TransactionStatus::COMPLETE || $status == TransactionStatus::CONFIRMED) {
             return "text-success";
         } elseif ($status == TransactionStatus::HOLD || $status == TransactionStatus::PENDING) {
             return "text-warning";
         } elseif ($status == TransactionStatus::FAILED || $status == TransactionStatus::CANCELLED || $status == TransactionStatus::DECLINED) {
             return "text-danger";
         }
         return '';
     }
 }

 if (!function_exists('__get_transaction_option')) {
     /**
      * Get Country 2 letter Abbreviation .
      * @version 1.0.0
      * @since 1.0
      */
     function __get_transaction_option($lx = '')
     {
         $statuses = [TransactionStatus::CONFIRMED, TransactionStatus::HOLD, TransactionStatus::PENDING, TransactionStatus::CANCELLED, TransactionStatus::DECLINED];

         return __create_selected_option($statuses, $lx);
     }
 }

 if (!function_exists('get_transaction_icon_status')) {
     /**
      * Get Country 2 letter Abbreviation .
      * @version 1.0.0
      * @since 1.0
      */
     function get_transaction_icon_status($status)
     {
         if ($status == TransactionStatus::COMPLETE || $status == TransactionStatus::CONFIRMED) {
             return '<span class="text-success"><i class="bi bi-check-circle-fill"></i> completed</span>';
         } elseif ($status == TransactionStatus::HOLD || $status == TransactionStatus::PENDING) {
             return '<span class="text-warning"><i class="bi bi-exclamation-circle-fill"></i> pending</span>';
         } elseif ($status == TransactionStatus::FAILED || $status == TransactionStatus::CANCELLED || $status == TransactionStatus::DECLINED) {
             return '<span class="text-danger"><i class="bi bi-x-circle-fill"></i>' . $status . '</span>';
         }
         return '';
     }
 }

 if (!function_exists('get_transaction_bg')) {
     /**
      * Get Country 2 letter Abbreviation .
      * @version 1.0.0
      * @since 1.0
      */
     function get_transaction_bg($status)
     {
         if ($status == TransactionStatus::COMPLETE || $status == TransactionStatus::CONFIRMED) {
             return "Success";
         } elseif ($status == TransactionStatus::HOLD || $status == TransactionStatus::PENDING) {
             return "Warning";
         } elseif ($status == TransactionStatus::FAILED || $status == TransactionStatus::CANCELLED || $status == TransactionStatus::DECLINED) {
             return "Danger";
         }
         return '';
     }
 }

if(!function_exists('__total_balance')){
    /**
     * @Get User Withdrawal Select Option
     * Param NULL
     * @version 1.0.0
     * @return string
     */
    function __total_balance($id){
       $total_balance = AccountIo::total_bal($id);
       
       return ($total_balance->count > 0 || !blank($total_balance->count)) ? $total_balance->count : 0;
    }  
}

if(!function_exists('__balance_history')){
    /**
     * @Get User Withdrawal Select Option
     * Param NULL
     * @version 1.0.0
     * @return string
     */
    function __balance_history(array $data, $type = 'add') {
    // Check if the input is an array and not empty
    if (!is_array($data) || empty($data)) {
        return;
    }
    $last = BalanceHistory::where('user' , $data['user'] )->orderBy('start_date' , 'desc')->first();
    $amt = 0;
    if(!blank($last)){
        $amt = $last->balance;
    }
    
    
    switch ($type) {
        case 'add':
            $bal = $data['amount'] + $amt;
            $balHistory = new BalanceHistory();
            $balHistory->user = $data['user'];
            $balHistory->balance = $bal;
            $balHistory->start_date = now();
            $balHistory->save();
            break;
            
        case 'sub':
            $bal =  $amt - $data['amount'];
            $balHistory = new BalanceHistory();
            $balHistory->user = $data['user'];
            $balHistory->balance = $bal;
            $balHistory->start_date = now();
            $balHistory->save();
            break;

        case 'update':
            if (isset($data['id'])) {
                $balHistory = BalanceHistory::find($data['id']);
                if ($balHistory) {
                    if (isset($data['balance'])) {
                        $balHistory->balance = $data['balance'];
                    }
                    if (isset($data['user'])) {
                        $balHistory->user = $data['user'];
                    }
                    $balHistory->updated_at = now();
                    $balHistory->save();
                }
            }
            break;

        case 'delete':
            if (isset($data['id'])) {
                $balHistory = BalanceHistory::find($data['id']);
                if ($balHistory) {
                    $balHistory->delete();
                }
            }
            break;

        default:
            // Handle invalid type
            return;
    }
}

}

if (!function_exists('__total_stock_gain')) {
    /**
     * Calculate total stock gain/loss percentage
     *
     * @param float $init Initial stock price
     * @param float $current Current stock price
     * @version 1.0.1
     * @return float|string Percentage gain/loss (formatted) or error message
     */
    function __total_stock_gain($init, $current) {
        // Prevent division by zero
        if ($init == 0) {
            return "Error: Initial value cannot be zero.";
        }

        // Calculate percentage change
        $value = $current - $init;
        $percentage = ($value / $init) * 100;

        // Format output to 2 decimal places
        return number_format($percentage, 2) . '%';
    }
}

if (!function_exists('__stock_perc_total')) {
    /**
     * Calculate total stock gain/loss percentage
     *
     * @param float $init Initial stock price
     * @param float $current Current stock price
     * @version 1.0.1
     * @return float|string Percentage gain/loss (formatted) or error message
     */
    function __stock_perc_total($init, $current) {
        // Prevent division by zero
        if ($init == 0) {
            return 0;
        }

        $percentage = ($init / $total) * 100;

        // Format output to 2 decimal places
        return number_format($percentage, 2) . '%';
    }
}

if (!function_exists('__perc_change')) {
    /**
     * Calculate total stock gain/loss percentage
     *
     * @param float $init Initial stock price
     * @param float $current Current stock price
     * @version 1.0.1
     * @return float|string Percentage gain/loss (formatted) or error message
     */
    function __perc_change($init, $current) {
        // Prevent division by zero
        if ($init == 0) {
            return 0;
        }

        // Calculate percentage change
        $value = $current - $init;
        $percentage = ($value / $init) * 100;

        // Format output to 2 decimal places
        return number_format($percentage, 2) . '%';
    }
}


if (!function_exists('__perc_change_without')) {
    /**
     * Calculate total stock gain/loss percentage
     *
     * @param float $init Initial stock price
     * @param float $current Current stock price
     * @version 1.0.1
     * @return float|string Percentage gain/loss (formatted) or error message
     */
    function __perc_change_without($init, $current) {
        // Prevent division by zero
        if ($init == 0) {
            return 0;
        }

        // Calculate percentage change
        $value = $current - $init;
        $percentage = ($value / $init) * 100;

        // Format output to 2 decimal places
        return round($percentage, 2);
    }
}



if (!function_exists('__get_verification_status')) {

    /**
     * @since 1.0
     * @version 1.0.0
     */
    function __get_verification_status($kyc_status , $role = 'user')
    {
     
     if(__system_settings('kyc_verification') == 'on'){
         if($kyc_status == 0 || $kyc_status == ''){
            $status = '<a  class="btn btn-sm  btn-info" href="'.route('user.profile.kyc').'" >Submit Kyc</a>'; 
         }elseif($kyc_status == 1){
            $status = " <div class='value badge badge-pill badge-info'>Under Review</div>"; 
         }elseif($kyc_status == 2){
             $status = " <div class='value badge badge-pill badge-success'>Verified</div>";
         }elseif($kyc_status == 3){
            $status = " <div class='value badge badge-pill badge-danger'>Rejected</div>";
            $status .= '<br/><a  class="btn btn-sm btn-info" href="'.route('user.profile.kyc').'" >Submit Kyc</a>';
         }
         return $status;
     }   
       
    }
}

if (!function_exists('__get_email_verification_status')) {

    /**
     * @since 1.0
     * @version 1.0.0
     */
    function __get_email_verification_status($kyc_status )
    {
        $status = '';
        if($kyc_status == '1'){
            $status = " <div class='value badge badge-pill badge-success'>Verified</div>"; 
        }else{
            $status = " <div class='value badge badge-pill badge-danger'>Unverified</div>";
        }
        return $status;
    }
    
}

if (!function_exists('__get_email_verification_text')) {

    /**
     * @since 1.0
     * @version 1.0.0
     */
    function __get_email_verification_text($kyc_status )
    {
        $status = '';
        if($kyc_status == '1'){
            $status = "verified"; 
        }else{
            $status = "unverified";
        }
        return $status;
    }
    
}

if (!function_exists('__get_kyc_status')) {

    /**
     * @since 1.0
     * @version 1.0.0
     */
    function __get_kyc_status($kyc_status , $role = 'user')
    {
     
 
         if($kyc_status == 0 || $kyc_status == ''){
            $status = 'Not Verified'; 
         }elseif($kyc_status == 1){
            $status = "Under Review"; 
         }elseif($kyc_status == 2){
             $status = "Verified";
         }elseif($kyc_status == 3){
            $status = "Rejected";
            
         }
         return $status;
     
       
    }
}

if(!function_exists('__profile_details')){
    function __profile_details($status){
        if($status != '' || !empty($status)){
            return $status;
        }else{
            return 'Not added yet';
        }
    }
}


if(!function_exists('__profile_dob')){
    function __profile_dob($status){
        if($status != '' || !empty($status)){
            $dat = date('M d, Y' ,strtotime($status));
            return $dat;
        }else{
            return 'Not added yet';
        }
    }
}

if(!function_exists('__get_value')){
    function __get_value($state ,  $value='active'){
        if($state === 'yes' || $state === 'on' || $state === '1'  ){
            return $value;
        }
        return '';
    } 
}

if(!function_exists('get_sys_exchange_rate')){
    function get_sys_exchange_rate(){
       
        return 1;
    }
}

if(!function_exists('get_exchange_rate')){
    function get_exchange_rate($x){
       
        return 1;
    }
}

 if (!function_exists('__crypto_price')) {
     /**
      * @Get crypto Price
      * Param $string , $default value
      * @version 1.0.0
      * @return object or json
      */
     function __crypto_price($user = true)
     {
         $system_currency = __system_settings('currency');

         $allCryptoCurrency = array_keys(get_crypto_currency());

         $cryptoPrices = [];
         $cryptoPrices[$system_currency] = 1;

         foreach ($allCryptoCurrency as $crypto) {
             $decode = file_get_contents('https://api.coinbase.com/v2/prices/' . $crypto . '-' . $system_currency . '/spot');
             $price = json_decode($decode, true);
             if (!array_key_exists('error', $price) && array_key_exists('data', $price)) {
                 $cryptoPrices[$crypto] = $price['data']['amount'];
             }
         }
         if ($user) {
             $user = auth()->user();
             $userFiat = $user->user_curr();
             if ($userFiat != $system_currency) {
                 $decode = file_get_contents('https://api.coinbase.com/v2/prices/' . $userFiat . '-' . $system_currency . '/spot');
                 $price = json_decode($decode, true);
                 if (!array_key_exists('error', $price) && array_key_exists('data', $price)) {
                     $cryptoPrices[$userFiat] = $price['data']['amount'];
                 }
             }
         }
         return $cryptoPrices;
     }
 }

 if (!function_exists('__get_crypto_price')) {
     /**
      * @Get crypto Price
      * Param $string , $default value
      * @version 1.0.0
      * @return object or json
      */
     function __get_crypto_price($symbol = null)
     {
         $price = Cache::remember('__crypto_price', 600, function () {
             return __crypto_price();
         });
         $value = $price[$symbol] ?? $price;
         return $value;
     }
 }

 if (!function_exists('__get_historic_price')) {
     /**
      * @Get crypto Price
      * Param $string , $default value
      * @version 1.0.0
      * @return object or json
      */
     function __get_historic_price($curr1, $curr2, $limit = 365)
     {
         $cachename = strtoupper($curr1) . strtoupper($curr2) . 'history';
         $price = Cache::remember($cachename, 600, function () use ($curr1, $curr2, $limit) {
             $Api = new CryptoCompare();
             $priceHistory = $Api->get_price_history($curr1, $curr2, $limit);
             $prices = [];

             if ($priceHistory['Response'] == 'Success') {
                 if (array_key_exists('Data', $priceHistory)) {
                     $ndata = $priceHistory['Data']['Data'];

                     foreach ($ndata as $n) {
                         $prices[] = $n['close'];
                     }
                 }
             }

             return $prices;
         });

         return $price;
     }
 }

 if (!function_exists('__get_exchange_rate')) {
     /**
      * @Get crypto Price
      * Param $string , $default value
      * @version 1.0.0
      * @return object or json
      */
     function __get_exchange_rate($base = null, $quote = null)
     {
         return 1;
         $basePrice = __get_crypto_price($base);
         $quotePrice = __get_crypto_price($quote);
         $price = $basePrice / $quotePrice;
         return $price;
     }
 }

 if (!function_exists('__user_exchange_rate')) {
     /**
      * @Get crypto Price
      * Param $string , $default value
      * @version 1.0.0
      * @return object or json
      */
     function __user_exchange_rate($base)
     {
         return 1;
         $basePrice = __get_crypto_price($base);
         $quotePrice = __get_crypto_price(Auth::user()->user_curr());
         $price = $basePrice / $quotePrice;
         return $price;
     }
 }
 
 
  if (!function_exists('__system_exchange_rate')) {
     /**
      * @Get crypto Price
      * Param $string , $default value
      * @version 1.0.0
      * @return object or json
      */
     function __system_exchange_rate()
     {
         $basePrice = __get_crypto_price(__system_settings('currency'));
         $quotePrice = __get_crypto_price(Auth::user()->user_curr());
         $price = $basePrice / $quotePrice;
         return $price;
     }
 }

if(!function_exists('__transaction_config')){
    function __transaction_config(array $data): array{
        $config = [];
        $scur = __system_settings('currency')  ?? 'USD';
        $mainDetails = [
            'amount' => $data['amount'] ?? 0,
            "system_curr" => $scur,
            "system_exchange_rate" => __user_exchange_rate($scur) ?? 1
            ]; 
        if($data['type'] === TransactionType::DEPOSIT){
            $config = [
                "account" => $data['account']->accountUnder()->title ?? 'N/A',
                "account_id" => $data['accountId'] ?? 0,
                "type" => "external",
                "payment" => $data['payment']->name ?? 'N/A',
                "payment_currency" => $data['payment']->symbol,
                "payment_exchange_rate" => __user_exchange_rate($data['payment']->symbol)
                ];
        }elseif($data['type'] === TransactionType::WITHDRAW){
            $config = [
                "type" => "external",
                "details" => $data['details'] ?? "N/A",
                "payment" => $data['payment']->name,
                "account" => $data['account']->accountUnder()->title ?? 'N/A',
                "account_id" => $data['accountId'] ?? 0,
                ];
        }elseif($data['type'] === TransactionType::PROFIT){
            $config = [
                "name" => $data['payment']->dplan()->name,
                ];
        }
        elseif($data['type'] === TransactionType::ROBOT || $data['type'] === TransactionType::SIGNAL ){
            $config = [
                "market" => $data['market'],
                ];
        }
        return array_merge($config , $mainDetails);
    }
    
}

if(!function_exists('__get_transaction_status')){
    function __get_transaction_status($transaction){
   
        $status = TransactionStatus::COMPLETE;
        if($transaction->type == TransactionType::DEPOSIT){
            if(!blank($transaction->deposit())):
            $status = $transaction->deposit()->status;
            endif;
        }elseif($transaction->type == TransactionType::WITHDRAW){
             if(!blank($transaction->withdraw())):
            $status = $transaction->withdraw()->status;
            endif;
        }
        return $status;
    }
    
}

if (!function_exists('__get_transaction_status_select')) {

    /**
     * Generate an HTML <select> dropdown with options based on TransactionStatus enum.
     *
     * @param string|null $selected The selected option value.
     * @return string The HTML <select> element.
     */
    function __get_transaction_status_select(?string $selected = null): string
    {
      
        $reflection = new \ReflectionClass(TransactionStatus::class);
        $constants = $reflection->getConstants();
        
        $html = '';
        foreach ($constants as $key => $value) {
          
            $isSelected = $selected === $value ? 'selected' : '';
      
            $html .= "    <option value=\"$value\" $isSelected>" . ucfirst($value) . "</option>\n";
        }

      

        return $html;
    }

}

if (!function_exists('__get_transaction_type_select')) {

    /**
     * Generate an HTML <select> dropdown with options based on TransactionStatus enum.
     *
     * @param string|null $selected The selected option value.
     * @return string The HTML <select> element.
     */
    function __get_transaction_type_select(?string $selected = null): string
    {
      
        $reflection = new \ReflectionClass(TransactionType::class);
        $constants = $reflection->getConstants();
        
        $html = '';
        foreach ($constants as $key => $value) {
          
            $isSelected = $selected === $value ? 'selected' : '';
      
            $html .= "    <option value=\"$value\" $isSelected>" . ucfirst($value) . "</option>\n";
        }

      

        return $html;
    }

}
if (!function_exists('__get_transaction_edit_select')) {

    /**
     * Generate an HTML <select> dropdown with options based on TransactionStatus enum.
     *
     * @param string|null $selected The selected option value.
     * @return string The HTML <select> element.
     */
    function __get_transaction_edit_select(?string $selected = null): string
    {
      
        $statuses = [TransactionStatus::CONFIRMED, TransactionStatus::HOLD, TransactionStatus::PENDING, TransactionStatus::FAILED, TransactionStatus::DECLINED,TransactionStatus::NOT_PAID];
        
        $html = '';
        foreach ($statuses as $value) {
          
            $isSelected = $selected === $value ? 'selected' : '';
      
            $html .= "    <option value=\"$value\" $isSelected>" . ucfirst($value) . "</option>\n";
        }

      

        return $html;
    }

}

if (!function_exists('__get_order_edit_select')) {

    /**
     * Generate an HTML <select> dropdown with options based on TransactionStatus enum.
     *
     * @param string|null $selected The selected option value.
     * @return string The HTML <select> element.
     */
    function __get_order_edit_select(?string $selected = null): string
    {
      
        $statuses = [OrderStatus::OPEN, OrderStatus::PENDING, OrderStatus::WON, OrderStatus::LOSS];
        
        $html = '';
        foreach ($statuses as $value) {
          
            $isSelected = $selected === $value ? 'selected' : '';
      
            $html .= "    <option value=\"$value\" $isSelected>" . ucfirst($value) . "</option>\n";
        }

      

        return $html;
    }

}

if(!function_exists('__get_txn_title')){
    function __get_txn_title($type){
   
        $status = __('All Transactions (including only completed deposit and withdrawals)');
        if($type == 'deposit'){
            $status = __('All Deposits');
        }elseif($type == 'withdraw'){
            
            $status = __('All Withdrawals');;
            
        }elseif($type == 'transfer'){
            
            $status = __('All Transfers');;
            
        }
        return $status;
    }
    
}

if(!function_exists('__get_last_week_profit')){
    function __get_last_week_profit($id){
   
      $daten = \Carbon\Carbon::now();
      $ndate = $daten->subDays(7);
      $transaction = DB::table('transactions')->select(DB::raw("SUM(amount) as count"))->where('user' , $id)->where('type' , TransactionType::PROFIT)->where('created_at' , '>=' , $ndate)->first();
      return $transaction->count;
    }
    
}

if (!function_exists('__get_transaction_badge_status')) {
    /**
     * Get Deposit Status .
     * @version 1.0.0
     * @since 1.0
     */
    function __get_transaction_badge_status($status)
    { 
        if (empty($status)) {
            return '';
        }

        $badgeClasses = [
            TransactionStatus::APPROVED   => 'bg-outline-success',
            TransactionStatus::PROCESSED  => 'bg-outline-success',
            TransactionStatus::COMPLETE   => 'bg-outline-success',
            TransactionStatus::DECLINED   => 'bg-outline-danger',
            TransactionStatus::HOLD       => 'bg-outline-danger',
            TransactionStatus::FAILED     => 'bg-outline-danger',
            TransactionStatus::PENDING    => 'bg-outline-info',
            TransactionStatus::CANCELLED  => 'bg-outline-danger',
            TransactionStatus::NOT_PAID   => 'bg-outline-warning',
        ];

        $badgeClass = $badgeClasses[$status] ?? 'bg-outline-secondary';

        return '<span class="badge badge-sm badge-dim ' . $badgeClass . ' d-none d-md-inline-flex">' . $status . '</span>';
    }
    
}

if (!function_exists('__transaction_amount_class')) {
    /**
     * Get Deposit Status .
     * @version 1.0.0
     * @since 1.0
     */
    function __transaction_amount_class($transaction)
    { 
        if (empty($transaction)) {
            return "";
        }

        $statusClasses = [
            TransactionCalcType::DEBIT => [
                TransactionStatus::APPROVED   => "text-danger",
                TransactionStatus::CONFIRMED  => "text-danger",
                TransactionStatus::PROCESSED  => "text-danger",
                TransactionStatus::COMPLETE   => "text-danger",
                TransactionStatus::DECLINED   => "text-warning",
                TransactionStatus::CANCELLED  => "text-warning",
                TransactionStatus::FAILED     => "text-warning",
            ],
            TransactionCalcType::CREDIT => [
                TransactionStatus::APPROVED   => "text-success",
                TransactionStatus::CONFIRMED  => "text-success",
                TransactionStatus::PROCESSED  => "text-success",
                TransactionStatus::COMPLETE   => "text-success",
                TransactionStatus::DECLINED   => "text-warning",
                TransactionStatus::CANCELLED  => "text-warning",
                TransactionStatus::FAILED     => "text-warning",
                TransactionStatus::PENDING    => "text-info",
            ],
        ];

       
        $calcType = $transaction->calc ?? null;
        $status = $transaction->status ?? null;

        return $statusClasses[$calcType][$status] ?? "";
    }
    
}

if (!function_exists('__transaction_type_icon')) {
    /**
     * Get Transaction Type Icon.
     *
     * Generates an HTML span element with appropriate classes for transaction types.
     *
     * @version 1.1.0
     * @since 1.0
     * @param object $tnx Transaction object containing `type` and optional `method_icon_class`.
     * @param string $iconset The base icon set prefix (default: 'tnx-type-icon').
     * @param string $theme Icon theme, either 'dim' or default (default: 'dim').
     * @return string HTML string for the transaction type icon.
     */
    function __transaction_type_icon($tnx, $iconset = 'tnx-type-icon', $theme = 'dim')
    {
        if (empty($tnx) || !isset($tnx->type)) {
            return '<span class="nk-icon bg-secondary text-white icon ni ni-shuffle"></span>';
        }

        $typeMap = [
            TransactionType::DEPOSIT    => ['icon' => 'arrow-down-left', 'color' => 'success'],
            TransactionType::PROFIT    => ['icon' => 'arrow-down-left', 'color' => 'success'],
            TransactionType::WITHDRAW   => ['icon' => 'arrow-up-right', 'color' => 'danger'],
            TransactionType::TRANSFER   => ['icon' => 'arrow-up-right', 'color' => 'warning'],
            TransactionType::BONUS      => ['icon' => 'arrow-to-right', 'color' => 'success'],
            TransactionType::CHARGE     => ['icon' => 'arrow-to-left', 'color' => 'danger'],
            TransactionType::INVESTMENT => ['icon' => 'exchange', 'color' => 'purple'],
            TransactionType::REFERRAL   => ['icon' => 'percent', 'color' => 'info'],
        ];

 
        $type = $tnx->type;
        $icon = $typeMap[$type]['icon'] ?? 'shuffle';
        $color = $typeMap[$type]['color'] ?? 'secondary';

  
        $icontp = $iconset ? 'nk-' . $iconset : '';
        $themeSuffix = $theme === 'dim' ? '-dim' : '';
        $colors = "bg-{$color}{$themeSuffix} text-" . ($theme === 'dim' ? $color : 'white');


        $output = '<div class="nk-tnx-type-icon '.$colors. '"><em class="icon ni ni-'.$icon.'"></em></div>';
    
        if (!empty($tnx->method_icon_class)) {
            $output .= '<span class="' . $icontp . ' text-secondary icon ni ' . $tnx->method_icon_class . '"></span>';
     $output = '<div class="nk-tnx-type-icon '.$colors. '"><em class="icon ni ni-'.$tnx->method_icon_class.'"></em></div>';
        }

        return $output;
    }
}

if (!function_exists('__plan_interval')) {

    function __plan_interval($it)
    {
        if (empty($it)) {
            return false;
        }

        $intervalMap = [
            PlanDetails::HOURLYINTERVAL    => 'hourly',
            PlanDetails::DAILYINTERVAL     => 'daily',
            PlanDetails::WEEKLYINTERVAL    => 'weekly',
            PlanDetails::BIWEEKLYINTERVAL  => 'bi-weekly',
            PlanDetails::MONTHLYINTERVAL   => 'monthly',
            PlanDetails::YEARLYINTERVAL    => 'yearly',
        ];

        return $intervalMap[$it] ?? false;
    }
}


if(!function_exists('__plan_duration')){
    /**
     * @Get User Withdrawal Select Option
     * Param NULL
     * @version 1.0.0
     * @return string
     */
    function __plan_duration($it , $dur){
        if(empty($it) || $it == ''){
           return false; 
        }
        $intv = '';
        if($it > 1 ){
          $intv = ucfirst($dur).'s';  
        }else{
           $intv = ucfirst($dur);  
        }
        return $intv;
    }
}

if (!function_exists('__plan_duration_seconds')) {
    /**
     * Get the duration of a plan in seconds.
     *
     * @param mixed $it The plan duration constant.
     * @version 1.1.0
     * @return int|false Duration in seconds, or `false` if invalid.
     */
    function __plan_duration_seconds($it)
    {
        if (empty($it)) {
            return 0;
        }

        // Map durations to their respective seconds
        $durationMap = [
            PlanDetails::HOURLYDURATION   => 3600,
            PlanDetails::DAILYDURATION    => 3600 * 24,
            PlanDetails::WEEKLYDURATION   => 3600 * 24 * 7,
            PlanDetails::MONTHLYDURATION  => 3600 * 24 * 30,
            PlanDetails::YEARLYDURATION   => 3600 * 24 * 365,
        ];

        return $durationMap[$it] ?? 0;
    }
}


if (!function_exists('__plan_interval_seconds')) {
    /**
     * Get the interval of a plan in seconds.
     *
     * @param mixed $it The plan interval constant.
     * @version 1.1.0
     * @return int|false Interval in seconds, or `false` if invalid.
     */
    function __plan_interval_seconds($it)
    {
        if (empty($it)) {
            return 0;
        }

        // Map intervals to their respective seconds
        $intervalMap = [
            PlanDetails::HOURLYINTERVAL   => 3600,
            PlanDetails::DAILYINTERVAL    => 3600 * 24,
            PlanDetails::WEEKLYINTERVAL   => 3600 * 24 * 7,
            PlanDetails::BIWEEKLYINTERVAL => 3600 * 24 * 14,
            PlanDetails::MONTHLYINTERVAL  => 3600 * 24 * 30,
            PlanDetails::YEARLYINTERVAL   => 3600 * 24 * 365,
        ];

        return $intervalMap[$it] ?? 0;
    }
}


if(!function_exists('__plan_return_amount')){
    /**
     * @Get User Withdrawal Select Option
     * Param NULL
     * @version 1.0.0
     * @return string
     */
    function __plan_return_amount($plan){
       $return_perc = get_json_value($plan->data_meta , 'return'); 
        $return_type = get_json_value($plan->data_meta , 'return_type');
        $return_amount = $return_perc;
        
        if($return_type == 'Percentage'){
         $rpercent = $return_perc / 100;  
         $return_amount = $rpercent * $plan->amount;
        }
        
        return $return_amount;
        
    }
}

if (!function_exists('__plan_badge')) {
    /**
     * Get the badge class for a given plan status.
     *
     * @param string|null $status Plan status
     * @return string|null CSS class for the badge
     */
    function __plan_badge($status)
    {
        $statusBadgeMap = [
            PlanStatus::ACTIVE   => 'badge-success',
            PlanStatus::INACTIVE => 'badge-primary',
            PlanStatus::ENDED    => 'badge-primary',
            PlanStatus::PAUSED   => 'badge-danger',
        ];

        return $statusBadgeMap[$status] ?? null;
    }
}




