<?php

namespace App\Http\Controllers\Admin;

use App\Http\Controllers\Controller as BaseController;

use App\Models\Order;
use App\Models\User;
use App\Models\Wallet_io;

use App\Services\AuthService;
use App\Services\WalletService;
use App\Enums\OrderStatus;
use App\Enums\TransactionStatus;
use App\Enums\TransactionCalcType;
use App\Enums\TransactionType;

use Illuminate\Http\RedirectResponse;
use Illuminate\Http\Request;
use Illuminate\Http\Response;
use Illuminate\Support\Facades\Auth;
use Illuminate\Support\Facades\Hash;
use Illuminate\Support\Facades\Log;
use Illuminate\Support\Facades\Redirect;
use Illuminate\Support\Str;
use Illuminate\Validation\ValidationException;
use Illuminate\View\View;

use Yajra\DataTables\Facades\DataTables;

use Arr;
use Carbon\Carbon;
use DB;
use Session;



class OrderController extends BaseController{
     
    function __construct(
        AuthService $authService

        
    ){
        $this->authService = $authService;

    }
    
    /****
     * 
     * **/
     
 public function list(Request $request , $u){
        $title = "History - Wallet ";
        $user = auth()->user();
        $client = User::where('id' , $u)->first();
        if(empty($client)){
           abort(404);
        }
        
        if ($request->ajax()) {
            $data = Order::query();
            if ($request->has('type') && !empty($request->type)) {
                if($request->type != ''){
                  $data->where('type', strtolower($request->type));  
                }
            }
         
            if ($request->has('stats') && !empty($request->stats)) {
                if($request->stats != ''){
                  $data->where('status', ucfirst($request->stats));  
                }
            }
         
            if(!blank($client)){
              $data->where('user' , $client->id); 
            }
            $data->select(['id','type','trade_date','orderID','symbol','volume','status'])->orderBy('trade_date','desc');
            return DataTables::of($data)
                ->addIndexColumn()
                ->addColumn('symbol' , function($row){
                    return $row->symbols()->symbol;
                })
                ->addColumn('action', function($row){
                  
                    $url =  route('admin.order.delete' , __the_hash($row->id));
                    $btn = "";
                    if($row->status === OrderStatus::PENDING || $row->status === OrderStatus::OPEN ){
                    $btn .= '<button class="edit_data me-2 btn btn-primary btn-sm"';
                    $btn .= "data-id='".$row->orderID."'";
                    $btn .= "data-name='".$row->orderID."'";
                    $btn .= '>Edit</button>';
                    }
                    $btn .= '<button class="delete_data mx-1 btn btn-danger btn-sm"';
                    $btn .= "data-url='".$url."'";
                    $btn .= "data-id='".$row->orderID."'";
                     $btn .= "data-name='".__($row->orderID." - ".$row->symbols()->symbol)."'";
                    $btn .= '>Delete</button>';
                    return $btn;
                })
                ->rawColumns(['action'])
                 ->setRowAttr([
            'data-id' => function($row) {
                return $row->id; // or any other attribute from your model
            },
            'data-status' => function($row) {
                return $row->status; // or any other attribute from your model
            },
            'data-trx-id' => function($row) {
                return $row->orderID; // or any other attribute from your model
            }
        ])
                ->make(true);
        }
        
       

        return view('admin.order.list' , [
            'title' => $title,
            'client' => $client,
        ]);
    }
    
 public function allList(Request $request , $t = '' , $s = ''){
        $title = "History - Wallet ";
        $user = auth()->user();
     
        $type = '';
        $status = '';
      
        
        if ($request->ajax()) {
            $data = Order::query();
            if ($request->has('type') && !empty($request->type)) {
                if($request->type != ''){
                  $data->where('type', strtolower($request->type));  
                }
            }
            if ($request->has('stats') && !empty($request->stats)) {
                if($request->stats != ''){
                  $data->where('status', ucfirst($request->stats));  
                }
            }
         
            $data->select(['id','orderID','created_date','type','amount','status'])->orderBy('created_date','desc');
            return DataTables::of($data)
                ->addIndexColumn()
                ->addColumn('method' , function($row){
                    return '';
                })
                ->addColumn('action', function($row){
                  
                    $url =  route('admin.order.delete' , $row->id);
                    $btn = "";
                    if($row->status === OrderStatus::PENDING || $row->status === OrderStatus::NOT_PAID || $row->status === OrderStatus::HOLD){
                    $btn .= '<button class="edit_data mx-1 btn btn-primary btn-sm"';
                    $btn .= "data-name='".$row->orderID."'";
                    $btn .= '>Edit</button>';
                    }
                    $btn .= '<button class="delete_data mx-1 btn btn-danger btn-sm"';
                    $btn .= "data-url='".$url."'";
                    $btn .= "data-name='".$row->orderID."'";
                    $btn .= '>Delete</button>';
                    return $btn;
                })
                ->rawColumns(['action'])
                 ->setRowAttr([
            'data-id' => function($row) {
                return $row->id; 
            },
            'data-status' => function($row) {
                return $row->status; 
            },
            'data-trx-id' => function($row) {
                return $row->orderID; 
            }
        ])
                ->make(true);
        }
        
         $typeArray = [OrderType::DEPOSIT,OrderType::WITHDRAW];
        if(in_array(strtolower($t), $typeArray)){
          $type =  $t; 
        }
        
        if(strtolower($s) == "pending"){
           $status = "Pending"; 
        }
        return view('admin.order.all_list' , [
            'title' => $title,
            'type' => $type,
            'status' =>$status
        ]);
    }
    
 public function addOrder(Request $request){
    
    $data =array();
   
    
    $user = User::where('id' , $request->xrid)->first();
    if(empty($user)){
        $data['status'] = false;
        $data['message'] = 'User Does Not Exist';
        echo json_encode($data);
        return true;
    }
    
    $ubalance = User_balance::where('user' , $request->xrid)->first();
    if(empty($ubalance)){
        $data['status'] = false;
        $data['message'] = 'User Balance Does Not Exist';
        echo json_encode($data);
        return true;
    }
    
    $pmethod = Payment_method::where('id' , $request->ptype)->first();
    if(empty($pmethod)){
        $data['status'] = false;
        $data['message'] = 'Payment Method Does Not Exist';
        echo json_encode($data);
        return true;
    }
    
     
    
        $symbol = new Deposit();
        $symbol->orderID = "DD". random_string('numeric', 7);;
        $symbol->type = 'Balance';
        $symbol->user = $user->id;
        $symbol->amount = $request->amt;
        $symbol->payment_mode = $pmethod->id;
        $symbol->status = 'Approved';
        $symbol->currency = $pmethod->symbol;
        $symbol->exchange_rate = 1;
        $symbol->memo = $request->mnvol;
        $symbol->account_type = $request->atype;
        $symbol->processed_at = \Carbon\Carbon::now();
     
        if($request->atype == 'real'){
            $balance = $ubalance->real_account + $request->amt ; 
          $ubalance->real_account =  $balance ;
          $ubalance->update();
        }else{
          $balance = $ubalance->demo_account + $request->amt ; 
          $ubalance->demo_account =  $balance ;
          $ubalance->update();
        }
        $symbol->save();
       
        $data['status'] = true;
        $data['message'] = 'Deposits Added Successfully';
        echo json_encode($data);
        
        
        
    }
 
   //process deposits
 public function processOrder(Request $request){
      
      $order = Order::where('id',$request->gixd)->first();
      if(!empty($order)){
       $client = User::where('id' , $order->user)->first();
       if($order->type == 'send' && ($request->processStats == 'Cancelled' || $request->processStats == 'Failed')){
           $wallet = Wallet_io::where('id' , $order->outgoing_wallet)->first();
           Wallet_io::where('id' , $wallet->id)->update([
          'balance' => $wallet->balance + $order->amount
          ]);
           
       }
   
      Order::where('id' , $order->id)->update([
          'status' => $request->processStats
          ]);
      return redirect()->back()
      ->with('sucmessage', $order->orderID.' has been processed - '.$request->processStats);
      }
      
     return redirect()->back()
      ->with('errmessage', 'Order not found');
      
    }
    
 public function cancelOrder($id){
    
      $deposit=Deposit::where('id',$id)->first();
    
      if(!empty($deposit)):
        $client =Users::where('id',$deposit->user)->first(); 
      
            
   DB::table('deposits')->where('id',$deposit->id)
      ->update([
      'status' => DepositsType::CANCELLED,
      'processed_at' => \Carbon\Carbon::Now(),
      ]);
      $feeInfo = 0;
      try { 
          
           ProcessEmail::dispatch('deposit-reject-customer', $deposit->account(), null, $deposit, $feeInfo);
} catch(\Exception $e){
    
}
      return redirect()->back()
      ->with('sucmessage', $deposit->orderID.' has been cancelled');
         
          
      endif;
      
   
 }
 
 public function deleteOrder($id){
     
    $order=Order::where('id',__get_hash($id))->first();
  
      if(!empty($order)):
        $order->delete();
 
      return redirect()->back()
      ->with('sucmessage', $order->orderID.' has been deleted');
         
          
      endif;
      return redirect()->back()
      ->with('sucmessage', $order->orderID.' delete failed');
     
 }

 public function Single($id, Request $request){
        
        if($request->ajax()){
            $data = array();
            $user = auth()->user();
           
        
         $order = Order::where('orderID' , $id)->first();
         
         if(empty($order)){
             return response()->json([
                  'status' => false,
                  'message' => "Order Not Found"
                  ]);
         }
          
         $view = view("admin.order.edit" , [
             'user' => $user,
             'order' => $order
             ])->render();
         return response()->json([
                  'status' => true, 
                  'view' => $view
                  ]);
         
       }  
        
    }
    
 public function editOrder(Request $request)
    {
        $order = Order::where('id', $request->gixd)->first();
        if (empty($order)) {
            return redirect()
                ->back()
                ->with('errmessage', 'Order update failed');
        }
      
      
            if($request->status == OrderStatus::WON){
               __update_balance($order->duser()->id,$request['payout'],'increase');
          }
        
    
        Order::where('id', $order->id)->update([
            'volume' => $request['amount'],
            'entry_price' => $request['entry_price'],
            'leverage' => $request['leverage'],
            'tProfit' => $request['take_profit'],
            'sLoss' => $request['stop_loss'],
            'payout' =>  $request['payout'],
            'status' => $request['status'],
        ]);

        return redirect()
            ->back()
            ->with('sucmessage', $order->orderID.' -  order  updated successfully');
    }
}