<?php

namespace App\Http\Controllers\Admin;


use App\Models\Plan;
use App\Models\Transaction;
use App\Models\User;
use App\Models\User_balance;
use App\Models\User_plan;

use App\Enums\PlanStatus;
use App\Enums\TransactionType;
use App\Enums\TransactionCalcType;
use App\Enums\TransactionStatus;

use App\Jobs\ProcessEmail;
use App\Services\FundsService;

use App\Http\Controllers\Controller as BaseController;
use Illuminate\Http\Request;
use Illuminate\Http\Response;
use DB;
use Config;
use Cache;
use Illuminate\Support\Facades\Auth;

class PlanController extends BaseController
{
    function __construct(
        FundsService $fundService
    ){
        $this->fundService = $fundService;
    }

    public function index($userid = null)
    {
        $user = auth()->user();
        $plans = Plan::all();

        return view('admin.plan.index', [
            'user' => $user,
            'plans' => $plans,
        ]);
    }

    public function invest($userid = null)
    {
        $user = auth()->user();
        $client = '';
        if (!empty($userid) || $userid != null) {
            $investment = User_plan::where('user', $userid)->orderBy('activated_at' , 'desc')->paginate(20);
            $client = User::where('id', $userid)->first();
        } else {
            $investment = User_plan::orderBy('activated_at' , 'desc')->paginate(20);
        }

        return view('admin.investment.index', [
            'user' => $user,
            'client' => $client,
            'investments' => $investment,
        ]);
    }

    public function addplan(Request $request)
    {
        $plan = new Plan();
        $wal = "";
        if (isset($request['pay'])) {
            if (is_array($request['pay'])) {
                $wal = implode(',', $request['pay']);
            }
        }

        $plan->name = $request['name'];
        $plan->price = $request['price'];
        $plan->min_price = $request['price'];
        $plan->max_price = $request['maxi'];
        $plan->type = $request['type'];
        $plan->intervals = $request['interv'];
        $plan->in_type = 'Percentage';
        $plan->percs = $request['perc'];
        $plan->duration = $request['dur'];
        $plan->duration_time = $request['dur_time'];

        $plan->save();
        return redirect()
            ->back()
            ->with('sucmessage', 'Plan created Sucessful!');
    }

    public function editplan(Request $request)
    {
        $planx = Plan::where('id', $request->gixd)->first();
        if (empty($planx)) {
            return redirect()
                ->back()
                ->with('errmessage', 'Plan update failed');
        }
        $wal = "";
        if (isset($request['pay'])) {
            if (is_array($request['pay'])) {
                $wal = implode(',', $request['pay']);
            }
        }

        $plan = [];

        $plan['name'] = $request['name'];
        $plan['price'] = $request['price'];
        $plan['min_price'] = $request['price'];
        $plan['max_price'] = $request['maxi'];
        $plan['type'] = $request['type'];
        $plan['intervals'] = $request['interv'];
        $plan['percs'] = $request['perc'];
        $plan['duration'] = $request['dur'];
        $plan['duration_time'] = $request['dur_time'];
        Plan::where('id', $planx->id)->update($plan);
        return redirect()
            ->back()
            ->with('sucmessage', $request['name'] . ' -- Plan updated Sucessful!');
    }

    public function deletePlan($id)
    {
        $plan = Plan::where('id', $id)->first();
        if (empty($plan)) {
            return redirect()
                ->back()
                ->with('errmessage', 'Plan not  found');
        }
        $plan->delete();
        $message = $plan->name . ' was deleted successfully';
        return redirect()
            ->back()
            ->with('sucmessage', $message);
    }

    public function pauseInvestment($id)
    {
        $planx = User_plan::where('id', $id)->first();

        if (empty($planx)) {
            return redirect()
                ->back()
                ->with('errmessage', 'Plan update failed');
        }
        User_plan::where('id', $planx->id)->update([
            'active' => PlanStatus::PAUSED,
        ]);
        try {
            ProcessEmail::dispatch('investment-paused-customer', $planx->duser(), null, $planx);
        } catch (\Exception $e) {
            //  save_mailer_log($e, 'deposit-placed');
        }
        return redirect()
            ->back()
            ->with('sucmessage', $planx->txn_id . ' - ' . $planx->dplan()->name . ' has been paused!');
    }

    public function resumeInvestment($id)
    {
        $planx = User_plan::where('id', $id)->first();
        if (empty($planx)) {
            return redirect()
                ->back()
                ->with('errmessage', 'Plan resumption failed');
        }
        User_plan::where('id', $planx->id)->update([
            'active' => PlanStatus::ACTIVE,
        ]);

        try {
            ProcessEmail::dispatch('investment-resume-customer', $planx->duser(), null, $planx);
        } catch (\Exception $e) {
            //  save_mailer_log($e, 'deposit-placed');
        }
        return redirect()
            ->back()
            ->with('sucmessage', $planx->txn_id . ' - ' . $planx->dplan()->name . ' has been resumed!');
    }

    public function autoInvestment($id)
    {
        $planx = User_plan::where('id', $id)->first();
        if (empty($planx)) {
            return redirect()
                ->back()
                ->with('errmessage', 'Plan  failed');
        }
        User_plan::where('id', $planx->id)->update([
            'autot' => 'yes',
        ]);
        return redirect()
            ->back()
            ->with('sucmessage', $planx->txn_id . ' - ' . $planx->dplan()->name . ' autotopup has been started!');
    }
    public function endAutoInvestment($id)
    {
        $planx = User_plan::where('id', $id)->first();
        if (empty($planx)) {
            return redirect()
                ->back()
                ->with('errmessage', 'Plan  failed');
        }
        User_plan::where('id', $planx->id)->update([
            'autot' => 'no',
        ]);
        return redirect()
            ->back()
            ->with('sucmessage', $planx->txn_id . ' - ' . $planx->dplan()->name . ' auto top up has been ended!');
    }

    public function endInvestment($id)
    {
        $planx = User_plan::where('id', $id)->first();

        if (blank($planx)) {
            return redirect()
                ->back()
                ->with('errmessage', 'Plan update failed');
        }

        User_plan::where('id', $planx->id)->update([
            'active' => PlanStatus::ENDED,
        ]);

        try {
            ProcessEmail::dispatch('investment-ended-customer', $planx->duser(), null, $planx);
        } catch (\Exception $e) {
            //  save_mailer_log($e, 'deposit-placed');
        }
   

        return redirect()
            ->back()
            ->with('sucmessage', $planx->txn_id . ' - ' . $planx->dplan()->name . ' has been ended!');
    }

    //Manually Add Trading History to User Route
    public function topProfit(Request $request)
    {
        $planx = User_plan::where('id', $request->gixd)->first();

        if (blank($planx)) {
            return redirect()
                ->back()
                ->with('errmessage', 'Plan topup failed');
        }
        $user = User::where('id', $planx->user)->first();

        User_plan::where('id', $planx->id)->update(['roi' => $planx->roi + $request->amount]);
        $user->account()->balance = $user->account()->balance + $request->amount;
        $user->account()->profit = $user->account()->profit + $request->amount;
        $data = array();
        $data['method'] = $planx->id;
        $data['amount'] = $request->amount;
        $data['payment'] = $planx;
        $data['trans_id'] = random_string('capalnum', 8);
        $data['type'] = TransactionType::PROFIT;
        $data['calc'] = TransactionCalcType::DEBIT;
        $data['descr'] = "Profit returned on  ". $planx->dplan()->name;
        $data['status'] = TransactionStatus::CONFIRMED;
        $data['details'] = '';
        
        
        $dep_id = $this->fundService->save_transaction($data);
    
        
        return redirect()
            ->back()
            ->with('sucmessage', $request->amount . ' has been ended to' . $planx->txn_id . ' - ' . $planx->dplan()->name);
    }
}
