<?php

namespace App\Http\Controllers\Admin;

use App\Http\Controllers\Controller as BaseController;

use App\Models\PaymentMethod;
use App\Models\Transaction;
use App\Models\User;
use App\Models\AccountIo;


use App\Services\AuthService;
use App\Services\FundsService;
use App\Enums\TransactionStatus;
use App\Enums\TransactionCalcType;
use App\Enums\TransactionType;

use Illuminate\Http\RedirectResponse;
use Illuminate\Http\Request;
use Illuminate\Http\Response;
use Illuminate\Support\Facades\Auth;
use Illuminate\Support\Facades\Hash;
use Illuminate\Support\Facades\Log;
use Illuminate\Support\Facades\Redirect;
use Illuminate\Support\Str;
use Illuminate\Validation\ValidationException;
use Illuminate\View\View;

use Yajra\DataTables\Facades\DataTables;

use Arr;
use Carbon\Carbon;
use DB;
use Session;



class TransactionController extends BaseController{
     
    function __construct(
        AuthService $authService,
        FundsService $walletService
        
    ){
        $this->authService = $authService;
        $this->sundsService = $walletService;
    }
    
    /****
     * 
     * **/
     
 public function list(Request $request , $u){
        $title = "History - Wallet ";
        $user = auth()->user();
        $client = User::where('id' , $u)->first();
        if(empty($client)){
           abort(404);
        }
        
        if ($request->ajax()) {
            $data = Transaction::query();
            
            if ($request->has('type') && !empty($request->type)) {
                if($request->type != ''){
                  $data->where('type', strtolower($request->type));  
                }
            }
         
            if ($request->has('stats') && !empty($request->stats)) {
                if($request->stats != ''){
                  $data->where('status', ucfirst($request->stats));  
                }
            }
         
            if(!blank($client)){
              $data->where('user' , $client->id); 
            }
            $data->select(['id','txnID','account_id','created_date','type','amount','method','status'])->orderBy('created_date','desc');
           
            return DataTables::of($data)
                ->addIndexColumn()
                ->addColumn('method' , function($row){
                    
                    if($row->type === TransactionType::DEPOSIT && !blank($row->depositMethod())){
                       return  $row->depositMethod()->first()->name;
                    }elseif($row->type === TransactionType::WITHDRAW  && !blank($row->withdrawalMethod())){
                        return $row->withdrawalMethod()->first()->name;
                    }
                    return "";
                })->addColumn('account' , function($row){
                
                    if(!blank($row->account())){
                       $d =  $row->account()->first();
                       return $d->accountUnder()->title;
                    }
                    return "yes";
                })
                ->addColumn('action', function($row){
                  
                    $url =  route('admin.transaction.delete' , $row->id);
                    $btn = "";
                    if($row->status === TransactionStatus::PENDING || $row->status === TransactionStatus::NOT_PAID  || $row->status === TransactionStatus::HOLD){
                    $btn .= '<button class="edit_data me-2 btn btn-primary btn-sm"';
                    $btn .= "data-name='".$row->txnID."'";
                    $btn .= '>Edit</button>';
                    }
                    $btn .= '<button class="delete_data mx-1 btn btn-danger btn-sm"';
                    $btn .= "data-url='".$url."'";
                    $btn .= "data-name='".$row->txnID."'";
                    $btn .= '>Delete</button>';
                    return $btn;
                })
                ->rawColumns(['action'])
                 ->setRowAttr([
            'data-id' => function($row) {
                return $row->id; // or any other attribute from your model
            },
            'data-status' => function($row) {
                return $row->status; // or any other attribute from your model
            },
            'data-trx-id' => function($row) {
                return $row->txnID; // or any other attribute from your model
            }
        ])
                ->make(true);
        }
        
       

        return view('admin.transactions.list' , [
            'title' => $title,
            'client' => $client,
        ]);
    }
    
 public function allList(Request $request , $t = '' , $s = ''){
        $title = "History - Wallet ";
        $user = auth()->user();
     
        $type = '';
        $status = '';
      
        
        if ($request->ajax()) {
            $data = Transaction::query();
            if ($request->has('type') && !empty($request->type)) {
                if($request->type != ''){
                  $data->where('type', strtolower($request->type));  
                }
            }
            if ($request->has('stats') && !empty($request->stats)) {
                if($request->stats != ''){
                  $data->where('status', ucfirst($request->stats));  
                }
            }
         
            $data->select(['id','txnID','created_date','type','amount','status'])->orderBy('created_date','desc');
            return DataTables::of($data)
                ->addIndexColumn()
                ->addColumn('method' , function($row){
                    return '';
                })
                ->addColumn('action', function($row){
                  
                    $url =  route('admin.transaction.delete' , $row->id);
                    $btn = "";
                    if($row->status === TransactionStatus::PENDING || $row->status === TransactionStatus::NOT_PAID || $row->status === TransactionStatus::HOLD){
                    $btn .= '<button class="edit_data mx-1 btn btn-primary btn-sm"';
                    $btn .= "data-name='".$row->txnID."'";
                    $btn .= '>Edit</button>';
                    }
                    $btn .= '<button class="delete_data mx-1 btn btn-danger btn-sm"';
                    $btn .= "data-url='".$url."'";
                    $btn .= "data-name='".$row->txnID."'";
                    $btn .= '>Delete</button>';
                    return $btn;
                })
                ->rawColumns(['action'])
                 ->setRowAttr([
            'data-id' => function($row) {
                return $row->id; 
            },
            'data-status' => function($row) {
                return $row->status; 
            },
            'data-trx-id' => function($row) {
                return $row->txnID; 
            }
        ])
                ->make(true);
        }
        
         $typeArray = [TransactionType::DEPOSIT,TransactionType::WITHDRAW];
        if(in_array(strtolower($t), $typeArray)){
          $type =  $t; 
        }
        
        if(strtolower($s) == "pending"){
           $status = "Pending"; 
        }
        return view('admin.transactions.all_list' , [
            'title' => $title,
            'type' => $type,
            'status' =>$status
        ]);
    }
    
 public function addTransaction(Request $request){
    
    $data =array();
   
    
    $user = User::where('id' , $request->xrid)->first();
    if(empty($user)){
        $data['status'] = false;
        $data['message'] = 'User Does Not Exist';
        echo json_encode($data);
        return true;
    }
    
    $ubalance = User_balance::where('user' , $request->xrid)->first();
    if(empty($ubalance)){
        $data['status'] = false;
        $data['message'] = 'User Balance Does Not Exist';
        echo json_encode($data);
        return true;
    }
    
    $pmethod = Payment_method::where('id' , $request->ptype)->first();
    if(empty($pmethod)){
        $data['status'] = false;
        $data['message'] = 'Payment Method Does Not Exist';
        echo json_encode($data);
        return true;
    }
    
     
    
        $symbol = new Deposit();
        $symbol->txnID = "DD". random_string('numeric', 7);;
        $symbol->type = 'Balance';
        $symbol->user = $user->id;
        $symbol->amount = $request->amt;
        $symbol->payment_mode = $pmethod->id;
        $symbol->status = 'Approved';
        $symbol->currency = $pmethod->symbol;
        $symbol->exchange_rate = 1;
        $symbol->memo = $request->mnvol;
        $symbol->account_type = $request->atype;
        $symbol->processed_at = \Carbon\Carbon::now();
     
        if($request->atype == 'real'){
            $balance = $ubalance->real_account + $request->amt ; 
          $ubalance->real_account =  $balance ;
          $ubalance->update();
        }else{
          $balance = $ubalance->demo_account + $request->amt ; 
          $ubalance->demo_account =  $balance ;
          $ubalance->update();
        }
        $symbol->save();
       
        $data['status'] = true;
        $data['message'] = 'Deposits Added Successfully';
        echo json_encode($data);
        
        
        
    }
 
   //process deposits
 public function processTransaction(Request $request){
      
      $transaction = Transaction::where('id',$request->gixd)->first();
      if(!empty($transaction)){
       $client = User::where('id' , $transaction->user)->first();
       if($transaction->type == 'send' && ($request->processStats == 'Cancelled' || $request->processStats == 'Failed')){
           $wallet = AccountIo::where('id' , $transaction->outgoing_wallet)->first();
           AccountIo::where('id' , $wallet->id)->update([
          'balance' => $wallet->balance + $transaction->amount
          ]);
           
       }
   
      Transaction::where('id' , $transaction->id)->update([
          'status' => $request->processStats
          ]);
      return redirect()->back()
      ->with('sucmessage', $transaction->txnID.' has been processed - '.$request->processStats);
      }
      
     return redirect()->back()
      ->with('errmessage', 'Transaction not found');
      
    }
    
 public function cancelTransaction($id){
    
      $deposit=Deposit::where('id',$id)->first();
    
      if(!empty($deposit)):
        $client =Users::where('id',$deposit->user)->first(); 
      
            
   DB::table('deposits')->where('id',$deposit->id)
      ->update([
      'status' => DepositsType::CANCELLED,
      'processed_at' => \Carbon\Carbon::Now(),
      ]);
      $feeInfo = 0;
      try { 
          
           ProcessEmail::dispatch('deposit-reject-customer', $deposit->account(), null, $deposit, $feeInfo);
} catch(\Exception $e){
    
}
      return redirect()->back()
      ->with('sucmessage', $deposit->txnID.' has been cancelled');
         
          
      endif;
      
   
 }
 
 public function deleteTransaction($id){
     
    $transaction=Transaction::where('id',$id)->first();
  
      if(!empty($transaction)):
        $transaction->delete();
 
      return redirect()->back()
      ->with('sucmessage', $transaction->txnID.' has been deleted');
         
          
      endif;
      return redirect()->back()
      ->with('sucmessage', $transaction->txnID.' delete failed');
     
 }

 public function Single($id, Request $request){
        
        if($request->ajax()){
            $data = array();
            $user = auth()->user();
           
        
         $transaction = Transaction::where('txnID' , $id)->first();
         
         if(empty($transaction)){
             return response()->json([
                  'status' => false,
                  'message' => "Transaction Not Found"
                  ]);
         }
          
         $view = view("admin.transactions.edit" , [
             'user' => $user,
             'transaction' => $transaction
             ])->render();
         return response()->json([
                  'status' => true, 
                  'view' => $view
                  ]);
         
       }  
        
    }
    
 public function editTransaction(Request $request)
{
    $transaction = Transaction::find($request->gixd);
    if (!$transaction) {
        return redirect()
            ->back()
            ->with('errmessage', 'Transaction update failed');
    }

    // Update transaction details
    Transaction::where('id', $transaction->id)->update([
            'amount' => $request['amount'],
            'status' => $request['status'],
        ]);

    $account = $transaction->account();
    $client = $transaction->user();

    if ($transaction->type == TransactionType::DEPOSIT) {
        if ($request->status == TransactionStatus::CONFIRMED) {
            $account->increment('balance', $request->amount);
            __balance_history([
                'amount' => $request->amount,
                'user' => $client->id
            ]);
        }
    } elseif ($transaction->type == TransactionType::WITHDRAW) {
        if ($request->status == TransactionStatus::DECLINED || $request->status == TransactionStatus::FAILED) {
            $account->increment('balance', $request->amount);
            __balance_history([
                'amount' => $request->amount,
                'user' => $client->id
            ]);
        }
    }

    return redirect()
        ->back()
        ->with('sucmessage', $transaction->txnID . ' - transaction updated successfully');
}

}