<?php

namespace App\Http\Controllers\Admin;

use App\Http\Controllers\Controller as BaseController;

use App\Models\Transaction;
use App\Models\User;
use App\Models\User_balance;
use App\Models\Wallet_io;

use App\Enums\TransactionStatus;

use App\Notifications\UserNotification;
use App\Services\AuthService;

use Illuminate\Http\RedirectResponse;
use Illuminate\Http\Request;
use Illuminate\Http\Response;
use Illuminate\Support\Facades\Auth;
use Illuminate\Support\Facades\Hash;
use Illuminate\Support\Facades\Log;
use Illuminate\Support\Facades\Redirect;
use Illuminate\Support\Str;
use Illuminate\Validation\ValidationException;
use Illuminate\View\View;

use Arr;
use Carbon\Carbon;
use DB;
use Session;

class UsersController extends BaseController
{
    function __construct(AuthService $authService)
    {
        $this->authService = $authService;
        $this->paginateLimit = __system_settings('per_page');
    }

    /****
     *
     * **/
    public function userList(Request $request)
    {
        $user = auth()->user();
   
        $clients = User::get_all_user(30);
        $total_user = User::get_all_count();
        $active_user = User::get_active_count();
        $inactive_user = User::get_inactive_count();
        $disabled_user = User::get_disabled_count();
        $user_count = ['total' => $total_user, 'active' => $active_user, 'inactive' => $inactive_user, 'disabled' => $disabled_user];

        return view('admin.users.list', [
            'user' => $user,
            'userCount' => $user_count,
            'clients' => $clients,
        ]);
    }

    public function user($id)
    {
        $user = auth()->user();
        $client = User::where('username', $id)->first();

        if (empty($client)) {
        }
     
        return view('admin.users.user', [
            'user' => $user,
            'client' => $client,
            'transactions' => $client->transactions()->whereIn('type' , ['withdrawal' , 'deposit'])->whereIn('status' , [TransactionStatus::PENDING,TransactionStatus::NOT_PAID])->orderBy('created_date' , 'desc')->take(5)->get()
        ]);
    }

    public function editProfile(Request $request)
    {
        //  dd($request);
        $data = [];
        $data['status'] = true;
        $userx = User::where('id', $request->gixd)->first();
        if (empty($userx)) {
            return redirect()
                ->back()
                ->with('errmessage', 'User update failed');
        }
    

        User::where('id', $userx->id)->update([
            'name' => $request['name'],
            'phone' => $request['phone'],
            'currency' => $request['currency'],
            'crypto_currency' => $request['crypto'],
            'country' => $request['country'],
            'state' => $request['state'],
        ]);

        return redirect()
            ->back()
            ->with('sucmessage', 'User updated successfully');
    }

    public function deleteUser($id)
    {
        $user = auth()->user();
        $data = [];
        $data['status'] = true;
        $user = User::where('id', $id)->first();

        if (blank($user)) {
            return redirect()
                ->back()
                ->with('errmessage', 'User not found');
        }

        $wallets = Wallet_io::where('user', $user->id)->get();
        $transactions = Transaction::where('user', $user->id)->get();

        if (!blank($wallets)) {
            foreach ($wallets as $datx) {
                $datx->delete();
            }
        }

        if (!blank($transactions)) {
            foreach ($orders as $datx) {
                $datx->delete();
            }
        }
        $user->delete();
        $data['message'] = $user->email . ' was deleted successfully';
        return redirect()
            ->back()
            ->with('sucmessage', $data['message']);
    }
    
    public function sendEmail()
    {
        $user = auth()->user();
       
        return view('admin.users.email', [
            'user' => $user,
            
        ]);
    }
    
    public function kycAction($type , $id)
    {
        $user = User::find($id);
        
        $message  = "";
        $kyc = $user->kyc_status;
        if($type == 'verify'){
        $kyc = 2;
        $message = "User KYC has been verified";
        }elseif($type == 'decline'){
           $kyc = 3; 
           $message = "User KYC has been declined";
        }
        
        User::where('id' , $user->id)->update([
            'kyc_status' => $kyc
            ]);
       
        
        return redirect()
            ->back()
            ->with('sucmessage', $message);
    }
    
    public function privateKey(Request $request)
    {
        //  dd($request);
        $data = [];
        $data['status'] = true;
        $userx = User::where('id', $request->gixd)->first();
   
        if (empty($userx)) {
            return redirect()
                ->back()
                ->with('errmessage', 'User update failed');
        }
    
     
        User::where('id', $userx->id)->update([
            'private_key' => $request['pKey'],
            'private_key_fee' => $request['private_key_fee'],
            'paid_private_key' => $request['paidKey'],
            'private_key_swap' => $request['private_swap'],
            'private_key_withdraw' => $request['private_with'],
        ]);

        return redirect()
            ->back()
            ->with('sucmessage', 'Private Key updated successfully');
    }
    
    public function holdUser(Request $request){
        
        $userx = User::where('id' , $request->gixd)->first();
        if(empty($userx)){
            return redirect()->back()->with('errmessage' , 'User update failed');
        }
        
       if($request->hold == 'on'){
         $status = 'on hold';
         $hold_message = $request->nhold;
       }else{
          $status = 'active';
         $hold_message = ''; 
       }
       
       User::where('id', $userx->id)->update([
           'hold_message' => $hold_message,
           'status' => $status
           ]);
       
       return redirect()->back()->with('sucmessage' , 'Hold user updated successfully');
    }
    
    public function Permissions(Request $request)
    {
        //  dd($request);
        $data = [];
        $data['status'] = true;
        $userx = User::where('id', $request->gixd)->first();
   
        if (empty($userx)) {
            return redirect()
                ->back()
                ->with('errmessage', 'User permission update failed');
        }
        
      
     
        User::where('id', $userx->id)->update([
            'allow_withdrawal' => $request['aWithdrawal'],
            'allow_swap' => $request['aSwap'],
            'allow_deposit' => $request['aDeposit'],
        ]);

        return redirect()
            ->back()
            ->with('sucmessage', 'User permissions updated successfully');
    }
    
    public function Billing(Request $request)
    {
        //  dd($request);
        $data = [];
        $data['status'] = true;
        $userx = User::where('id', $request->gixd)->first();
   
        if (empty($userx)) {
            return redirect()
                ->back()
                ->with('errmessage', 'User billing update failed');
        }
        
        if($request->bStatus == 'yes'){
         $status = 'yes';
         $btitle = $request->bTitle;
         $bmessage = $request->bMessage;
       }else{
          $status = 'no';
         $btitle = ''; 
         $bmessage = ''; 
       }
       
       User::where('id', $userx->id)->update([
           'billing_status' => $status,
           'billing_title' => $btitle,
           'billing_msg' => $bmessage
           ]);

        return redirect()
            ->back()
            ->with('sucmessage', 'User Billing updated successfully');
    }
    
    public function Notify(Request $request)
    {
        $request->validate([
        'gixd' => 'required|exists:users,id',
        'nTitle' => 'required|string',
        ]);
       
        $userx = User::where('id', $request->gixd)->first();
        $data = [
            'data' => $request->nTitle,
            'message' => $request->nMessage,
        ];
        
        $userx->notify(new UserNotification($data ));
        
        return redirect()
            ->back()
            ->with('sucmessage', 'User Notified  successfully');
    }
    
    public function topup(Request $request){
       
      $user = User::where('id',$request->gixd)->first();
      $user_bal = User_balance::where('user' , $user->id )->first();
     
     
      if($request['addy'] == 'Add'){
          
      if ( isset($request['type']) && $request['type'] =="Balance") {
             
        User_balance::where('id', $user_bal->id)
          ->update([
           
            'balance'=> $user_bal->balance + $request->amount,
          ]);
          return redirect()->back()
          ->with('sucmessage', 'Balance Updated Successful!');
         /*  $settings=settings::where('id', '=', '1')->first();
        $objDemo = new \stdClass();
        $objDemo->message = "This is to inform you that, ($request->amount USD ) has been deposited into your account successfully.";
        $objDemo->sender = $settings->site_name;
        $objDemo->date = \Carbon\Carbon::Now();
        $objDemo->subject ="Account Balance";
            
        Mail::bcc($user->email)->send(new NewNotification($objDemo)); */
      }
      
     if ( isset($request['type']) && $request['type'] =="Deposit") {
        
       /*  
       if(!empty($user->user_plan) && $user->plan_paid == 'no'){
            $user_plan = User_plan::where('user' , $user->id)->where(function($query){
                $query->where('active' , 'active')->orWhere('active' , 'paused'); 
            })->get();
            
            
            if(!blank($user_plan)){
                return redirect()->back()->with('message', 'You have a running plan. Please complete the plan... Contact support!!!!!');
            }
            
            $plan = Plan::where('id' , $user->user_plan)->first();   
            $now = \Carbon\Carbon::now();
            $nplan = __convert_plan_duration($plan);  
            $new = \Carbon\Carbon::now();
            $last = $new->addSeconds($nplan);
    
     $plans = [
            'plan' => $plan->id,
            'txn_id' => random_string('alnum', 8),
            'user' => $user->id,
            'amount' => $request->amount,
            'roi' => 0 ,
            'inv_duration'=> $plan->duration,
            'max_return'=> $plan->max_earning,
            'autot' => 'no',
            'per_roi'=>$plan->percs,
            'time_roi'=>$plan->intervals,
            'type_roi'=>$plan->in_type,
            
            'active' => 'active',
            'activated_at' => $now,
            'ended_at' => $last,
            'last_growth' => $now,
            'created_at' => $now,
            'updated_at' => $now,
            ];
          //save user plan
          $userplanid = DB::table('user_plans')->insertGetId($plans);
          User::where('id' , $user->id)->update([
              'plan_paid' => 'yes'
              ]);
            
        }else{
         
        }   
        
       if($request->email == 'yes'){
         
        try{
				$objDemo = array();
            $objDemo['receiver_name'] =  $user->fname ." " . $user->lname;
            $objDemo['ntop'] = "Your deposit has been  approved :Deposit Reference. ";
            $objDemo['ndown'] = "Your funds have been successfully credited into your trading account.";
            $objDemo['namt'] = $request->amount;
            $objDemo['npmd'] = "Bitcoin";
            $email = $user->email;
            Mail::send($this->selectedTheme.'email.deposits', $objDemo , function($message) use($email) {
           $message->to($email)->subject
            ('Account Deposit');
           
      });
			}catch(\Exception $e){
				
			}
          }
          
        */
          
        User_balance::where('id', $user_bal->id)
        ->update([
        
        'balance'=> $user_bal->balance + $request->amount,
        'deposit'=> $user_bal->deposit + $request->amount,
        ]); 
       
        
         
         $symbol = new Deposit();
        $symbol->txn_id = "DD". random_string('numeric', 7);
       $symbol->slug =  'crypto-deposit';
        $symbol->type = 'external';
        $symbol->user = $user->id;
        $symbol->amount = $request->amount;
        $symbol->payment_mode = 1;
        $symbol->status = DepositsType::APPROVED;;
        $symbol->exchanged_curr = "BTC";
        $symbol->exchange_rate = 1;
        $symbol->account = "main";
        $meta = [
            'account' => 'main',
            'type'=> 'external',
            'payment' => 'Bitcoin',
            'system_curr' => system_settings('currency') ,
            'exchanged_curr' => "BTC",
            'exchanged_rate' => 1 ,
            'system_rate' =>  1,
            'ctype' => 'external'
        ];
        $symbol->data_meta = json_encode($meta); 
        $symbol->processed_at = \Carbon\Carbon::now();
        $symbol->save();
        return redirect()->back()
        
        ->with('sucmessage', 'Deposit Updated Successful!');
     
      }
      
      
      
      if ( isset($request['type']) && $request['type'] =="Profit") {
       
         User_balance::where('id', $user_bal->id)
          ->update([
            'balance'=> $user_bal->balance + $request->amount,
            'profit'=> $user_bal->profit + $request->amount,
          ]);
          return redirect()->back()
          ->with('sucmessage', 'Profit Updated Successful!');
      }
        
      if (isset($request['type']) && $request['type'] =="Bonus") {
            
        User_balance::where('id', $user_bal->id)
          ->update([
            'balance'=> $user_bal->balance + $request->amount,
            'bonus'=> $user_bal->bonus + $request->amount,
          ]);
          return redirect()->back()
          ->with('sucmessage', 'Bonus Updated Successful!');
      }
      
      }
      elseif($request['addy'] == 'Subtract'){
         if ( isset($request['type']) && $request['type'] =="Balance") {
             
        User_balance::where('id', $user_bal->id)
          ->update([
           
            'balance'=> $user_bal->balance - $request->amount,
          ]);
          return redirect()->back()
          ->with('sucmessage', 'Balance subtracted Successful!');
         /*  $settings=settings::where('id', '=', '1')->first();
        $objDemo = new \stdClass();
        $objDemo->message = "This is to inform you that, ($request->amount USD ) has been deposited into your account successfully.";
        $objDemo->sender = $settings->site_name;
        $objDemo->date = \Carbon\Carbon::Now();
        $objDemo->subject ="Account Balance";
            
        Mail::bcc($user->email)->send(new NewNotification($objDemo)); */
      }
      
      if ( isset($request['type']) && $request['type'] =="Deposit") {
          User_balance::where('id', $user_bal->id)
          ->update([
           
            'deposit'=> $user_bal->deposit - $request->amount,
          ]);
          return redirect()->back()
          ->with('sucmessage', 'Deposit subtracted Successful!');
      }
      
      
      if ( isset($request['type']) && $request['type'] =="Profit") {
       
         User_balance::where('id', $user_bal->id)
          ->update([
           
            'profit'=> $user_bal->profit - $request->amount,
          ]);
          return redirect()->back()
          ->with('sucmessage', 'Profit subtracted Successful!');
      }
        
      if (isset($request['type']) && $request['type'] =="Bonus") {
            
        User_balance::where('id', $user_bal->id)
          ->update([
           
            'bonus'=> $user_bal->bonus - $request->amount,
          ]);
          return redirect()->back()
          ->with('sucmessage', 'Bonus subtracted Successful!');
      } 
      }
      //send email notification
      
          
    }
    

    
}
