<?php

namespace App\Http\Controllers\Admin;

use App\Http\Controllers\Controller as BaseController;

use App\Enums\NotificationType;
use App\Enums\TransactionStatus; 
use App\Enums\TransactionCalcType;
use App\Enums\TransactionType;

use App\Models\Coin;
use App\Models\Coin_network;
use App\Models\Transaction;
use App\Models\User;
use App\Models\Wallet_io;
use App\Notifications\TransactionNotification;

use App\Services\AuthService;

use Illuminate\Http\RedirectResponse;
use Illuminate\Http\Request;
use Illuminate\Http\Response;
use Illuminate\Support\Facades\Auth;
use Illuminate\Support\Facades\Hash;
use Illuminate\Support\Facades\Log;
use Illuminate\Support\Facades\Redirect;
use Illuminate\Support\Str;
use Illuminate\Validation\ValidationException;
use Illuminate\View\View;
use Yajra\DataTables\Facades\DataTables;

use Arr;
use Carbon\Carbon;
use DB;
use Session;



class WalletController extends BaseController{
     
    function __construct(
        AuthService $authService
    ){
        $this->authService = $authService;
    }
    
    /****
     * 
     * **/
    public function List(Request $request , $u = null){
        $title = "History - Wallet ";
        $user = auth()->user();
        $client = User::where('id' , $u)->first();
      
       
        
        if ($request->ajax()) {
            $data = Wallet_io::query();
          
            if ($request->has('coin') && !empty($request->coin)) {
                if($request->coin != ''){
                  $coin = Coin::where('symbol' , strtoupper($request->coin))->first();
                  $data->where('coin', $coin->id);  
                }
            }
           
            if(!blank($client)){
              $data->where('user' , $client->id); 
            }
            $data->select(['id','wallet_id','user','created_date','address','coin','balance','status'])->orderBy('created_date','desc');
            return DataTables::of($data)
                ->addIndexColumn()
                ->addColumn('coin' , function($row){
                    return $row->coinSymbol();
                })->addColumn('convertedBal' , function($row){
                    return __fiatFormat($row->convertedSysBalance()) ." ".__system_settings('currency');
                })->addColumn('user' , function($row){
                    $us = '';
                  
                    if(!blank($row->duser())){
                    $thisUser = $row->duser();
                    $url = route("admin.user.single", $thisUser->username);
                    
                    $us = '<a href="'.$url.'" class="text-body text-truncate"><span class="fw-semibold">'.$thisUser->email.'</span></a>';
                        
                    }
                    return $us;
                })
                ->addColumn('action', function($row){
                
                    $btn = '<div class="d-flex flex-column gap-1">';
                  
                    $btn .= '<button class="wallet_data mx-1 btn btn-primary btn-sm"';
                    $btn .= "data-name='".$row->wallet_id."'";
                    $btn .= "data-type='deposit'";
                    $btn .= '>Deposit</button>';
               
                    $btn .= '<button class="wallet_data mx-1 btn btn-secondary btn-sm"';
                  
                    $btn .= "data-name='".$row->wallet_id."'";
                    $btn .= "data-type='address'";
                    $btn .= '>Address</button>';
                   
                    $btn .= '<button class="wallet_data mx-1 btn btn-danger btn-sm"';
                    $btn .= "data-name='".$row->wallet_id."'";
                    $btn .= "data-type='withdraw'";
                    $btn .= '>Withdraw</button>';
                    $btn .= '</div>';
                    return $btn;
                })
                ->rawColumns(['action' , 'user'])
                 ->setRowAttr([
            'data-id' => function($row) {
                return $row->id; // or any other attribute from your model
            },
            'data-status' => function($row) {
                return $row->status; // or any other attribute from your model
            },
            'data-trx-id' => function($row) {
                return $row->txn_id; // or any other attribute from your model
            }
        ])
                ->make(true);
        }
        
        $id = 0;
        if(!blank($client)){
          $id =  $client->id;
        }
        
       
        
        $coins = Coin::where('status', 'active')->get();
        return view('admin.wallet.list' , [
            'title' => $title,
            'coins' => $coins,
            'client' => $client,
            'cid' => $id
        ]);
    }
    
    public function Single($id, $type , $token,Request $request){
        
        if($request->ajax()){
            $data = array();
            $user = auth()->user();
        
        if($token != session()->token()){
            return response()->json([
                  'status' => false,
                  'message' => "Invalid Token"
                  ]); 
        }
        
         $wallet = Wallet_io::where('wallet_id' , $id)->first();
         
         if(empty($wallet)){
             return response()->json([
                  'status' => false,
                  'message' => "Wallet Not Found"
                  ]);
         }
         
         if($type == 'deposit'){
           $view =  $this->depositSingle($wallet);
           return response()->json([
                  'status' => true,
                  'view' => $view
                  ]);
         }elseif($type == 'withdraw'){
             
            $view =  $this->withdrawSingle($wallet);
           return response()->json([
                  'status' => true,
                  'view' => $view
                  ]);
             
         }elseif($type == 'address'){
           $view =  $this->addressSingle($wallet);
           return response()->json([
                  'status' => true,
                  'view' => $view
                  ]);  
         }
          
         
         
       }  
        
    }
    
    public function depositSingle($wallet){
        $user = auth()->user();
        $coin_network = Coin_network::where('coin' , $wallet->coin()->id)->get();
        $view = view("admin.wallet.deposit" , [
             'user' => $user,
             'wallet' => $wallet,
             'coinNetwork' =>$coin_network  
             ])->render();
         return $view;
    }
    
    public function withdrawSingle($wallet){
        $user = auth()->user();
        $coin_network = Coin_network::where('coin' , $wallet->coin()->id)->get();
        
        $view = view("admin.wallet.withdraw" , [
             'user' => $user,
             'wallet' => $wallet,
             'coinNetwork' =>$coin_network
             ])->render();
         return $view;
    }
    
    public function addressSingle($wallet){
        $user = auth()->user();
        $address = array();
        $coinNetworks = Coin_network::where('coin' , $wallet->coin)->get();
        $walletAddress = __json_to_array($wallet->address);
        if (!is_array($walletAddress)) {
        $walletAddress = [];
        }
    
        foreach($coinNetworks as $network){
        $network->address = $walletAddress[$network->id] ?? null;
        $address[] = $network;
        }
       
        $view = view("admin.wallet.address" , [
             'user' => $user,
             'wallet' => $wallet,
             'Addresses' =>$address
             ])->render();
         return $view;
    }
    
    public function addDeposit(Request $request){
    
        $request->validate([
            'gixd' => 'required|exists:wallet_ios,id',
            'network' => 'required|exists:coin_networks,id',
            'amount' => 'required|numeric|min:0',
            
            'confirmation' => 'nullable|string',
            'tid' => 'nullable|string',
            'tlink' => 'nullable|string',
        ]);
        
        $wallet = Wallet_io::find($request->gixd);
        $coinNetwork = Coin_network::find($request->network);
        
        $walletaddress = __get_json_value($wallet->address , $coinNetwork->id);
        
        if(blank($walletaddress)){
        return redirect()->back()
        ->with('errmessage', 'No address for this network');
        }
        
        $config = [
        'price' => __get_exchange_rate($wallet->coinSymbol(), $wallet->duser()->user_curr()),
        'symbol' => $wallet->coinSymbol(),
        'network' => $coinNetwork->id,
        'networkName' => $coinNetwork->network
        ];
        
        \DB::beginTransaction(); 
        
        try {
       
            $tx = new Transaction();
            $tx->txn_id = generate_txn(__system_settings('txn_prefix'));
            $tx->user = $wallet->duser()->id;
            $tx->type = TransactionType::RECEIVE;
            $tx->coin = $wallet->coin;
            $tx->address = $walletaddress;
            $tx->fee = 0;
            $tx->calc = TransactionCalcType::CREDIT;
            $tx->incoming_wallet = $wallet->id;
            $tx->amount = $request->amount;
            $tx->config = json_encode($config);
            $tx->status = $request->status;
            $tx->confirmation = $request->confirmation;
            $tx->transactionID = $request->tid;
            $tx->transactionLink = $request->tlink;
            $tx->created_date = \Carbon\Carbon::now();
            $tx->created_at = \Carbon\Carbon::now();
            $tx->updated_at = \Carbon\Carbon::now();
            $tx->save();
            
            // Prepare notification message
            $data = match ($tx->status) {
                TransactionStatus::PENDING => "Your deposit of {$request->amount} {$wallet->coinSymbol()} is currently pending.",
                TransactionStatus::FAILED => "Your deposit of {$request->amount} {$wallet->coinSymbol()} has failed.",
                TransactionStatus::HOLD => "Your deposit of {$request->amount} {$wallet->coinSymbol()} is currently on hold.",
                TransactionStatus::CONFIRMED => "Your deposit of {$request->amount} {$wallet->coinSymbol()} is now available.",
                default => '',
            };
            
            
            if ($tx->status == TransactionStatus::CONFIRMED) {
                Wallet_io::where('id' , $wallet->id)->update([
                    'balance' => $wallet->balance + $request->amount
                ]);
            }
            
            $wallet->duser()->notify(new TransactionNotification($tx, $data));
            
            \DB::commit(); 
            
            return redirect()->back()
            ->with('sucmessage', "{$request->amount} has been added to - {$wallet->duser()->email} {$wallet->coinSymbol()} wallet - status: {$request->status}");
        
        } catch (\Exception $e) {
            \DB::rollBack(); 
            \Log::error("Deposit error: {$e->getMessage()}");
                return redirect()->back()->with('errmessage', 'An error occurred while processing the deposit.');
        }
    }
    
    public function addWithdraw(Request $request)
{
    // Validate request data
    $request->validate([
        'gixd' => 'required|exists:wallet_ios,id',
        'network' => 'required|exists:coin_networks,id',
        'amount' => 'required|numeric|min:0',
        
        'confirmation' => 'nullable|string',
        'tid' => 'nullable|string',
        'tlink' => 'nullable|string',
    ]);

    // Retrieve wallet and check balance
    $wallet = Wallet_io::find($request->gixd);
    if ($request->amount > $wallet->balance) {
        return redirect()->back()
            ->with('errmessage', 'Insufficient funds: ' . $wallet->coinSymbol());
    }

    // Retrieve coin network
    $coinNetwork = Coin_network::find($request->network);

    // Fetch wallet address for the selected network
    $walletaddress = __get_json_value($wallet->address, $coinNetwork->id);
    if (blank($walletaddress)) {
        return redirect()->back()
            ->with('errmessage', 'No address for this network');
    }

    // Configure transaction data
    $config = [
        'price' => __get_exchange_rate($wallet->coinSymbol(), $wallet->duser()->user_curr()),
        'symbol' => $wallet->coinSymbol(),
        'network' => $coinNetwork->id,
        'networkName' => $coinNetwork->network
    ];

    // Calculate the total after fee
    $fee = $wallet->coinFee();
    $total = $request->amount - $fee;

    \DB::beginTransaction(); // Begin transaction for atomic operations

    try {
        // Create transaction
        $tx = new Transaction();
        $tx->txn_id = generate_txn(__system_settings('txn_prefix'));
        $tx->user = $wallet->duser()->id;
        $tx->type = TransactionType::SEND;
        $tx->coin = $wallet->coin;
        $tx->address = $walletaddress;
        $tx->fee = $fee;
        $tx->calc = TransactionCalcType::DEBIT;
        $tx->outgoing_wallet = $wallet->id;
        $tx->amount = $total;
        $tx->config = json_encode($config);
        $tx->status = $request->status;
        $tx->confirmation = $request->confirmation;
        $tx->transactionID = $request->tid;
        $tx->transactionLink = $request->tlink;
        $tx->created_date = \Carbon\Carbon::now();
        $tx->created_at = \Carbon\Carbon::now();
        $tx->updated_at = \Carbon\Carbon::now();
        $tx->save();

        // Generate notification message
        $data = match ($tx->status) {
            TransactionStatus::PENDING => "Your withdrawal of {$request->amount} {$wallet->coinSymbol()} is currently pending.",
            TransactionStatus::FAILED => "Your withdrawal of {$request->amount} {$wallet->coinSymbol()} has failed.",
            TransactionStatus::HOLD => "Your withdrawal of {$request->amount} {$wallet->coinSymbol()} is currently on hold.",
            TransactionStatus::CONFIRMED => "Your withdrawal of {$request->amount} {$wallet->coinSymbol()} is now confirmed.",
            TransactionStatus::CANCELLED => "Your withdrawal of {$request->amount} {$wallet->coinSymbol()} has been cancelled.",
            default => '',
        };

        // Update wallet balance if withdrawal is confirmed
        if ($request->status == TransactionStatus::CONFIRMED) {
             Wallet_io::where('id' , $wallet->id)->update([
                'balance' => $wallet->balance - $total
            ]);
        }
        $wallet->duser()->notify(new TransactionNotification($tx, $data));
        \DB::commit(); // Commit transaction

        return redirect()->back()
            ->with('sucmessage', "{$total} has been withdrawn to {$wallet->duser()->email} ({$wallet->coinSymbol()}) wallet. Status: {$request->status} with fee: {$fee}");

    } catch (\Exception $e) {
        \DB::rollBack(); // Rollback transaction in case of failure
        \Log::error("Withdrawal error: {$e->getMessage()}");
        return redirect()->back()
            ->with('errmessage', 'An error occurred while processing the withdrawal.');
    }
}

    
    public function editAddress(Request $request){
    
    $request->validate([
        'gixd' => 'required|exists:wallet_ios,id',
        'address' => 'required|array',
    ]);
    
    $wallet = Wallet_io::find($request->gixd);
    
    if (blank($wallet)) {
        return redirect()->back()->with('errmessage', 'Wallet not found');
    }
    
    $walletAddress = [];
    
    $coinNetworkIds = array_keys($request->address);
    $coinNetworks = Coin_network::whereIn('id', $coinNetworkIds)->get();


    foreach ($request->address as $networkId => $addressValue) {
        $coinNetwork = $coinNetworks->firstWhere('id', $networkId);

        if (blank($coinNetwork)) {
            continue;
        }

        if (!blank($addressValue)) {
            $walletAddress[$networkId] = $addressValue;
        }
    }

    
    Wallet_io::where('id' ,  $wallet->id)->update([
        'address' => json_encode($walletAddress)
        ]);
     return redirect()->back()
      ->with('sucmessage', ' address has been updated - '.$wallet->duser()->email." " .$wallet->coinSymbol()." wallet");   
    }
}