<?php

namespace App\Http\Controllers\Auth;

use App\Http\Controllers\Controller;

use App\Enums\UserRoles;
use App\Models\User;
use App\Services\AuthService;

use App\Http\Requests\Auth\LoginRequest;
use Illuminate\Http\RedirectResponse;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\Auth;
use Illuminate\Validation\ValidationException;
use Illuminate\View\View;
use DB;

class AuthenticatedSessionController extends Controller
{
    public function  __construct(AuthService $authService){
        $this->auth = $authService;
    }
    
    /**
     * Display the login view.
     */
    public function create(): View
    {
        return view('auth.login');
    }

    /**
     * Handle an incoming authentication request.
     */
    public function store(LoginRequest $request): RedirectResponse
    {
       
       
        $request->authenticate();
        
        $user = auth()->user();
        $request->session()->regenerate();
        
        $this->auth->saveLastLogin();
        return $this->redirectUser($user);
        //return redirect()->intended(route('dashboard', absolute: false));
    }

    /**
     * Destroy an authenticated session.
     */
    public function destroy(Request $request): RedirectResponse
    {
        Auth::guard('web')->logout();

        $request->session()->invalidate();

        $request->session()->regenerateToken();

        return redirect('/');
    }
    
    
    private function redirectUser(User $user)
    {
        $welcome = false;

        /*if(empty($user->meta('first_login_at'))) {
            $user->user_metas()->create(['meta_key' => 'first_login_at', 'meta_value' => now()]);
            $welcome = true;
        }

        if (!system_admin_setup() && $user->role == UserRoles::SUPER_ADMIN) {
            $welcome = true;
        }

        if (in_array($user->role, [UserRoles::ADMIN, UserRoles::SUPER_ADMIN])) {
            if ($welcome && $user->role == UserRoles::SUPER_ADMIN) {
                return redirect()->route('admin.quick-setup');
            }
            return redirect()->route('admin.dashboard');
        }

        if ($user->role === UserRoles::USER) {
            if ($welcome) {
                return redirect()->route('account.welcome');
            }
            return redirect()->route('dashboard');
        }*/
        
        if (in_array($user->role, [UserRoles::SUPER_ADMIN])) {
          
            return redirect()->intended(route('superadmin.dashboard', absolute: false));
        }
        if (in_array($user->role, [UserRoles::ADMIN])) {
           
           return redirect()->intended(route('admin.dashboard', absolute: false));
        }
        
        
       return redirect()->intended(route('user.dashboard', absolute: false));
        
    }
}
