<?php

namespace App\Http\Controllers\Auth;

use App\Http\Controllers\Controller;

use App\Enums\UserRoles;
use App\Models\User;
use App\Models\AccountName;
use App\Services\AuthService;
use App\Jobs\ProcessCustom;
use App\Jobs\ProcessEmail;
use Illuminate\Auth\Events\Registered;
use Illuminate\Http\RedirectResponse;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\Auth;
use Illuminate\Support\Facades\Hash;
use Illuminate\Validation\Rules;
use Illuminate\Validation\ValidationException;
use Illuminate\View\View;
use DB;
use Str;
use Session;

class RegisteredUserController extends Controller
{
    /**
     * Display the registration view.
     */
    public function  __construct(AuthService $authService){
        $this->auth = $authService;
    
    }
    
    
    public function create(): View 
    {
        $accounts = AccountName::where('status' , 'active')->get();
        return view('auth.account' , compact('accounts'));
    }
    
    public function registerForm($slug , $id): View 
    {
        $account = AccountName::whereSlug($slug)->findorFail(__get_hash($id));
        return view('auth.register' , compact('account'));
    }
    


public function codeRegister(Request $request): RedirectResponse
    {
        $request->validate([
            'email' => ['required', 'string', 'lowercase', 'email', 'max:255', 'unique:'.User::class],
        ],[
            'email.required' => 'Email is required',
            'email.email' => 'Please enter a valid email',
            'email.unique' => 'An account already exists under this email'
            ]);
            
        
        try{
          $emailcode = Str::random(6);
          $email = $request->email;
          $data = [
            'email' => $request->email,
            'code' => $emailcode,
          ];
          ProcessCustom::dispatchSync('users-verify-email-code',$email, $data);
          //ProcessEmail::dispatchSync('users-verify-email-code', $data);
          $request->session()->put(['reg_code' => $emailcode , 'reg_email' => $request->email  ]);
          return redirect()->route('auth.register.code');
        }catch(\Exception $e){
            \Log::error("Error occured during registration " .$e->getMessage());
            dd($e);
            throw ValidationException::withMessages([
                'email' => __('Sorry, due to technical issues we unable to proceed. Please try again after sometimes or contact us.')
            ]);
        }
        
    }
    public function store(Request $request): RedirectResponse
    {
        
        $request->validate([
            'firstname' => ['required', 'string', 'max:255'],
            'surname' => ['required', 'string', 'max:255'],
            'gender' => ['required', 'string', 'in:male,female,other'],
            'day' => ['required', 'integer', 'min:1', 'max:31'],
            'month' => ['required', 'integer', 'min:1', 'max:12'],
            'year' => ['required', 'integer', 'min:1900', 'max:' . date('Y')],
            'country' => ['required', 'string', 'max:255'],
            'email' => ['required', 'string', 'email', 'max:255', 'unique:users,email'],
            'password' => ['required', 'string', 'min:8', 'same:cpassword'],
            'account' => ['required']
        ], [
            'firstname.required' => 'First name is required.',
            'surname.required' => 'Surname is required.',
            'gender.required' => 'Gender is required.',
            'gender.in' => 'Gender must be male, female, or other.',
            'day.required' => 'Day of birth is required.',
            'day.integer' => 'Day must be a valid number.',
            'month.required' => 'Month of birth is required.',
            'month.integer' => 'Month must be a valid number.',
            'year.required' => 'Year of birth is required.',
            'year.integer' => 'Year must be a valid number.',
            'year.min' => 'Year must be later than 1900.',
            'year.max' => 'Year cannot be in the future.',
            'country.required' => 'Country is required.',
            'email.required' => 'Email is required.',
            'email.email' => 'Enter a valid email address.',
            'email.unique' => 'This email is already registered.',
            'password.required' => 'Password is required.',
            'password.min' => 'Password must be at least 8 characters.',
            'password.confirmed' => 'Passwords do not match.',
            'account.required' => 'Account is required.',
        ]);

        
        DB::beginTransaction();
        try {
            $account = AccountName::where('id' , __get_hash($request->account))->first();
            if(blank($account)){
                throw ValidationException::withMessages([
                    'account' => 'invalid account selection'
                    ]);
            }
            $data = $request->all();
            
            
            $user = $this->auth->createUser($data , $account);
          
            if (!$user) {
                throw ValidationException::withMessages([
                    'email' => __('An error occurred during registration, please try again later. If the issues continues, contact us.')
                ]);
            }

            //ProcessEmail::dispatchSync('users-welcome-email', $user);
           
           $request->session()->forget(['refers','reg_proceed','reg_email']);
           $request->session()->put(['reg_complete' => 'yes' , 'reg_user' => $user]);
           
           event(new Registered($user));
            DB::commit();
            if (in_array($user->role, [UserRoles::ADMIN, UserRoles::SUPER_ADMIN])) {
                return redirect()->route('admin.login.form');
            }
            
            return redirect()->route('auth.register.complete')->with('reg_msg' , 'You have been registered successfully. Check your email to activate your account');

        } catch (\Exception $e) {
            DB::rollBack();
            dd($e);
            \Log::error('Error occuring during registration details '.$e->getMessage());
            throw ValidationException::withMessages([
                'email' => __('Sorry, due to technical issues we unable to proceed. Please try again after sometimes or contact us.')
            ]);
        }

        

        
    }
  
  public function complete(): View | RedirectResponse
    {
        if(!Session::has(['reg_complete','reg_user'])){
            return redirect()->route('auth.register');
        }
        $user = Session::get('reg_user');
        Session::forget(['reg_user','reg_complete']);
        return view('auth.complete' , compact('user'));
    }

}
