<?php

namespace App\Http\Controllers\User;

use App\Http\Controllers\Controller as BaseController;

use App\Models\Notification as UserNotification;
use App\Models\AccountIo;
use App\Models\PaymentMethod;
use App\Models\Transaction;
use App\Models\User;
use App\Models\WithdrawalMethod;

use App\Jobs\ProcessEmail;
use App\Notifications\TransactionNotification;

use App\Enums\NotificationType;

use App\Enums\TransactionCalcType;
use App\Enums\TransactionStatus;
use App\Enums\TransactionType;

use App\Services\AuthService;
use App\Services\FundsService;


use Illuminate\Http\RedirectResponse;
use Illuminate\Http\Request;
use Illuminate\Http\Response;
use Illuminate\Support\Facades\Auth;
use Illuminate\Support\Facades\Hash;
use Illuminate\Support\Facades\Log;
use Illuminate\Support\Facades\Redirect;
use Illuminate\Support\Str;
use Illuminate\Validation\ValidationException;
use Illuminate\View\View;
use Illuminate\Support\Facades\Validator;

use Illuminate\Support\Facades\Notification;



use Arr;
use Carbon\Carbon;
use DB;
use Session;



class FundsController extends BaseController{
     
    function __construct(
        AuthService $authService,
        FundsService $fundService
    ){
        $this->authService = $authService;
        $this->fundService = $fundService;
    }
    
    /****
     * 
     * **/
    public function deposits(): View{
        $title = "Deposit";
        $user = auth()->user();
        if(__system_settings('allow_deposit') === 'yes'){
            
          $accounts = AccountIo::where('user' , $user->id)->get();
          $selectedAccount = "";
          if(!blank($accounts)){
             $selectedAccount = $accounts->where('id' , $user->account_id)->first();
          }
          
          
          $payment_methods = PaymentMethod::where('status' , 'active')->get();
        
        return view('user.funds.deposits' , [
            'title' => $title,
            'user' => $user,
            'accounts' => $accounts,
            'pmethods' => $payment_methods,
            'selectedAccount' => $selectedAccount
            
             ]); 
        }
        
        return view('user.funds.disableDeposits' , [
                'title' => $title,
                'user' => $user,
                
                ]);
        
    }
    
    /****
     * 
     * **/
    public function  deposit($id){
      $title = 'deposit';
      $user = auth()->user();
      $transaction = Transaction::where('txnID' , __get_hash($id))->where('user' , $user->id)->firstorFail();
      
      if(blank($transaction->depositMethod)){
         abort(404); 
      }
      
      $date2 = Carbon::now();
  
      return view('user.funds.deposit' , [
          'title' => 'Deposits',
          'user' => $user,
          'transaction' => $transaction,
          'depositMethod' => $transaction->depositMethod,
          'timeLeft' => $date2->diffInSeconds($transaction->depositEndDate())
          ]);
    }
    
    public function  checkdeposit(Request $request){
        
         $user = auth()->user();
        $data = array();
        
        $res = $this->fundService->check_deposits($request);
     
        if($res['status'] == true){
    
        $usdamt = 1 * $request['amt'];
        $deposit_id = random_string('md5');
        
        $views = view('user.funds.components.depositsPreview' , [
          'pmethod' => $res['payment'],
          'account' => $res['account'],
          'user' => $user,
          'amount' => $request['amt'],
          
          'deposit_id' => $deposit_id,
          ]);  
          
        $data['status'] = true;
        $data['view'] = $views->render();
       
       $request->session()->put('account' , $res['account']);
        $request->session()->put('pmethod' , $res['payment']);
        $request->session()->put('amount' , $request['amt']);
        $request->session()->put('deposit_id' , $deposit_id);
    
        
        
        }else{
            $data = $res;
            
        }
        
       echo json_encode($data);
        
       
    }
    
     public function confirmDeposit($id){
        $user = auth()->user();
        $deposit_id = session()->get('deposit_id');
       
      
        
        if(empty($id) || $id != $deposit_id || !session()->has(['pmethod','account', 'amount'])){
            return redirect()->route('user.funds.deposits')->with('errmessage' , 'Invalid deposit token');
        }
     
        try{
        DB::beginTransaction();
        $pmethod = session()->get('pmethod');
        $account = session()->get('account');
        $data = array();
        $data['method'] = $pmethod->id;
        $data['accountId'] =  $account->id;
        $data['amount'] = session()->get('amount');
        $data['payment'] = $pmethod;
        $data['account'] = $account;
        $data['trans_id'] = random_string('capalnum', 8);
        $data['type'] = TransactionType::DEPOSIT;
        $data['calc'] = TransactionCalcType::CREDIT;
        $data['descr'] = "External Deposit via ". $pmethod->name;
        $data['status'] = TransactionStatus::NOT_PAID;
        
        
        $dep_id = $this->fundService->save_transaction($data);
        
        try {
            
                //ProcessEmail::dispatch('deposit-placed-customer', $user, null, $depss, $feeInfo);
                //ProcessEmail::dispatch('deposit-placed-admin', $user, null, $depss);
        } catch (\Exception $e) {
            DB::rollback();
         Log::error('Email dispatch failed: ' . $e->getMessage());       
        return redirect()->route('user.funds.deposits')->with('errmessage' , 'Payment error occurred');  
        }
        session()->forget(['pmethod','wallet_d','account_type','account','amount','deposit_id']);
        DB::commit();
        return redirect()->route('user.funds.deposit' , __the_hash($data['trans_id']));
        }catch(\Exception $e){
            DB::rollback(); 
            Log::error('Deposit Failed: ' .$user->email." - ". $e->getMessage()); 
           return redirect()->route('user.funds.deposits')->with('errmessage' , 'Payment error occurred');  
        }

    }
    
    public function  cancelDeposit(){ 
       session()->forget(['pmethod' , 'wallet_d', 'wallet_d', 'amount', 'deposit_id']); 
       return redirect()->route('user.funds.deposits');
     }
    
    /****
     * 
     * **/
    public function withdrawals(): View{
        $title = "Withdrawals";
        $user = auth()->user();
 
        if(__system_settings('allow_withdrawal') === 'yes'){
            
            $accounts = AccountIo::where('user' , $user->id)->get();
          $selectedAccount = "";
          if(!blank($accounts)){
             $selectedAccount = $accounts->where('id' , $user->account_id)->first();
          }
          
            $withdrawal_methods = WithdrawalMethod::where('status' , 'active')->get();
            return view('user.funds.withdrawals' , [
            'title' => $title,
            'user' => $user,
            'wmethods' => $withdrawal_methods,
            'accounts' => $accounts,
            'selectedAccount' => $selectedAccount
            ]);
        }
       
        
        
        return view('user.funds.disableWithdrawal' , [
                'title' => $title, 
                'user' => $user,
                
                ]);
    }
    
    public function withdrawal($id = ''){
        
     
        if(empty($id) || !session()->has('withdrawal_done')){
            return redirect()->route('user.funds.withdrawals')->with('errmessage' , 'No withdrawal ID');
        }

        $user = auth()->user();
        $transaction = Transaction::where('txnID' , __get_hash($id))->where('user' , $user->id)->firstorFail();
        if(blank($transaction->withdrawalMethod)){
         abort(404); 
      }
      
       session()->forget('withdrawal_done');
       return view('user.funds.withdrawal' , [
          'title' => 'Withdrawal',
          'user' => $user,
          'transaction' => $transaction,
          'withdrawalMethod' => $transaction->withdrawalMethod
          ]);  
    }
    
    public function  startWithdrawal(Request $request){
        
        $user = auth()->user();
        $data = $this->fundService->start_withdrawal($request);
     
        if($data['status'] == true){
            $nd = array();
            $nd['status'] = true;
            
           
            $views = view('user.funds.components.withdrawalStep' , [
          'user' => $user,
          'wmethods' => $data['method'],
          'account_type' => $data['account_type'],
          'account_send' => $data['result'],
        
              ]);  
         $nd['view'] = $views->render();
        
        $request->session()->put('account_type' , $data['account_type']);
        $request->session()->put('account_send' , $data['result']);
      
        $request->session()->put('with_method' , $data['method']);
        
        echo json_encode($nd);
        
        }else{
            echo json_encode($data);
        }
    }
    
    public function checkWithdrawal(Request $request){
        $user = auth()->user();
       $data = $this->fundService->checkWithdraw($request);
 
       if($data['status'] == true){
          
            $nd = array();
            $nd['status'] = true;
            $withdrawal_id = random_string('md5');
            $views = view('user.funds.components.withdrawPreview' , [
            'user' => $user,
            'amount' => $request['amt'],
            'wmethod'=> $data['method'],
            'account'=> $data['account'],
            'withdrawal_id' => $withdrawal_id,
            ]);  
            $nd['view'] = $views->render();
   
            $request->session()->put('wmethod' , $data['method']);
            $request->session()->put('account' , $data['account']);
            $request->session()->put('withdrawal_id' , $withdrawal_id);
            $request->session()->put('amount' , $request['amt']);
            echo json_encode($nd);
       }else{
            
            echo json_encode($data);
       }
       
       
       
    }
    
    public function confirmWithdrawal(Request $request , $id = ''){
        $user = auth()->user();
       $withdrawal_id = session()->get('withdrawal_id');
      
        if(empty($id) || $id != $withdrawal_id || !session()->has(['wmethod', 'amount','account'])){
            return redirect()->route('user.funds.withdrawals')->with('errmessage' , 'Invalid withdrawal token');
        } 
        
        try{
        DB::beginTransaction();
        $wmethod = session()->get('wmethod');
        $account = session()->get('account');
        $amount = session()->get('amount');
        
        $data = array();
        $data['method'] = $wmethod->id;
        $data['amount'] = $amount;
        $data['payment'] = $wmethod;
        $data['accountId'] =  $account->id;
        $data['trans_id'] = random_string('capalnum', 8);
        $data['type'] = TransactionType::WITHDRAW;
        $data['calc'] = TransactionCalcType::DEBIT;
        $data['descr'] = "External withdrawal to ". $wmethod->name;
        $data['status'] = TransactionStatus::PENDING;
        $data['details'] = $request->address;
        $data['account'] = $account;
        
        
        $dep_id = $this->fundService->save_transaction($data);
        $account->decrement('balance', $amount);
            __balance_history([
                'amount' => $amount,
                'user' => $user->id
            ], 'sub');
        
        try {
            //ProcessEmail::dispatch('withdraw-request-customer', $user, null, $depss, $feeInfo);
            //ProcessEmail::dispatch('withdraw-request-admin', $user, null, $depss); 
               
        } catch (\Exception $e) {
         DB::rollback();
         Log::error('Email dispatch failed: ' . $e->getMessage());       
        return redirect()->route('user.funds.withdrawals')->with('errmessage' , 'Technical error occured');  
        }
        
        session()->forget(['account','type','amount','account','wmethod','details']);
        session()->put('withdrawal_done' , 'yes');
        DB::commit();
        return redirect()->route('user.funds.withdrawal' , __the_hash($data['trans_id']));
        }catch(\Exception $e){
            DB::rollback(); 
            Log::error('Withdrawal Failed: ' .$user->email." - ". $e->getMessage()); 
           return redirect()->route('user.funds.withdrawals')->with('errmessage' , 'Withdrawal Failed , Try again later');  
        }
    }
    
    
    
    /****
     * 
     * **/
    public function  history(Request $request){
      $title = 'Funds History';
      $user = auth()->user();
      $transaction = $this->fundService->get_transaction($request);

     $txnTypes = __get_enums(TransactionType::class, false);
     
      return view('user.funds.history' , [
          'title' => 'Deposits',
          'user' => $user,
          'type' => $transaction['type'],
          'trans' => $transaction['transactions'],
          'txnTypes' => $txnTypes
          ] );
    }
    
    public function cancelTransactionMade($id = '')
{
    if (empty($id)) {
        return redirect()->back()->with('errmessage', 'Transaction ID is required.');
    }

    $hashedId = __get_hash($id);
    $data = $this->fundService->cancel_transaction($hashedId);
    
    if ($data['status'] ?? false) {
        session()->put('cancelled_Deposit', 'cancelled');
        return redirect()->route('user.funds.cancel.view', __the_hash($hashedId));
    }

    return redirect()->route('user.funds.history')->with('errmessage', $data['error'] ?? 'An error occurred during the cancellation.');
}

    public function cancelTransactionView($id = '')
{    
    $user = auth()->user();
    if (empty($id)) {
        return redirect()->route('user.funds.history')->with('errmessage', 'Transaction ID is required.');
    }
    
     $transaction = Transaction::where('txnID' , __get_hash($id))->where('user' ,$user->id)->first();
    if(blank($transaction)){
        return redirect()->route('user.funds.history')->with('errmessage', 'Invalid Transaction');
    };
    
    Transaction::where('id' , $transaction->id)->update([
            'status' => TransactionStatus::CANCELLED
            ]);
            
    return view('user.funds.transactionCancel' , compact('user' , 'transaction'));
   
}


public function saveDeposit(Request $request, Transaction $transaction)
{
    // Initialize validation rules and custom messages
    $rules = [];
    $messages = [];
    $user = auth()->user();
    $transaction = Transaction::where('id' , __get_hash($request->transact_hash))->where('txnID' , __get_hash($request->transact_id))->where('user' ,$user->id)->first();
    if(blank($transaction)){
        return redirect()->back()->with('errmessage', 'Invalid Transaction');
    };
    
    $proof = '';
    if ($request->has('reference')) {
        // Determine if the reference is crypto or bank transfer
        if (is_string($request->reference)) {
            // Crypto-specific validation
            $rules = [
                'reference' => 'required|string|min:10|max:64',
            ];
            $messages = [
                'reference.required' => 'The crypto reference is required.',
                'reference.string' => 'The crypto reference must be a valid string.',
                'reference.min' => 'The crypto reference must be at least 10 characters.',
                'reference.max' => 'The crypto reference must not exceed 64 characters.',
            ];

        } elseif ($request->hasFile('reference')) {
            // Bank transfer file validation
            $rules = [
                'reference' => 'required|file|mimes:jpeg,png,jpg,gif|max:2048',
            ];
            $messages = [
                'reference.required' => 'The bank transfer file is required.',
                'reference.file' => 'The uploaded file must be a valid file.',
                'reference.mimes' => 'Only image files (jpeg, png, jpg, gif) are allowed.',
                'reference.max' => 'The file size must not exceed 2MB.',
            ];

        } else {
            return redirect()->back()->with('errmessage', 'An error occurred during the update.');
        }

        // Validate the request
        $validator = Validator::make($request->all(), $rules, $messages);
        if ($validator->fails()) {
          
            return redirect()->back()->with('errors', $validator->errors());
        }

        if (is_string($request->reference)) {
    
            $proof = $request->reference;
        } elseif ($request->hasFile('reference')) {
            // Handle file upload for bank transfer using move and renaming the file
            $file = $request->file('reference');
            $newFileName = Str::uuid() . '.' . $file->getClientOriginalExtension();
            $destinationPath = public_path('uploads/bank_transfers');

            if (!file_exists($destinationPath)) {
                mkdir($destinationPath, 0755, true);
            }

            $file->move($destinationPath, $newFileName);
            $transaction->proof = $newFileName;
        }

        Transaction::where('id' , $transaction->id)->update([
            'proof' => $proof,
            'status' => TransactionStatus::PENDING
            ]);

        return redirect()->route('user.funds.history' , ['type' => 'deposit'])->with('sucmessage', 'Proof uploaded successfully. Wait for system to confirm');;
    }

    
}

/**
 * Determine if the reference string indicates a crypto reference.
 *
 * @param string $reference
 * @return bool
 */
private function isCryptoReference(string $reference): bool
{
    // Simple check for crypto reference (adjust logic as needed)
    return preg_match('/^[a-zA-Z0-9]{10,64}$/', $reference) === 1;
}



    
 
}