<?php

namespace App\Http\Controllers\User;

use App\Http\Controllers\User\UserController as BaseController;

use App\Models\Notification as UserNotification;
use App\Models\Plan;
use App\Models\Transaction;
use App\Models\User;
use App\Models\UserPlan;
use App\Models\AccountName;
use App\Models\AccountIo;


use App\Jobs\ProcessEmail;
use App\Notifications\TransactionNotification;
use App\Enums\NotificationType;
use App\Enums\TransactionCalcType;
use App\Enums\TransactionStatus;
use App\Enums\TransactionType;


use Illuminate\Http\RedirectResponse;
use Illuminate\Http\Request;
use Illuminate\Http\Response;
use Illuminate\Support\Facades\Auth;
use Illuminate\Support\Facades\Hash;
use Illuminate\Support\Facades\Log;
use Illuminate\Support\Facades\Redirect;
use Illuminate\Support\Str;
use Illuminate\Validation\ValidationException;
use Illuminate\View\View;

use Illuminate\Support\Facades\Notification;



use Arr;
use Carbon\Carbon;
use DB;
use Session;



class PlansController extends BaseController{
     
 
    
    /****
     * 
     * **/
    public function index($id , Request $request): View
{
    $title = "Plans";
    $user = auth()->user();
    $request->session()->forget(['plan','account']);
    $accountId = __get_hash($id);
    
    if (!$accountId) {
        abort(404, 'Invalid account ID');
    }
    
    $account = AccountIo::where('user' ,  $user->id)->findOrFail($accountId);
    
    $plans = Plan::where('account', $account->account)->get();
    if ($plans->isEmpty()) {
        abort(404, 'No plans found for this account');
    }

    return view('user.plans.plan', [
        'title' => $title,
        'user' => $user,
        'mainplans' => $plans,
        'account' => $account,
        'promoplans' => Plan::getPromoPlan(),
    ]);
}

    public function startPlan(Request $request){

        $request->validate([
            'plan' => 'required',
            'account' => 'required'
            ],[
                'plan.required' => 'Please select a plan',
                'account.required' => 'Please select an account'
            ]);
        
        $plan = $this->fundService->start_plan($request->all());
        if($plan['status'] === true){
        
        $request->session()->put(['plan' => $plan['plan'],
        'account' => $plan['account']
        
        ]);
        return redirect()->route('user.plans.invest');  
        }else{
            if(!empty($plan['error'])){
                return redirect()->back()->with('errmessage' ,$plan['error']);
            };
        }
        return redirect()->back()->with('errmessage' , 'An error occured, Please try again' );
    }
    
    public function payPlan(): View{
        if(!Session::has(['plan','account'])){
            return redirect()->route('user.plans.index');
        }
        $title = "Plans";
        $user = auth()->user();
        $plan = Session::get('plan');
        $account = Session::get('account');
        
        $dateNow = \Carbon\Carbon::now();
        $plan_duration = __plan_duration_seconds($plan->duration_time) * $plan->duration;
        $dateEnd = \Carbon\Carbon::now()->addSeconds($plan_duration);
        $profit = ($plan->percs / 100) * $plan->c_price();
        return view('user.plans.invest' , [
            'title' => $title,
            'user' => $user,
            'plan' => $plan,
            'account' => $account,
            'profit' =>$profit,
            'dateNow' => $dateNow,
            'dateEnd' => $dateEnd
        ]);
    }
    
    public function joinPlan(Request $request)
{
    // Validate input
    $request->validate([
        'amount' => 'required|numeric|min:1',
        'iv-plan' => 'required|string',
        'iv-account' => 'required|string',
    ], [
        'amount.required' => 'Please provide an investment amount.',
        'amount.numeric' => 'Investment amount must be a valid number.',
        'iv-plan.required' => 'Invalid plan selection.',
        'iv-account.required' => 'Invalid account selection.',
    ]);

    $user = auth()->user();
    $checkPlan = $this->fundService->check_plan($request);
   
    if ($checkPlan['status'] !== true) {
        return redirect()->back()->with('errmessage', $checkPlan['error']);
    }

    DB::beginTransaction();

    try {
        
        $planID = random_string('capalnum', 8);
        $checkPlan['amount'] = $request['amount'];
        $accounts = $checkPlan['account'];
        $checkPlan['trans_id'] = $planID;
        $planId = $this->fundService->save_plan($checkPlan);
        
        $transactionData = [
            'method' => $planId,
            'amount' => $request['amount'],
            'trans_id' => $planID,
            'type' => TransactionType::INVESTMENT,
            'calc' => TransactionCalcType::DEBIT,
            'descr' => "Investment on " . $checkPlan['plan']->name,
            'status' => TransactionStatus::CONFIRMED,
        ];
       
        $this->fundService->save_transaction($transactionData);
        $accounts = AccountIo::where('id' , __get_hash($request['iv-account']))->first();
        $accounts->decrement('balance', $request['amount']);
            __balance_history([
                'amount' => $request['amount'],
                'user' => $user->id
            ], 'sub');

        $invest = UserPlan::find($planId);
        //ProcessEmail::dispatch('investment-placed-customer', $user, null, $invest);
        //ProcessEmail::dispatch('investment-placed-admin', $user, null, $invest);
        
        DB::commit();
        $request->session()->forget(['plan','account']);
        return redirect()->route('user.plans.user')
            ->with('sucmessage', 'You successfully purchased a plan and your plan is now active.');
    } catch (\Exception $e) {
        DB::rollback();

        \Log::error('Plan join failed for user ' . $user->id, ['error' => $e->getMessage()]);

        return redirect()->back()->with('errmessage', 'Joining a plan failed. Please try again.');
    }
}

    
    /****
     * 
     * **/
    
    public function  userPlan(): View{
      $title = 'My Plans';
      $user = auth()->user();
     
      return view('user.plans.userplan' , [
          'title' => 'Plans',
          'user' => $user,
          'plans' => Plan::all(),
          'total_invested' => UserPlan::total_invested($user->id),
          'total_roi' => UserPlan::total_profit($user->id),
          'active_invested' => UserPlan::active_invested($user->id),
          'active_roi' => UserPlan::active_profit($user->id),
          'active_plans' => UserPlan::active_plans($user->id),
          'paused_plans' => UserPlan::paused_plans($user->id),
          'inactive_plans' => UserPlan::inactive_plans($user->id),
          'ended_plans' => UserPlan::ended_plans($user->id),
          ] );
    }
    
    public function  singlePlan($id){
      $title = 'My Plans';
      $user = auth()->user();
      $id = __get_hash($id);
      $uplan = UserPlan::where('user',$user->id)->where('txn_id' , $id)->first();
      
      if(empty($uplan)){
          return redirect()->route('user.plans.user');
      }
    
      return view('user.plans.single' , [
          'title' => 'Plans',
          'user' => $user,
          'myplan' => $uplan,
          'transacts'=> Transaction::where('user',$user->id)->where('method' , $uplan->id)->where('type' , 'roi')->get(),
          
          ] );
    }
    
    /****
     * 
     * **/
    public function upgradePlan(): View{
        $title = "Upgrade Plans";
        $user = auth()->user();
 
       
        return view('user.plans.upgrade' , [
            'title' => $title,
            'user' => $user,
            'mainplans' => Plan::getUpgradePlan($user->id),
        ]);
    }
    
    
    
 
}