<?php

namespace App\Http\Controllers\User;

use App\Http\Controllers\User\UserController as BaseController;

use App\Models\Notification as UserNotification;
use App\Models\Transaction;
use App\Models\User;
use App\Models\UserLogin;

use App\Jobs\ProcessEmail;
use App\Notifications\TransactionNotification;
use App\Enums\NotificationType;
use App\Enums\TransactionStatus;


use Illuminate\Http\RedirectResponse;
use Illuminate\Http\Request;
use Illuminate\Http\Response;
use Illuminate\Support\Facades\Auth;
use Illuminate\Support\Facades\Hash;
use Illuminate\Support\Facades\Log;
use Illuminate\Support\Facades\Redirect;
use Illuminate\Support\Str;
use Illuminate\Validation\ValidationException;
use Illuminate\View\View;

use Illuminate\Support\Facades\Notification;



use Arr;
use Carbon\Carbon;
use DB;
use Session;



class ProfileController extends BaseController{
     
    /****
     * 
     * **/
    public function index(): View{
        $title = "Profile";
        $user = auth()->user();
 
       
        return view('user.profile.profile' , [
            'title' => $title,
            'user' => $user,
        ]);
    }
    
    public function editProfile(Request $request){
         $data = array();
      
       
        if($request['name'] == '' || $request['name'] == NULL){
           return redirect()->back()->with('errmessage' , 'First name can not be empty');
        }
         
      
         if($request['profile_phone'] == '' || $request['profile_phone'] == NULL){
            return redirect()->back()->with('errmessage' , 'Phone Number can not be empty');
        }
        if($request['profile_gender'] == '' || $request['profile_gender'] == NULL){
             return redirect()->back()->with('errmessage' , 'Gender can not be empty');
        }
        if($request['profile_dob'] == '' || $request['profile_dob'] == NULL){
             return redirect()->back()->with('errmessage' , 'Date of birth can not be empty');
        }
        if($request['theme_style'] == '' || $request['theme_style'] == NULL){
             return redirect()->back()->with('errmessage' , 'Theme style can not be empty');
        }
      
        User::where('id' , auth()->user()->id)->update([
            'name' => $request['name'],
             'gender' => $request['profile_gender'],
              'dob' => $request['profile_dob'],
               'phone' => $request['profile_phone'],
                'theme_style' => $request['theme_style'],
            ]);
         
            return redirect()->back()->with('sucmessage' , 'User Details Updated');
    }
    
    public function editAddress(Request $request){
         $data = array();
     
        if($request['profile_address_line_1'] == '' || $request['profile_address_line_1'] == NULL){
           return redirect()->back()->with('errmessage' , 'Address can not be empty');
        }
         
         if($request['profile_city'] == '' || $request['profile_city'] == NULL){
            return redirect()->back()->with('errmessage' , 'City can not be empty');
        }
         if($request['profile_state'] == '' || $request['profile_state'] == NULL){
            return redirect()->back()->with('errmessage' , 'State can not be empty');
        }
        if($request['profile_zip'] == '' || $request['profile_zip'] == NULL){
             return redirect()->back()->with('errmessage' , 'Zip Code can not be empty');
        }
        if($request['profile_country'] == '' || $request['profile_country'] == NULL){
             return redirect()->back()->with('errmessage' , 'Country can not be empty');
        }
      
        User::where('id' , auth()->user()->id)->update([
            'address' => $request['profile_address_line_1'],
            'city' => $request['profile_city'],
             'state' => $request['profile_state'],
              'zipcode' => $request['profile_zip'],
               'country' => $request['profile_country'],
            
            ]);
         
            return redirect()->back()->with('sucmessage' , 'Address Details Updated');
    }
    
    
    /****
     * 
     * **/
    public function security(): View{
        $title = "Security";
        $user = auth()->user();
 
       
        return view('user.profile.security' , [
            'title' => $title,
            'user' => $user,
        ]);
    }
    
    
    public function updatePassword(Request $request){
        $user = auth()->user();
        $data = array();
        $request->validate([
        'oldpass' => 'required',
        'newpass' => 'required|min:8|different:oldpass',
        'repassword' => 'required|same:newpass',
        ], [
        'oldpass.required' => 'Old Password cannot be empty.',
        'newpass.required' => 'New Password cannot be empty.',
        'newpass.min' => 'New Password must be at least 8 characters long.',
        'newpass.different' => 'New Password must be different from the old password.',
        'repassword.same' => 'Re-entered Password does not match the New Password.',
         ]);
            
        if (!$this->authService->checkPassword($user, $request->oldpass)) { 
                $data['status'] = false;
          $data['message'] = '';  
           return redirect()->back()->with('errmessage' , 'Old Password does not match our record');
            }
        
       
        Auth::user()->update([
                'password' => bcrypt($request->newpass),
                'last_update_password' => \Carbon\Carbon::now(),
                ]);
             
       return redirect()->back()->with('sucmessage' , 'Password updated Successfully');
     
    }
    
    public function delete_activity($id){
        $user = auth()->user();
        $ulogin = UserLogin::where('user_id' , $user->id)->where('lid' , $id)->first();
        if(empty($ulogin)){
            return redirect()->back()->with('errmessage' , 'Login Activity does not exist');
        }
        $ulogin->delete();
        return redirect()->back();
    }
    
    public function clear_activity(){
        $user = auth()->user();
        $ulogin = UserLogin::where('user_id' , $user->id)->get();
        if(empty($ulogin)){
            return redirect()->back()->with('errmessage' , 'Login Activity does not exist');
        }
        foreach($ulogin as $login):
        $login->delete();
        endforeach;
        return redirect()->back()->with('sucmessage' , 'Login Activity has been cleared');
    }
    /****
     * 
     * **/
    public function activity(): View{
        $title = "User Activity";
        $user = auth()->user();
 
       
        return view('user.profile.activity' , [
            'title' => $title,
            'user' => $user,
            'u_logins' => UserLogin::where('user_id' , $user->id)->orderBy('session' , 'desc')->paginate(20),
        ]);
    }
    
    public function security_activity(Request $request){
        
        $user = auth()->user();
        $data = array();
        if($request->ajax()){

            if($user->activity_log == 'on'){
                
                $user->activity_log = 'off';
                $user->save();
                $data['status'] = true;
                $data['message'] = 'Activity Logs have been turned off';
                echo json_encode($data);
                return false;
            
            }else{
                $user->activity_log = 'on';
                $user->save(); 
                $data['status'] = true;
                $data['message'] = 'Activity Logs have been turned on';
                echo json_encode($data);
                return false;
            } 
        }
        
    }
    
    /****
     * 
     * **/
    public function kyc(): View{
        $title = "Kyc Verification";
        $user = auth()->user();
 
       
        return view('user.profile.kyc' , [
            'title' => $title,
            'user' => $user,
        ]);
    }
    
    public function uploadKyc(Request $request)
{
    // Validate the request
    $validated = $request->validate([
        'name' => 'required|string|max:255',
        'dob' => 'required|date',
        'phone' => 'required|string|max:15',
        'country' => 'required|string|max:100',
        'state' => 'required|string|max:100',
        'city' => 'required|string|max:100',
        'zipcode' => 'required|string|max:10',
        'address' => 'required|string|max:255',
        'id-proof' => 'required|string',
        'passport' => 'nullable|file|mimes:jpeg,png,jpg,pdf|max:42048', // Max 2MB
        'passport2' => 'nullable|file|mimes:jpeg,png,jpg,pdf|max:42048', // Max 2MB (for nidcard back)
    ]);

    // File upload helper
    $id_front = $this->uploadFile($request->file('passport'), 'kyc/idfront/');
    $id_back = $request->hasFile('passport2') ? $this->uploadFile($request->file('passport2'), 'kyc/idback/') : null;

    // Update user data
    DB::table('users')->where('id' , auth()->user()->id)->update([
        'name' => $validated['name'],
        'dob' => $validated['dob'],
        'phone' => $validated['phone'],
        'country' => $validated['country'],
        'state' => $validated['state'],
        'city' => $validated['city'],
        'zipcode' => $validated['zipcode'],
        'address' => $validated['address'],
        'document_type' => $validated['id-proof'],
        'document_front' => $id_front,
        'document_back' => $id_back,
        'kyc_status' => 1,
    ]);

    return redirect()->back()->with('succmsg', 'KYC UPDATED. Please wait for admin to verify');
}

/**
 * Upload a file and return the new file name.
 *
 * @param  \Illuminate\Http\UploadedFile|null $file
 * @param  string $path
 * @return string|null
 */
private function uploadFile($file, $path)
{
    if (!$file) {
        return null;
    }

    $filename = md5(uniqid() . time()) . '.' . $file->getClientOriginalExtension();
    $file->storeAs($path, $filename, 'public');
    return $filename;
}

    
 
}