<?php

namespace App\Http\Controllers\User;

use App\Http\Controllers\User\UserController as BaseController;

use App\Models\Notification as UserNotification;
use App\Models\Transaction;
use App\Models\User;
use App\Models\Stock;
use App\Models\StockPrice;
use App\Models\Symbol;
use App\Models\SymbolGroup;
use App\Models\AccountIo;
use App\Models\TradingAccount;


use App\Jobs\ProcessEmail;
use App\Notifications\TransactionNotification;
use App\Enums\NotificationType;
use App\Enums\TransactionStatus;


use Illuminate\Http\RedirectResponse;
use Illuminate\Http\Request;
use Illuminate\Http\Response;
use Illuminate\Support\Facades\Auth;
use Illuminate\Support\Facades\Hash;
use Illuminate\Support\Facades\Log;
use Illuminate\Support\Facades\Redirect;
use Illuminate\Support\Str;
use Illuminate\Validation\ValidationException;
use Illuminate\View\View;

use Illuminate\Support\Facades\Notification;



use Arr;
use Carbon\Carbon;
use DB;
use Session;



class TradeController extends BaseController{
     
public function  tradeStock(Request $request , $id){
      $title = 'Orders';
      $user = auth()->user();
      $symbols = SymbolGroup::symbolByType('stock')?->map(function ($query) {
            $price = $query->dstockprice()->first();
            $query->setAttribute('price', $price ? $price->price : 0);
            return $query;
        });
  
      //$first = $trade->first();
      //dd($first->dstockprice()->first());
      $acc = AccountIo::where('user' , $user->id)->findOrFail(__get_hash($id));
      
      if($acc->balance <= 0){
          return redirect()->route('dashboard')->with('errmessage' , 'Please load the account');
      }
      return view('user.trade.stock' , [
          'title' => 'Deposits',
          'user' => $user,
          'account' => $acc,
          'symbols' => $symbols
          ]);
    }
    
    public function buyStock(Request $request) {
    // Validate input
    $validated = $request->validate([
        'stock' => 'required|string',
        'aid' => 'required|string',
        'amount' => 'required|numeric|min:0.01',
        'stock_price' => 'required|numeric|min:0.01'
    ]);

    $user = auth()->user();


    return DB::transaction(function () use ($user, $request) {
       
        $symbol = Symbol::findOrFail(__get_hash($request->stock));

        // Find account
        $account = AccountIo::where('user', $user->id)
            ->findOrFail(__get_hash($request->aid));
       $amount = $request->stock_price * $request->amount;
        // Check balance
        if ($account->balance < $amount) {
            return redirect()->back()
                ->with('errmessage', 'Insufficient balance. Please deposit.');
        }
      
        // Create stock order
        $stockOrder = Stock::create([
            'user_id' => $user->id,
            'order_num' => 'OD' . Str::upper(Str::random(8)), // More unique
            'symbol' => $symbol->id,
            'volume' => $request->amount,
            'wallet_id' => $account->id,
            'price' => $request->stock_price,
            'type' => 'purchase',
            'status' => 'Open',
            'start_date' => now()
        ]);
        
        
        // Update account balance
        $account->decrement('balance', $amount); 
        __balance_history([
            'amount' => $amount,
            'user' => $user->id
            ] , 'sub');
        return redirect()->back()
            ->with('sucmessage', "Bought {$request->amount} units of {$symbol->symbol}");
    });

    // Transaction will auto-rollback if any operation fails
}

    public function reInvestStock($id)
{
    $user = auth()->user();
    $stock = Stock::where('user_id' , $user->id)->findOrFail(__get_hash($id));
    
    // Ensure stock price exists
    if (!$stock->stockprice()) {
        return redirect()->back()->with('errmessage', 'Stock price unavailable.');
    }

    $price = $stock->stockprice()->price;
    if ($price <= 0) {
        return redirect()->back()->with('errmessage', 'Invalid stock price.');
    }

    $volume = $stock->dividend / $price;

    
    // Check if the user's account exists before decrementing balance
    $account = AccountIo::where('user', $user->id)
        ->where('id', $stock->wallet_id)
        ->first();

    if (!$account) {
        return redirect()->back()->with('errmessage', 'Account not found.');
    }
    $dividends = $stock->dividend;
    // Create a new stock order
    $stockOrder = Stock::create([
        'user_id' => $user->id,
        'order_num' => 'OD' . Str::upper(Str::random(8)), // Unique Order ID
        'symbol' => $stock->symbol,
        'volume' => $volume,
        'wallet_id' => $stock->wallet_id,
        'price' => $price,
        'type' => 'reinvestment',
        'status' => 'Open',
        'start_date' => now()
    ]);


    // Deduct balance
    $account->decrement('balance', $stock->dividend);
    $stock->decrement('dividend', $dividends);

    // Log balance deduction
    __balance_history([
        'amount' => $dividends,
        'user' => $user->id
    ], 'sub');

    return redirect()->back()->with('sucmessage', "Bought " . number_format($volume, 2) . " units of {$stock->symbols()->symbol}");
}
 
 public function  startTrading(Request $request , $id){
      $title = 'Start Trading';
      
      $user = auth()->user();
      $acc = AccountIo::where('user' , $user->id)->findOrFail(__get_hash($id));
      $instruments = $acc->accountUnder()->instruments;
      $symbols = SymbolGroup::where('status' , 1)->whereIn('id' , $instruments)->get();
      if($acc->balance <= 0){
          return redirect()->back()->with('errmessage' , 'Please fund the account');
      }
      return view('user.trade.start' , [
          'title' => 'Deposits',
          'user' => $user,
          'account' => $acc,
          'symbols' => $symbols
          ]);
    }
    
    public function addWebTrader(Request $request) {
    // Validate input
    
    
    $validated = $request->validate([
        'instrument' => 'required|array',
        'aid' => 'required|string',
        'leverage' => 'required|string',
        'platform' => 'required|string',
        'pass' => 'required|string'
    ]);
    
    $user = auth()->user();


    return DB::transaction(function () use ($user, $request) {

        // Find account
        $tid  = "TID" .random_string('numeric' , 8);
        $account = AccountIo::where('user', $user->id)
            ->findOrFail(__get_hash($request->aid));
        
        
        // Check balance

        $tradeaccount = TradingAccount::where('account_id' , $account->id)->where('user_id' ,  $user->id)->first();
        
        if (!blank($tradeaccount)) {
            return redirect()->back()
                ->with('errmessage', 'A trading account already exists under this account.');
        }
        
        $instrument = [];
    
        foreach($request->instrument as $ins){
        $symbol = SymbolGroup::where('id' , __get_hash($ins))->first();
    
            if(!blank($symbol)){
                $instrument[] = $symbol->id;
            }
        }
    
     $array = [
         'user_id' => $user->id,
         'trade_id' => $tid,
         'account_used' => 'real',
         'currency' => $user->user_curr(),
         'status' => 'active',
         'demo_balance' => 0,
         'account_id' => $account->id,
         'leverage' => $request->leverage,
         'instrument' => $instrument,
         'password' => $request->pass,
         'date_made' => now()
         ];
      
        // Create stock order
        $tAccount = TradingAccount::create($array);
        
        
        // Update account balance
        return redirect()->route('user.trade.webtrader' , $tid)
            ->with('sucmessage', "Trading account created");
    });

    // Transaction will auto-rollback if any operation fails
}

    public function webtrader(Request $request, $id)
{
    $title = 'Webtrader';
    
    $user = auth()->user();
    $tradeAccount = TradingAccount::where('user_id', $user->id)
                                  ->where('trade_id', $id)
                                  ->firstOrFail();
    $symbols = [];
    foreach($tradeAccount->instrument as $ins){
        $symbol = SymbolGroup::where('id' , $ins)->first();
            if(!blank($symbol)){
                $s = Symbol::where('cat' , $symbol->id)->get();
                $symbols[$symbol->group_name] = $s;
            }
    }
    dd($symbols);
    //if(is_array($tradeAccount))
    return view('user.trade.webtrader', [
        'title' => $title,
        'user' => $user,
        'tradeAccount' => $tradeAccount,
        'symbols' => $symbols
    ]);
}

}