<?php

namespace App\Jobs;

use App\Enums\EmailRecipientType;
use App\Enums\EmailTemplateStatus;
use App\Enums\UserRoles;

use App\Models\Email_template;
use App\Models\User;

use App\Mail\SendCustom;
use Illuminate\Bus\Queueable;
use Illuminate\Contracts\Queue\ShouldQueue;
use Illuminate\Foundation\Bus\Dispatchable;
use Illuminate\Queue\InteractsWithQueue;
use Illuminate\Queue\SerializesModels;
use Illuminate\Support\Facades\Log;
use Illuminate\Support\Facades\Mail;
use Illuminate\Validation\ValidationException;

class ProcessCustom implements ShouldQueue
{
    use Dispatchable, InteractsWithQueue, Queueable, SerializesModels;

    private $slug;
    private $data;
    private $order;
    private $mailTo;
    private $others;
    private $catchs;

    public function __construct($slug, $mailTo, $data)
    {
        $this->slug = $slug;
        $this->data = $data;
        $this->mailTo = $mailTo;

    }

    private function getTemplate($slug)
    {
        $template = Email_template::where('slug', $slug)->first();
        if (blank($template)) {
            throw new \Exception(__("Invalid email template!"));
        }
        return $template;
    }



    public function handle()
    {
        
        try {
            $template = $this->getTemplate($this->slug);

            if ($template->status == EmailTemplateStatus::ACTIVE) {
                // Send email
                Mail::to($this->mailTo)
                    ->send(new SendCustom($this->slug, $template, $this->data));
                
                \Log::info('Email sent successfully to: ' . ($this->mailTo ?? $this->user->email));
            }
        } catch (\Exception $e) {
            // Log any exceptions
            \Log::error('Error in ProcessEmail job: ' . $e->getMessage());
         
            if ($this->catchs == true) {
                throw new \Exception($e->getMessage());
            }
        }
    }
}

