<?php

namespace App\Models;
use App\Enums\TransactionType;
use App\Enums\TransactionStatus;

use App\Models\Coin;
use App\Models\PaymentMethod;
use App\Models\WithdrawalMethod;
use App\Models\User;

use Illuminate\Database\Eloquent\Factories\HasFactory;
use Illuminate\Database\Eloquent\Model;
use Illuminate\Notifications\Notifiable;
use DB;

class Transaction extends Model
{
    use HasFactory , Notifiable;
    
    protected $casts = [
        'config' => 'array'
        ];
    
    public function user(){
        return $this->belongsTo(User::class,'user','id')->first();
    }
    
    public function account(){
        return $this->belongsTo(AccountIo::class,'account_id','id')->first();
    }
    
    public function depositMethod(){
        return $this->hasOne(PaymentMethod::class , 'id' , 'method');
    }
    
    public function withdrawalMethod(){
        return $this->hasOne(WithdrawalMethod::class , 'id' , 'method');
    }
    
    public function get_config($data){
        if(blank($this->config) || !is_array($this->config)){
            return "";
        }
        if(isset($this->config[$data])){
            return $this->config[$data];
        }
        return "";
    }
    
    public function scopePending($query) {
    return $query->where(function ($q) {
        $q->where('status', TransactionStatus::NOT_PAID)
          ->orWhere('status', TransactionStatus::PENDING);
    });
    }
    
    public function scopeConfirmed($query) {
    return $query->where(function ($q) {
        $q->where('status', TransactionStatus::COMPLETE)
          ->orWhere('status', TransactionStatus::PROCESSED)->orWhere('status', TransactionStatus::CONFIRMED)->orWhere('status', TransactionStatus::APPROVED);
    });
    }

    public static function pendingTransaction($id){
        return Self::where('user' ,  $id)
        ->pending();
    }
    
    public static function confirmedTransaction($id){
        return Self::where('user' ,  $id)
        ->confirmed();
    }
    
    public static function pendingTransactionType($type , $id){
        return Self::where('user' ,  $id)
        ->where('type' , $type)
        ->pending();
    }
    
    public static function confirmedTransactionType($type , $id){
        return Self::where('user' ,  $id)
        ->where('type' , $type)
        ->confirmed();
    }
    
    public static function transactionSummary($userId)
{
    return [
        'pending_withdrawal' => self::pendingTransactionType(TransactionType::WITHDRAW, $userId)->sum('amount'),
        'pending_deposit' => self::pendingTransactionType(TransactionType::DEPOSIT, $userId)->sum('amount'),
        'confirmed_withdrawal' => self::confirmedTransactionType(TransactionType::WITHDRAW, $userId)->sum('amount'),
        'confirmed_deposit' => self::confirmedTransactionType(TransactionType::DEPOSIT, $userId)->sum('amount'),
    ];
}
    
    
    public static function total_referral_by_date($id , $start_date){
        
        return DB::table('transactions')->select(DB::raw("sum(amount) as count"))->where('user' , $id)->where('type' , 'referral')->where('created_at','>',$start_date)->first();
    }
    
    public function depositEndDate(){
        return \Carbon\Carbon::parse($this->created_date)->addSeconds(172800);
    }
}
