<?php

namespace App\Models;

use App\Models\AccountIo;
use App\Models\TradingAccount;
use App\Models\Transaction;

// use Illuminate\Contracts\Auth\MustVerifyEmail;
use Illuminate\Database\Eloquent\Factories\HasFactory;
use Illuminate\Foundation\Auth\User as Authenticatable;
use Illuminate\Notifications\Notifiable;

use DB;

class User extends Authenticatable
{
    use HasFactory, Notifiable;

    /**
     * The attributes that are mass assignable.
     *
     * @var array<int, string>
     */
    protected $fillable = [
        'role',
        'username',
        'name',
        'email',
        'password',
        'pnum',
        'phone',
        'country',
        'currency',
        'email_verify',
        'kyc_status',
        'email_verified_at',
        'join_date',
        'last_login',
        'act_session',
        'act_code',
        'status'
    ];

    /**
     * The attributes that should be hidden for serialization.
     *
     * @var array<int, string>
     */
    protected $hidden = [
        'password',
        'remember_token',
    ];

    /**
     * Get the attributes that should be cast.
     *
     * @return array<string, string>
     */
    protected function casts(): array
    {
        return [
            'email_verified_at' => 'datetime',
            'password' => 'hashed',
        ];
    }
    
    public function account()
    {
    return AccountIo::where('user', $this->id)->where('id', $this->account_id)->first();
    }
    
    public function accounts()
{
    return $this->hasMany(AccountIo::class, 'user', 'id');
}

public function tradingAccounts()
{
    return $this->hasMany(TradingAccount::class, 'user_id', 'id');
}

    
    public function user_initials()
    {
        $array = explode(" ", trim($this->first_name));
        if (count($array) > 1) {
            return strtoupper(substr(trim($array[0]), 0, 1)) . strtoupper(substr(trim($array[1]), 0, 1));
        } elseif (count($array) === 1) {
            return strtoupper(substr(trim($array[0]), 0, 2));
        }
        return "NA";
    }
    
    public function user_curr(){
        if(!blank($this->currency)){
          return   $this->currency;
        }
        return __system_settings('currency');
    }
    public function crypto_curr(){
        if(!blank($this->crypto_currency)){
          return   $this->crypto_currency;
        }
        return __system_settings('crypto_currency');
    }
    
    public function get_address()
    {
        $address = '';
        if (!empty($this->address)) {
            $address .= $this->address;
            if (!empty($this->city)) {
                $address .= ', ' . $this->city;
                if (!empty($this->state)) {
                    $address .= ', ' . $this->state;
                    if (!empty($this->zipcode)) {
                        $address .= '. ' . $this->zipcode;
                    }
                }
            }
        }

        if (!empty($address)) {
            return $address;
        } else {
            return null;
        }
    }
    
    public static function get_all_user($limit){
       
       return self::where('role' , 'user')->orderBy('join_date' , 'desc')->paginate($limit);
    }
    
    public static function get_all_count(){
    
    $total_all = DB::table('users')->where('role' , 'user')->count();  
    return $total_all;
    }
    
    public static function get_active_count(){
        $total_pending = DB::table('users')->where('role' , 'user')->where('status','active')->count(); 
        return $total_pending;
    }
    
    public static function get_inactive_count(){
        $total_pending = DB::table('users')->where('role' , 'user')->where('status','inactive')->count();
        return $total_pending;
    }
    
    public static function get_disabled_count(){
        $total_pending = DB::table('users')->where('role' , 'user')->where('status','disabled')->count();
        return $total_pending;
    }
    
    public function transactions(){
        return $this->hasMany(Transaction::class,'user','id');
    }
    

    
   
}
