<?php

namespace App\Models;

use App\Models\AccountIo;
use App\Models\Plan;
use Illuminate\Database\Eloquent\Factories\HasFactory;
use Illuminate\Database\Eloquent\Model;

use App\Enums\PlanStatus;
use DB;

class UserPlan extends Model
{
    use HasFactory;
    
     protected $casts = [
        'data_meta' => 'array'
        ];
    protected $table = 'user_plans';
    public function dplan(){
        return $this->belongsTo(Plan::class , 'plan', 'id')->where('id' , $this->plan)->first();
    }
    
    public function duser(){
        return $this->belongsTo(User::class , 'user', 'id')->where('id' , $this->user)->first();
    }
    
    public function account(){
        return $this->belongsTo(AccountIo::class , 'account', 'id')->first();
    }
    
   public function get_meta_value($data){
        if(blank($this->data_meta) || !is_array($this->data_meta)){
            return "";
        }
        if(isset($this->data_meta[$data])){
            return $this->data_meta[$data];
        }
        return "";
    }
     
    public static function total_invested($id){
        $data = DB::table('user_plans')->select(DB::raw("SUM(amount) as count"))->where('user', $id)->first();
        return $data;
    }
    
    public static function total_profit($id){
        $data = DB::table('user_plans')->select(DB::raw("SUM(roi) as count"))->where('user', $id)->first();
        return $data;
    }
    
    public static function active_invested($id){
        $data = DB::table('user_plans')->select(DB::raw("SUM(amount) as count"))->where('user', $id)->where('active' , PlanStatus::ACTIVE)->first();
        return $data;
    }
    
    public static function active_profit($id){
        $data = DB::table('user_plans')->select(DB::raw("SUM(roi) as count"))->where('user', $id)->where('active' , PlanStatus::ACTIVE)->first();
        return $data;
    }
    
    public static function active_plans($id){
        $data = self::where('user', $id)->where('active' , PlanStatus::ACTIVE)->orderBy('activated_at' , 'desc')->get();
        return $data;
    }
    
    public static function paused_plans($id){
        $data = self::where('user', $id)->where('active' , PlanStatus::PAUSED)->orderBy('activated_at' , 'desc')->get();
        return $data;
    }
    public static function inactive_plans($id){
        $data = self::where('user', $id)->where(function($query){ $query->where('active' , PlanStatus::INACTIVE); })->orderBy('activated_at' , 'desc')->get();
        return $data;
    }
    public static function ended_plans($id){
        $data = self::where('user', $id)->where('active' , PlanStatus::ENDED)->orderBy('activated_at' , 'desc')->get();
        return $data;
    }
    
    public function getSummaryTitleAttribute()
    {
        

        return sprintf(
            '%s plan - %s%s %s for %d %s via %s' ,
            ucfirst(get_json_value($this->data_meta, 'name')),
            get_json_value($this->data_meta, 'return'),
            get_json_value($this->data_meta, 'return_type') == 'Percentage' ? '%' : ' '.strtoupper($this->duser()->user_curr()),
            ucfirst(get_json_value($this->data_meta, 'return_interval')),
            get_json_value($this->data_meta, 'duration'),
            __plan_duration(get_json_value($this->data_meta, 'duration') , get_json_value($this->data_meta, 'duration_time')), 
            $this->dwallet()->sym
        );
    }
}
