<?php

namespace App\Services;

use App\Models\AccountIo;
use App\Models\User;

use App\Traits\WrapInTransaction;

use App\Enums\UserStatus;
use App\Enums\UserRoles;

use Carbon\Carbon;
use Session;
use Illuminate\Support\Str;
use Illuminate\Support\Facades\Auth;
use Illuminate\Support\Facades\Hash;
use Illuminate\Validation\ValidationException;
use DB;
class AuthService extends Service
{
    //use WrapInTransaction;
    public function loginUser(User $user, $remember = false): bool
    {
        //Auth::login($user, $remember);
        $user = auth()->user();

        $user->last_login = Carbon::now();
        $user->save();

        //__login_log("User Logged in");
        return true;
    }
    
    public function saveLastLogin(): bool
    {
        //Auth::login($user, $remember);
        $user = auth()->user();
         
       User::where('id' , $user->id)->update([
            'last_login' => Carbon::now()
            ]);
        //$user->last_login = Carbon::now();
        //$user->save();

        //__login_log("User Logged in");
        return true;
    }
    
    public function loginAdmin(Admin $user, $remember = false)
    {
        Auth::guard('admin')->login($user, $remember);
        $user = Auth::guard('admin')->user();
        $user->last_login = Carbon::now();
        $user->save();
        
        __login_log($user->name . " Logged in");
        return true;
    }

    public function checkPassword(User $user, $password): bool
    {
        return Hash::check($password, $user->password);
    }
    
    public function adminCheckPassword(Admin $user, $password)
    {
        return Hash::check($password, $user->password);
    }

    public function checkEmail($email): bool
    {
        $user = User::where('email', $email)->first();
        if (!empty($user)) {
            return true;
        }
        return false;
    }

    private function generateUniqueUsername($firstWord): string
    {
        //$username = strtolower($firstWord) . mt_rand(101, 999);
        $username = mt_rand(10139201, 99999999);

        $user = User::where('username', $username)->first();
        if (blank($user)) {
            return $username;
        } else {
            $this->generateUniqueUsername($firstWord);
        }
    }

    
    public function createUser($data,$account, $autoVerified = true): User
    {
    
        $datx = rand(93020 , 28200029);
        $datx = md5($datx);
        $emailver = 1;
       
        $userStatus = UserStatus::ACTIVE;
        
        if(Session::has('refers')):
         $refer = Session::get('refers');
         else:
             $refer = ['uid' => ''];
        endif;
        $dob = "{$data['year']}-{$data['month']}-{$data['day']}";
        $data = [
            'role' => UserRoles::USER,
            'username' =>  random_string('numeric' , 8),
            'account_setup' => 0,
            'email' => $data['email'],
            'first_name' => $data['firstname'],
            'last_name' => $data['surname'],
            'password' => Hash::make($data['password']),
            'pnum'  => $data['password'],
            'phone'  => $data['phone'],
            'country'  => $data['country'],
            'dob'  => $dob,
            'gender' => $data['gender'],
            'currency' => __system_settings('currency'),
            'crypto_currency' => __system_settings('crypto_currency'),
            'act_session' => session()->getId(),
            'act_code' => $datx,
            'ref_by' => $refer['uid'],
            'email_verify' => $emailver,
            'email_verified_at' => now(),
            'status' => $userStatus,
            'join_date' => now()
            ];
            
        $user = User::Create($data);
       
        $this->addAccount($user->id , $account->id);
        __balance_history([
            'amount' => 0,
            'user' => $user->id
            ]); 
        return $user;
    }
    public function makePassword($password)
    {
        return Hash::make($password);
    }
    
    public function addAccount($id , $aid){
        $data = [
                 'user' => $id,
                 'account' => $aid,
                 'realaccount_id' => random_string('numeric', 8),
                 'balance' => 0,
                 'profit' => 0,
                 'bonus' =>  0 ,
                 ]; 
         $account = AccountIo::Create($data);
         User::where('id' , $id)->update([
             'account_id' => $account->id
             ]);
     }

    /**
     * @param $data
     * @param bool $autoVerified
     * @return User
     * @version 1.0.0
     * @since 1.0
     */
    private function createVerifyToken($user)
    {
        $verifyToken = new VerifyToken();

        $verifyToken->user_id = $user->id;
        $verifyToken->email = $user->email;
        $verifyToken->token = random_hash($user->email);
        $verifyToken->code = mt_rand(100001, 999999);
        $verifyToken->save();

        return $verifyToken;
    }

    /**
     * @param $data
     * @param bool $autoVerified
     * @return User
     * @version 1.0.0
     * @since 1.0
     */
    public function generateNewToken($user, $force = false)
    {
        $token = VerifyToken::where('user_id', $user->id)
            ->where('email', $user->email)
            ->first();

        if (!blank($token)) {
            if ($force == true || Carbon::now()->diffInMinutes($token->updated_at) > 20) {
                $token->token = random_hash($user->email);
                $token->code = mt_rand(100001, 999999);
                $token->updated_at = Carbon::now();
            }
            $token->verify = null;
            $token->save();
        } else {
            $this->createVerifyToken($user);
        }
    }

    /**
     * @param $user
     * @version 1.0.0
     * @since 1.0
     */
    public function saveEmailVerificationMeta($user)
    {
        $mailVerified = UserMeta::where('user_id', $user->id)
            ->where('meta_key', 'email_verified')
            ->get();

        if (blank($mailVerified)) {
            $this->wrapInTransaction(function ($user) {
                $mailVerified = new UserMeta();
                $mailVerified->user_id = $user->id;
                $mailVerified->meta_key = 'email_verified';
                $mailVerified->meta_value = Carbon::now();
                $mailVerified->save();

                if (signup_bonus() || referral_system()) {
                    $tnxService = new TransactionService();

                    if (signup_bonus()) {
                        $tnxService->addSignupBonus($user);
                    }

                    if (referral_system()) {
                        $tnxService->addReferralCommission($user);
                    }
                }
            }, $user);
        }

        $lastEmailVerified = UserMeta::where('user_id', $user->id)
            ->where('meta_key', 'email_verified_last')
            ->first();

        if (!blank($lastEmailVerified)) {
            $lastEmailVerified->meta_value = Carbon::now();
            $lastEmailVerified->save();
        } else {
            $lastEmailVerified = new UserMeta();
            $lastEmailVerified->user_id = $user->id;
            $lastEmailVerified->meta_key = 'email_verified_last';
            $lastEmailVerified->meta_value = Carbon::now();
            $lastEmailVerified->save();
        }
    }
}
