<?php


namespace App\Services;

use \App\Models\AccountName;
use \App\Models\AccountIo;
use \App\Models\User;
use \App\Models\User_balance;
use \App\Models\Plan;
use \App\Models\UserPlan;
use \App\Models\PaymentMethod;
use \App\Models\WithdrawalMethod;
use \App\Models\Transaction;
use App\Traits\WrapInTransaction;
use \App\Models\User_account;

use App\Enums\UserStatus;
use App\Enums\UserRoles;
use App\Enums\TransactionStatus;
use App\Enums\PlanStatus;
use App\Enums\WithdrawStatus;

use DB;
use Carbon\Carbon;
use Illuminate\Support\Str;
use Illuminate\Support\Facades\Auth;
use Illuminate\Support\Facades\Hash;
use Illuminate\Validation\ValidationException;
use Illuminate\Http\Request;
use Session;

class FundsService extends Service
{
    function __construct($exchnge_rate = 0){
        $this->exchange_rate = $exchnge_rate;
    }
    
    public function get_transaction(Request $request)
{
    $user = auth()->user(); // Get the authenticated user
    $type = 'transact';
    
    $query = Transaction::where('user', $user->id)->orderBy('created_date', 'desc');
    
    $requestedType = $request->get('type');
    if ($requestedType) {
        if (in_array($requestedType, ['deposit', 'withdrawal'])) {
            $type = $requestedType;
            $query->where('type', $requestedType);
        }
    }
    

    $transactions = $query->get();
    
    
    return [
        'type' => $type,
        'transactions' => $transactions,
    ];
}

    public function get_transaction_details($typex , $id){
        
        $user = auth()->user();
        $trans =  '';
        $type = '';
        $view = '';
        
        if(!empty($typex)){
            if($typex == 'deposit'){
                $type = 'deposit';
                $trans = Deposit::where('user' , $user->id)->where('txn_id', $id)->orderBy('date_made' , 'desc')->first(); 
                $view = 'depositDetails';
               
            }elseif($typex == 'withdraw'){
                $type = 'withdraw';
                $trans = Withdrawal::where('user' , $user->id)->where('txn_id', $id)->orderBy('date_made' , 'desc')->first();
                $view = 'withdrawalDetails';
            }elseif($typex == 'transfer'){
                $type = 'transfer';
                $trans = Withdrawal::where('user' , $user->id)->where('txn_id', $id)->orderBy('date_made' , 'desc')->first();
                $view = 'transferDetails';
            }elseif($typex == 'transact'){
                $type = 'transact';
                $trans = Transaction::where('user' , $user->id)->where('tx_no', $id)->first();
                $view = 'transactionDetails';
            }
        }
         
        
        return array('type' => $type , 'trans' => $trans, 'view' => $view);
        
    }
    
    public function save_transaction($data){
        $user = auth()->user();
       
        $now = \Carbon\Carbon::now();
        $tx = new Transaction();
        $tx->txnID  = $data['trans_id'];
        $tx->user = $user->id;
        $tx->type = $data['type'];
        $tx->calc = $data['calc'];
        $tx->account_id = $data['accountId'] ?? 0;
        $tx->method = $data['method'] ?? 0;
        $tx->amount =  $data['amount'];
        $tx->descr =  $data['descr'];
        $tx->config = json_encode(__transaction_config($data));
        $tx->status = $data['status'];
        $tx->created_date = $now;
        $tx->created_at = $now; 
        $tx->updated_at = $now;
        $tx->save();
        return $tx;
        
    }
    
    public function save_admin_transaction($data , $type , $plan = ''){
    
       
        $now = \Carbon\Carbon::now();
        $tx = new Transaction();
        $tx->tx_no  = $data['trans_id'];
        $tx->user = $data['user']->id;
        $tx->type_id = $type['id'];
        $tx->type = $type['type'];
        $tx->calc = $type['calc'];
        $tx->plan = $plan;
        $tx->amount =  $data['amount'];
        if(isset($data['config'])){
         $tx->config =    $data['config'];
        }
        
        $tx->exchange_rate =  get_sys_admin_exchange_rate($data['user']);
        $tx->sys_amount = $data['amount'] * $tx->exchange_rate;
        $tx->descr = $type['descr'];
        $tx->created_at = $now;
        $tx->updated_at = $now;
        $tx->save();
        
    }
    
    public function cancel_transaction($id){
        $user = auth()->user();
        $transaction = Transaction::where('txnID' , $id)->where('user' ,$user->id)->first();
        if(empty($transaction)){
        $data['status'] = false;
        $data['error'] = 'Transaction not found';  
        return $data;
        };
        
        $status_array = [
            TransactionStatus::COMPLETE,
            TransactionStatus::PROCESSED,
            TransactionStatus::CONFIRMED,
            TransactionStatus::APPROVED,
            TransactionStatus::CANCELLED,
            TransactionStatus::DECLINED,
            TransactionStatus::FAILED,
            ];
        
        if(in_array($transaction->status , $status_array)){
            $data['status'] = false;
            $data['error'] = $id . ' - cannot alter transaction status';  
            return $data;
        }
        DB::beginTransaction();
        try{
         Transaction::where('id' , $transaction->id)->update([
            'status' => TransactionStatus::CANCELLED,
            'processed_at' => \Carbon\Carbon::now()
            ]);
        DB::commit();
        $data['status'] = true;
        return $data;   
        }catch(\Exception $e){
            DB::rollback();
            $data['status'] = false;
            $data['error'] =  ' - a technical error occured';  
            return $data;
        }
        
        
    }
    
    

    /******** DEPOSIT ********/

    
    public function check_deposits($request){
       
        $user = auth()->user(); 
        $data = array();
        
        if($request['account'] == '' || $request['account'] == NULL){
            $data['status'] = false;
            $data['error'] = 'Select a account to fund';  
            return $data;
        }
        
        $account = AccountIo::where('id' ,  __get_hash($request['account']))->where('user' , $user->id)->first();
       
        if(empty($account)){
            $data['status'] = false;
            $data['error'] = 'Invalid account selected';  
            return $data;
        }
        
       if($request['paymethod'] == '' || $request['paymethod'] == NULL){
            $data['status'] = false;
            $data['error'] = 'Select a payment method';  
            return $data;
        }
        
        $req_payment = __get_hash($request['paymethod']);
        $pmethod = PaymentMethod::where('id' , $req_payment)->first();
       
        if(empty($pmethod)){
            $data['status'] = false;
            $data['error'] = 'Invalid payment method selected';  
            return $data;
        }
        
        
        if($request['amt'] == '' || $request['amt'] == NULL){
        $data['status'] = false;
        $data['error'] = 'Amount can not be empty';  
        return $data;
        }
      
       
        if($request['amt'] < $pmethod->min_deposit()){
        $data['status'] = false;
        $data['error'] = 'Amount is less than minimum amount for this method';  
        return $data;
        } 
        
        $data['status'] = true;
        $data['payment'] = $pmethod;
        $data['account'] = $account;
       
        return $data;
    }
    
    
    
    
    
    /******** WITHDRAWAL ********/
    public function start_withdrawal($request){
        $user = auth()->user(); 
        $data = array();
        $uwithdrawal = Transaction::pendingWithdrawal($user->id); 
        $withdraw_count = $uwithdrawal->count();
        
        if($withdraw_count >= 1){
            if(__system_settings('withdrawal_multiple_allow')  == 'on'){
            
                if(!empty(__system_settings('withdrawal_multiple_cutoff')) && __system_settings('withdrawal_multiple_cutoff')  > 0 ){
                   
                    $cuttoff = __system_settings('withdrawal_multiple_cutoff');
                    
                    if($withdraw_count >= $cuttoff){
                        
                        $data['status'] = false;
                        $data['error'] = 'Withdrawal not allowed. You have '.$withdraw_count.' pending withdrawal; Please complete pending withdrawal first'; 
                        return $data;
                     
                    } 
                }
            
            }else{
                
                $data['status'] = false;
                $data['error'] = 'Multiple Pending Withdrawal Not Allowed'; 
                return $data;
            }
        }
        $accounts = "";
       $wtype = "external";
       if($wtype == 'external'){
          $met = __get_hash($request['paymethod']); 
            if($met == '' || $met == NULL){
                $data['status'] = false;
                
                $data['error'] = 'Please select a withdrawal method';  
                return $data;
            }
            
            $wmethod = WithdrawalMethod::where('id' , $met)->first();
             
            if(empty($wmethod)){
                $data['status'] = false;
                $data['error'] = 'Please select a withdrawal method';  
                return $data;
            }
            $accounts = $wmethod;
        }
     
        $data['status'] = true;
        $data['result'] = 'external';
        $data['account_type'] = 'main';
        $data['method'] = $accounts;
        return $data; 
    }
    
  
    public function checkWithdraw($request){
       
       $user = auth()->user(); 
        $data = array();
        
        if($request['account'] == '' || $request['account'] == NULL){
            $data['status'] = false;
            $data['error'] = 'Select a account to fund';  
            return $data;
        }
        
        $account = AccountIo::where('id' ,  __get_hash($request['account']))->where('user' , $user->id)->first();
       
        if(empty($account)){
            $data['status'] = false;
            $data['error'] = 'Invalid account selected';  
            return $data;
        }
        
       if($request['paymethod'] == '' || $request['paymethod'] == NULL){
            $data['status'] = false;
            $data['error'] = 'Select a payment method';  
            return $data;
        }
        
        $req_payment = __get_hash($request['paymethod']);
        $wmethod = WithdrawalMethod::where('id' , $req_payment)->first();
       
        if(empty($wmethod)){
            $data['status'] = false;
            $data['error'] = 'Invalid payment method selected';  
            return $data;
        }
        
        
        if($request['amt'] == '' || $request['amt'] == NULL){
        $data['status'] = false;
        $data['error'] = 'Amount can not be empty';  
        return $data;
        }
        
        
        if($request['amt'] > $account->balance){
                $data['status'] = false;
                $data['error'] = 'Insufficient Balance';  
                return $data;
        } 
        
        if($request['amt'] < $wmethod->min_withdrawal()){
            $data['status'] = false;
            $data['error'] = 'Amount is less than minimum amount for this method';  
            return $data;
        } 
        
        $data['status'] = true;
        $data['method'] = $wmethod;
        $data['account'] = $account;
        return $data;
        
        
    }
    
   public function cancel_withdrawal($id){
        $user = auth()->user();
        $deposit = Withdrawal::where('txn_id' , $id)->where('user' ,$user->id)->first();
        if(empty($deposit)){
        $data['status'] = false;
        $data['error'] = 'Withdrawal not found';  
        return $data;
        };
        
        if($deposit->status != WithdrawStatus::PENDING  ){
            $data['status'] = false;
            $data['error'] = $id . ' - Unable to alter status';  
            return $data;
        }
        
        Withdrawal::where('id' , $deposit->id)->update([
            'status' => WithdrawStatus::CANCELLED,
            'processed_at' => \Carbon\Carbon::now()
            ]);
        $data['status'] = true;
        return $data;
    }
    
      
    
    /******** PLAN ********/
    public function start_plan($request){
        
        // Check if Plan Exists
        $user = auth()->user();
        $data = array();
        
        $inv_id = __get_hash($request['plan']);
        
        
        $plan= Plan::where('id',$inv_id)->first();
        
        if(empty($plan)){
            $data['status'] = false;
            $data['error'] = 'Plan not found.';
            return $data;
            
        }
       
       $account= AccountIo::where('id', __get_hash($request['account']))->first();
        
        if(empty($account)){
            $data['status'] = false;
            $data['error'] = 'Account selected not found.';
            return $data;
            
        }
       
       
 
        return array('status' => true, 'plan' => $plan, 'account' => $account);

    }
    
    public function check_plan($request)
{
    $user = auth()->user();
    $response = ['status' => false]; 
    
    if (!Session::has('account')) {
        $response['error'] = 'Invalid Account.';
        return $response;
    }
     
    $acc_id = __get_hash($request['iv-account']);
    $sessionAccount = Session::get('account');
    if ($sessionAccount->id != $acc_id) {
        $response['error'] = 'Account not found.';
        return $response;
    }

    $account = AccountIo::find($acc_id);
    if (!$account) {
        $response['error'] = 'Account not found.';
        return $response;
    }
   
    if (!Session::has('plan')) {
        $response['error'] = 'Invalid Plan.';
        return $response;
    }
     
    $inv_id = __get_hash($request['iv-plan']);
    $sessionPlan = Session::get('plan');
    if ($sessionPlan->id != $inv_id) {
        $response['error'] = 'Plan not found.';
        return $response;
    }

    $plan = Plan::find($inv_id);
    if (!$plan) {
        $response['error'] = 'Plan not found.';
        return $response;
    }

    $userPlanExists = UserPlan::where('user', $user->id)
        ->where('plan', $plan->id)
        ->exists();

    $minAmount = $plan->c_price();
    if ($request['amount'] < $minAmount) {
        $response['error'] = 'Your invested amount is less than the plan minimum.';
        return $response;
    }

    if ($account->balance < $request['amount']) {
        $response['error'] = 'Your wallet investment balance is insufficient to purchase this plan.';
        return $response;
    }

    return ['status' => true, 'plan' => $plan , 'account' => $account];
}

    
    public function save_plan($data){
    
    
    $user = auth()->user();
    $plan = $data['plan'];
    $nowx = \Carbon\Carbon::now();
    $nowx2 = \Carbon\Carbon::now();
    if($plan->duration > 0){
    
    $plandd = $plan->duration * __plan_duration_seconds($plan->duration_time);
    $new_time = $nowx->addSecond($plandd);
    }
    else{
      $new_time  = '';
    }
    $meta = ['name' => $plan->name , 'type'=>$plan->type, 'return_interval' => $plan->intervals ,'return' => $plan->percs , 'return_type' => $plan->in_type  , 'duration'=>$plan->duration , 'duration_time'=>$plan->duration_time ] ;
 
    $planx =  [
            'plan' => $plan->id,
            'txn_id' => $data['trans_id'],
            'user' => $user->id,
            'amount' => $data['amount'],
            'account' => $data['account']->id,
            'exchange_rate' => __system_exchange_rate(),
            'active' => PlanStatus::ACTIVE,
            //'wallet_io' => $acctype->id,
            'roi' => 0 ,
            'data_meta' => json_encode($meta),
            'activated_at' => $nowx2,
            'end_date' => $new_time,
            'last_growth' => $nowx2,
            'created_at' => $nowx2,
            'updated_at' => $nowx2,
            ];
   
    
    $userplanid = DB::table('user_plans')->insertGetId(
            $planx
        );
        return $userplanid;
    }
    
}
