<?php

namespace App\Services;

//use App\Models\Transaction;
//use App\Models\Deposit;
//use App\Models\Withdrawal;
//use App\Models\User_plan;
//use App\Models\Support;
use App\Models\User;
//use App\Enums\TransactionType;

use Illuminate\Support\Facades\URL;

class Shortcut extends Service
{
    private $shortcuts = [];

    public function __construct()
    {
        $this->setDefaultShortcuts();
    }

    private function setDefaultShortcuts()
    {
        $defaultShortcuts = [
            "[[site_name]]" => __system_settings("site_name"),
            "[[site_url]]" => URL::to('/'),
            "[[site_email]]" => __system_settings('site_email'),
        ];

        $this->shortcuts = array_merge($this->shortcuts, $defaultShortcuts);

        return $this;
    }

    public function setOrderShortcuts($order)
    {
   
        if (!blank($order)) {
            if ($order instanceof Transaction) {
                $orderShortcuts = [
                    "[[order_id]]" => the_tnx($order->tnx),
                    "[[order_by]]" => $order->transaction_by->username,
                    "[[order_time]]" => show_date($order->created_at),
                    "[[order_amount]]" => money($order->tnx_amount, $order->tnx_currency),
                    "[[order_all_amount]]" => ($order->currency == $order->tnx_currency) ? money($order->amount, $order->currency) : money($order->tnx_amount, $order->tnx_currency) . ' (' . money($order->amount, $order->currency) . ')',
                    "[[order_details]]" => $this->generateDetailTable($order->shortcut_details),
                    "[[withdraw_details]]" => $this->generateDetailTable($order->shortcut_details),
                    "[[withdraw_to]]" => $order->pay_to,
                    "[[payment_method]]" => $order->method_name,
                    "[[withdraw_reference]]" => $order->reference,
                    "[[withdraw_note]]" => $order->note,
                    "[[payment_information]]" => $order->payment_info,
                ];

                if ($order->type == TransactionType::TRANSFER) {
                    $orderShortcutsTransfer = [
                        "[[transfer_details]]" => $this->generateDetailTable($order->shortcut_details),
                        "[[user_to_name]]" => get_user(data_get($order, 'meta.transfer.user'))->username,
                        "[[user_to_email]]" => get_user(data_get($order, 'meta.transfer.user'))->email,
                        "[[user_from_name]]" => get_user(data_get($order, 'user_id'))->username,
                        "[[user_from_email]]" => get_user(data_get($order, 'user_id'))->email,
                    ];
                    $orderShortcuts = array_merge($orderShortcutsTransfer, $orderShortcuts);
                }
            }
            
            if ($order instanceof Deposit) {
                $orderShortcuts = [
                    "[[order_id]]" => $order->txn_id,
                    "[[order_by]]" => $order->account()->fname,
                    "[[order_account]]" => $order->account,
                    "[[order_type]]" => $order->type,
                    "[[order_code]]" => $order->code,
                    "[[order_time]]" => __date_parse($order->date_made),
                    "[[order_amount]]" => money($order->amount, $order->account()->currency),
                     "[[order_details]]" => $this->generateDetailTable($order->shortcut_details()),
                    "[[payment_method]]" => $order->dpayment()->name,
                    "[[order_wallet]]" => $order->dwallet()->sym,
                    "[[payment_information]]" => $order->payment_info,
                ];
            
            }
           
            if ($order instanceof Withdrawal) {
                $orderShortcuts = [
                    "[[order_id]]" => $order->txn_id,
                    "[[order_by]]" => $order->duser()->fname,
                    "[[order_account]]" => $order->account,
                    "[[order_wtype]]" => $order->wtype,
                    "[[order_type]]" => $order->type,
                    "[[order_withdraw_mode]]" => $order->w_mode,
                    "[[order_time]]" => __date_parse($order->date_made),
                    "[[order_amount]]" => money($order->amount, $order->duser()->currency),
                    "[[order_updated_bal]]" => money($order->updated_bal, $order->duser()->currency),
                   
                     "[[withdraw_details]]" => $this->generateDetailTable($order->shortcut_details()),
                     "[[transfer_details]]" => $this->generateDetailTable($order->transfer_shortcut_details()),
                    "[[order_wallet]]" => $order->dwallet_from()->sym,
                    
                    "[[payment_information]]" => $order->payment_info,
                ];
                if(!blank($order->dwallet_to())){
                   $orderShortcuts["[[order_wallet_to]]"]  = $order->dwallet_to()->sym;
                }
                if(!blank($order->dmode())){
                   $orderShortcuts["[[withdraw_method]]"]  = $order->dmode()->name;
                }
             
            }

            if ($order instanceof User_plan) {
                $orderShortcuts = [
                    "[[order_id]]" => $order->txn_id,
                    "[[order_by]]" => $order->duser()->fname,
                    "[[order_time]]" => __date_parse($order->activated_at),
                    "[[order_end]]" => __date_parse($order->end_time),
                    "[[plan_name]]" => get_json_value($order->data_meta, 'name'),
                    "[[invest_amount]]" => money($order->amount, $order->duser()->user_curr()),
                    "[[order_wallet]]" => $order->dwallet()->sym,
                    "[[plan_return]]" => get_json_value($order->data_meta, 'return').'% '.get_json_value($order->data_meta, 'return_interval'),
                    "[[invest_details]]" => $order->getSummaryTitleAttribute()
                ];
            }
            
            if ($order instanceof Support) {
                
                 $orderShortcuts = [
                    "[[support_id]]" => $order->ticketno,
                    "[[support_subject]]" => $order->subject,
                    "[[support_message]]" => $order->message,
                    "[[support_user]]" => $order->account()->fname,
                    "[[support_user_email]]" => $order->account()->email,
                ];
                
            }

            $this->shortcuts = array_merge($this->shortcuts, $orderShortcuts);
    
        }

        return $this;
    }

    private function generateDetailTable($data)
    {
        if (empty($data)) {
            return "";
        }

        $table = '<table width="100%">';
        foreach ($data as $key => $value) {
            $table .= $value ? '<tr><td width="150">' . ucwords(str_replace('_', ' ', $key)) . '</td><td width="25">&nbsp;&nbsp;:&nbsp;&nbsp;</td><td>'. $value .'</td></tr>' : '';
        }
        $table .= '</table>';

        return $table;
    }

    public function setUserShortcuts(User $user)
    {
        $userShortcuts = [
            '[[user_id]]' => __the_uid($user->id),
            "[[username]]" => $user->username,
            "[[user_fname]]" => $user->fname,
            "[[user_lname]]" => $user->lname,
            "[[user_name]]" => $user->fname . ' ' .$user->lname,
            "[[user_email]]" => $user->email,
            "[[user_detail]]" => $this->generateDetailTable($user->shortcut_details ?? []),
        ];
       
        $this->shortcuts = array_merge($this->shortcuts, $userShortcuts);

        return $this;
    }
    
    public function setDataShortcuts($data)
    {
        $dataShortcut =  array();
        
        if (!is_null($data)) {
        foreach ($data as $key => $value) {
            $placeholder = "[[{$key}]]";
            $dataShortcut[$placeholder] = $value;
        }
       }
       
        $this->shortcuts = array_merge($this->shortcuts, $dataShortcut);

        return $this;
    }

    public function processContent(string $content)
    {
        return strtr($content, $this->shortcuts);
    }
}
