
@extends('layouts.base')

@section('title', __($transaction->txnID.' | Complete Deposit'))

@section('css')

<style>
    .dark-mode .card-stats-data .amount , .dark-mode .popup-close{
        color: #fff !important;
    }
    .dark-mode .popup-content{
    background-color: #141c26;
    }
</style>

@endsection

@section('content')
<div class="nk-content nk-content-fluid">
    <div class="container-xl wide-lg">
        <div class="nk-content-body">
            <div class="nk-block">
            <div class="page-dw wide-xs m-auto">
                <div class="nk-pps-apps">
                    <div class="nk-pps-title text-center">
                        <h3 class="title">Make Your Payment</h3>
                         @include('components.error-message')
                        <p class="caption-text">
                            Your deposit <strong class="text-dark">{{$transaction->txnID}}</strong> has been placed successfully. To complete, please send the exact amount of <strong class="text-dark">{{$transaction->amount}} {{$transaction->get_config('payment_currency')}}</strong> to the address below.
                        </p>
                        @if ($errors->has('tnx'))
                        <div class="alert alert-danger alert-thick alert-plain">
                        <div class="alert-cta flex-wrap flex-md-nowrap g-2">
                            <div class="alert-text has-icon">
                                <em class="icon ni ni-info-fill text-danger"></em>
                                <p>{{$errors->first('tnx')}}</p>
                            </div>
                            <button type="button" class="close" data-bs-dismiss="alert" aria-label="Close">
    <span aria-hidden="true" style="color:#000">&times;</span>
  </button> 
                        </div>
                    </div>
                    
                      @endif
                    </div>
                    <div class="nk-pps-card card card-bordered popup-inside">
                        <div class="card-inner-group">
                            <div class="card-inner card-inner-sm">
                                <div class="card-head mb-0">
                                    <h6 class="title mb-0">Pay {{$transaction->get_config('payment_currency')}}</h6>
                                    <div class="card-opt"><span class="counter" data-countdown-second="{{$timeLeft}}" id="counterclock" data-countdown-text="Expire in">Expire in {{__date_parse($transaction->depositEndDate())}}</span></div>
                                </div>
                            </div>
                            <div class="card-inner">
                                @if($depositMethod->slug === 'crypto-deposit')
                                <div class="qr-media mx-auto mb-3 w-max-100px">
                                @if($transaction->get_config('qr_code') == 'show')
                                    {!! QrCode::size(200)->generate($transaction->get_config('qrcode')); !!}
                                    @endif
                                </div>
                                <div class="pay-info text-center">
                                    <h5 class="title text-dark mb-0 clipboard-init" data-clipboard-text="{{$transaction->amount}}">{{$transaction->amount}} {{$transaction->get_config('payment_currency')}} <em class="click-to-copy icon ni ni-copy-fill nk-tooltip" title="" data-original-title="Click to Copy"></em></h5>
                                    <p class="text-soft">{{$transaction->amount}} {{$user->user_curr()}}</p>
                                </div>
                                @elseif($depositMethod->slug === 'bank-transfer')
                                <div class="pay-info text-center">
                                    <h5 class="title text-dark mb-0 clipboard-init" data-clipboard-text="{{$transaction->amount}}">{{$transaction->amount}} {{$transaction->get_config('payment_currency')}} <em class="click-to-copy icon ni ni-copy-fill nk-tooltip" title="" data-original-title="Click to Copy"></em></h5>
                                    <p class="text-soft">{{$transaction->amount}} {{$user->user_curr()}}</p>
                                </div>
                                @endif
                                <div class="form-group">
                                    <div class="form-label overline-title-alt lg text-center">{{$transaction->get_config('payment')}} Address</div>
                                    <div class="form-control-wrap">
                                        <div class="form-clip clipboard-init nk-tooltip" data-clipboard-target="#wallet-address" title="" data-original-title="Copy">
                                            <em class="click-to-copy icon ni ni-copy"></em>
                                        </div>
                                        <div class="form-icon"></div>
                                        <input readonly="" type="text" class="form-control form-control-lg" id="wallet-address" value="{{$depositMethod->details}}" />
                                    </div>
                                </div>

                                <div class="nk-pps-action">
                                    <a href="#crypto-paid" class="btn btn-block btn-primary popup-open"><span>Paid {{$transaction->get_config('payment')}}</span></a>
                                </div>
                                <div class="nk-pps-action pt-2 text-center">
                                    <a href="{{route('user.funds.history')}}?type=deposit" class="link link-btn link-primary">Pay Later</a>
                                </div>
                                <div id="crypto-paid" class="popup">
                                    <div class="popup-content">
                                        <h6 class="mb-2">Confirm your payment</h6>
                                        <p>If you already paid, please provide us your payment reference to speed up verification procces.</p>
                                        <form class="form" action="{{route('user.funds.deposit.save')}}" method="POST" id="crypto-pay-reference">
                                            <div class="form-group">
                                                <div class="form-label">Payment Reference <span class="text-danger">*</span></div>
                                                <div class="form-control-wrap">
                                                    <input name="reference" type="text" class="form-control" value="" placeholder="Enter your reference id / hash" required />
                                                </div>
                                            </div>
                                            <ul class="btn-group justify-between align-center gx-4">
                                                <li><button type="submit" id="confirm-payment" class="btn btn-primary btn-block">Confirm Payment</button></li>
                                                <li><a href="#" class="link link-btn link-secondary popup-close">Close</a></li>
                                            </ul>
                                            @csrf
                                            <input type="hidden" name="transact_id" value="{{__the_hash($transaction->txnID)}}" />
                                            <input type="hidden" name="transact_hash" value="{{__the_hash($transaction->id)}}" />
                                        </form>
                                        <div class="alert-note is-plain mt-4">
                                            <em class="icon ni ni-alert-circle"></em>
                                            <p>Account will credited once we confirm that payment has been received.</p>
                                        </div>
                                    </div>
                                    <div class="popup-overlay"></div>
                                </div>
                            </div>
                            <div class="card-inner bg-lighter">
                                <ul>
                                    <li class="alert-note is-plain text-danger">
                                        <em class="icon ni ni-alert-circle"></em>
                                        <p>Be aware of that this order will be cancelled, if you send any other {{$transaction->get_config('payment_currency')}} amount.</p>
                                    </li>

                                    <li class="alert-note is-plain text-primary">
                                        <em class="icon ni ni-info"></em>
                                        <p>Account will credited once we received your payment.</p>
                                    </li>
                                </ul>
                            </div>
                        </div>
                    </div>
                    <div class="nk-pps-action mt-n2">
                        <ul class="btn-group justify-between align-center gy-3">
                            <li><a href="{{route('user.funds.transaction.cancel' , __the_hash($transaction->txnID))}}" class="link link-danger">Cancel Order</a></li>
                            <li>
                                <a href="{{secure_url('dashboard')}}" class="link link-primary"><span>Back to Dashboard</span> <em class="icon ni ni-arrow-long-right"></em></a>
                            </li>
                        </ul>
                    </div>
                </div>
            </div>
                
            </div>
        </div>
    </div>
</div>

@endsection

@section('modals')


@endsection

@section('scripts')
<script>
    $('.popup-open').on('click' , function(){
        var id = $(this).attr('href');
        $(id).addClass('opened');
    });
    
    $('.popup-close').on('click' , function(){
        $('.popup').removeClass('opened');
    });
    $('#confirm-payment').on('click' , function(){
        $('#crypto-pay-reference').submit();
    })
</script>
<script>
    function makeTimer() {

    const $counterclock = $('#counterclock');

    let timeLeft = parseInt($counterclock.attr('data-countdown-second'), 10);

    if (timeLeft <= 0) {
        $counterclock.html("00:00:00");
        return;
    }

    timeLeft--;
    const hours = String(Math.floor(timeLeft / 3600)).padStart(2, '0'); // Total hours (no 24-hour limit)
    const minutes = String(Math.floor((timeLeft % 3600) / 60)).padStart(2, '0');
    const seconds = String(timeLeft % 60).padStart(2, '0');

    $counterclock.html(`${hours}:${minutes}:${seconds}`);
    $counterclock.attr('data-countdown-second', timeLeft);
}

  setInterval(makeTimer, 1000);

</script>
@endsection