<?php

use Illuminate\Support\Facades\Route;
use App\Http\Controllers\Admin\CoinController;
use App\Http\Controllers\Admin\DashboardController;
use App\Http\Controllers\Admin\OrderController;
use App\Http\Controllers\Admin\PlanController;
use App\Http\Controllers\Admin\ProfileController;
use App\Http\Controllers\Admin\SettingsController;
use App\Http\Controllers\Admin\TradeController;
use App\Http\Controllers\Admin\TransactionController;
use App\Http\Controllers\Admin\UsersController;




Route::prefix('admin/')->group( function(){
   
    Route::get('dashboard', [DashboardController::class, 'index'])->name('dashboard');
    
    Route::prefix('user/')->name('user.')->group( function(){
       Route::get('list/{type?}' ,[UsersController::class , 'userList'] )->name('index');  
       Route::get('kyc' ,[UsersController::class , 'kycList'] )->name('kyc');
       Route::get('email' ,[UsersController::class , 'sendEmail'] )->name('email');
       Route::get('delete/{id}', [UsersController::class , 'deleteUser'])->name('delete');
       Route::post('permission/edit', [UsersController::class , 'editPermission'])->name('permission.edit');
       Route::post('profile/edit', [UsersController::class , 'editProfile'])->name('profile.edit');
       Route::get('kycaction/{action}/{id}', [UsersController::class , 'kycAction'])->name('kyc.action');
       Route::post('private_key', [UsersController::class , 'privateKey'])->name('private.key');
       Route::post('hold_user', [UsersController::class , 'holdUser'])->name('hold');
       Route::post('permissions', [UsersController::class , 'Permissions'])->name('permission');
       Route::post('billing', [UsersController::class , 'Billing'])->name('billing');
       Route::post('topup', 'UsersController@topup')->name('topup');
       Route::post('notifyuser', [UsersController::class , 'Notify'])->name('notify');
        Route::get('{id}', [UsersController::class , 'user'])->name('single');
    });
    
    Route::prefix('transactions/')->name('transaction.')->group( function(){
        
        Route::get('list/{id?}' ,[TransactionController::class , 'List'])->name('list'); 
        Route::get('single/{id?}' ,[TransactionController::class , 'Single'] )->name('single');
        Route::get('all_list/{type?}/{status?}' ,[TransactionController::class , 'allList'])->name('list.all'); 
        Route::get('delete/{id}', [TransactionController::class , 'deleteTransaction'])->name('delete');
        Route::post('process' ,[TransactionController::class , 'processTransaction'])->name('process'); 
        Route::post('edit' ,[TransactionController::class , 'editTransaction'])->name('edit'); 
        
    });
    
    Route::prefix('orders/')->name('order.')->group( function(){
        
        Route::get('list/{id?}' ,[OrderController::class , 'List'])->name('list'); 
        Route::get('single/{id?}' ,[OrderController::class , 'Single'] )->name('single');
        Route::get('all_list/{type?}/{status?}' ,[OrderController::class , 'allList'])->name('list.all'); 
        Route::get('delete/{id}', [OrderController::class , 'deleteOrder'])->name('delete');
        Route::post('edit' ,[OrderController::class , 'editOrder'])->name('edit'); 
        
    });
    
    Route::prefix('investment/')->name('investment.')->group( function(){
        
        
        Route::get('/pause/{id}', 'PlanController@pauseInvestment')->name('pause');
        Route::get('resume/{id}', 'PlanController@resumeInvestment')->name('resume');
        Route::get('auto/{id}', 'PlanController@autoInvestment')->name('auto');
        Route::get('end_auto/{id}', 'PlanController@endAutoInvestment')->name('admin.investment.autoend');
        Route::get('end/{id}', 'PlanController@endInvestment')->name('end');
        Route::post('topup', 'PlanController@topProfit')->name('topup');
        Route::get('/{user?}', 'PlanController@invest')->name('index');
        
        
        Route::get('admin/addinvestment', 'DepositsController@addplan')->name('admin.plan.form');
        Route::post('admin/addinvestment', 'DepositsController@addplan')->name('admin.plan.dd');
    });
    
    Route::prefix('plans/')->name('plan.')->group( function(){
    
        Route::post('add', [PlanController::class, 'addPlan'])->name('add');
        Route::post('edit', [PlanController::class, 'editPlan'])->name('edit');
        Route::get('delete/{id}', [PlanController::class, 'deletePlan'])->name('delete');
        Route::get('{type?}', [PlanController::class, 'index'])->name('index');
    });
    
    Route::prefix('symbols/')->name('symbols.')->group( function(){
    
        Route::post('add', [TradeController::class, 'addSymbols'])->name('add');
        Route::post('edit', [TradeController::class, 'editSymbols'])->name('edit');
        Route::get('delete/{id}', [TradeController::class, 'deleteSymbols'])->name('delete');
        Route::get('{type?}', [TradeController::class, 'index'])->name('index');
    });
    
    Route::get('profile', [ProfileController::class, 'profile'])->name('profile');
    Route::post('profile/update', [ProfileController::class, 'updateProfile'])->name('profile.update');
    Route::get('password', [ProfileController::class, 'changePassword'])->name('password');
    Route::post('password/change', [ProfileController::class, 'changePasswordPost'])->name('password.update');
    
    Route::get('settings', [SettingsController::class, 'settings'])->name('settings');
    
    Route::prefix('settings/')->name('settings.')->group( function(){
        
      Route::get('general', [SettingsController::class, 'general'])->name('general'); 
      Route::post('general', [SettingsController::class, 'updateGeneral'])->name('general.update');
      Route::get('payment', [SettingsController::class, 'payment'])->name('payment'); 
      Route::post('payment', [SettingsController::class, 'updatePayment'])->name('payment.update');
      
      Route::get('tier', [SettingsController::class, 'tier'])->name('tier'); 
      Route::post('tier', [SettingsController::class, 'updateTier'])->name('tier.update');
      Route::post('tier/add', [SettingsController::class, 'addTier'])->name('tier.add'); 
      
      Route::get('logo', [SettingsController::class, 'logo'])->name('logo'); 
      Route::post('logo', [SettingsController::class, 'updateLogo'])->name('logo.update'); 
      
      Route::get('permissions', [SettingsController::class, 'permissions'])->name('permissions'); 
      Route::post('permissions', [SettingsController::class, 'updatePermissions'])->name('permissions.update');
      
      Route::get('api', [SettingsController::class, 'api'])->name('api'); 
      Route::post('api', [SettingsController::class, 'updateApi'])->name('api.update'); 
      
      Route::get('seo', [SettingsController::class, 'seo'])->name('seo');
       Route::post('seo', [SettingsController::class, 'updateSeo'])->name('seo.update');
      
      Route::get('extensions', [SettingsController::class, 'extensions'])->name('extensions'); 
      Route::post('extensions', [SettingsController::class, 'updateExtensions'])->name('extensions.update'); 
     
    });
});
