<?php

use App\Http\Controllers\Auth\AuthenticatedSessionController;
use App\Http\Controllers\Auth\ConfirmablePasswordController;
use App\Http\Controllers\Auth\EmailVerificationNotificationController;
use App\Http\Controllers\Auth\EmailVerificationPromptController;
use App\Http\Controllers\Auth\NewPasswordController;
use App\Http\Controllers\Auth\PasswordController;
use App\Http\Controllers\Auth\PasswordResetLinkController;
use App\Http\Controllers\Auth\RegisteredUserController;
use App\Http\Controllers\Auth\VerifyEmailController;
use Illuminate\Support\Facades\Route;

Route::middleware('guest')->name('auth.')->group(function () {
    Route::get('login', [AuthenticatedSessionController::class, 'create'])
                ->name('login.form');
    Route::post('login', [AuthenticatedSessionController::class, 'store'])->name('login');
    Route::get('open-account', [RegisteredUserController::class, 'create'])
                ->name('register.start');
    Route::get('create-account/{slug}/{id}', [RegisteredUserController::class, 'registerForm'])->name('register');
    Route::post('register/otp', [RegisteredUserController::class, 'codeRegister'])->name('register.post');
    Route::post('register/save', [RegisteredUserController::class, 'store'])->name('register.save');
    Route::get('forgot-password', [PasswordResetLinkController::class, 'create'])->name('password.request');
    

});

Route::middleware('auth')->group(function () {
    Route::get('verify-email', EmailVerificationPromptController::class)
                ->name('verification.notice');

    Route::get('verify-email/{id}/{hash}', VerifyEmailController::class)
                ->middleware(['signed', 'throttle:6,1'])
                ->name('verification.verify');

    Route::post('email/verification-notification', [EmailVerificationNotificationController::class, 'store'])
                ->middleware('throttle:6,1')
                ->name('verification.send');

    Route::get('confirm-password', [ConfirmablePasswordController::class, 'show'])
                ->name('password.confirm');

    Route::post('confirm-password', [ConfirmablePasswordController::class, 'store']);

    Route::put('password', [PasswordController::class, 'update'])->name('password.update');

    Route::post('logout', [AuthenticatedSessionController::class, 'destroy'])
                ->name('auth.logout');
});
