<?php

use Illuminate\Support\Facades\Route;
use App\Http\Controllers\User\AccountController;
use App\Http\Controllers\User\DashboardController;
use App\Http\Controllers\User\FundsController;
use App\Http\Controllers\User\MarketsController;
use App\Http\Controllers\User\MessageController;
use App\Http\Controllers\User\PlansController;
use App\Http\Controllers\User\ProfileController;
use App\Http\Controllers\User\SupportController;
use App\Http\Controllers\User\TradeController;

Route::get('dashboard', [DashboardController::class , 'index'])->name('dashboard');

Route::get('referrals', [DashboardController::class , 'referrals'])->name('referrals');
Route::get('support', [DashboardController::class , 'support'])->name('support');

Route::get('my-accounts', [AccountController::class , 'myAccount'])->name('account.my');
Route::get('create-new-account', [AccountController::class , 'createAccount'])->name('account.new');
Route::get('open-new-account/{id}', [AccountController::class , 'openNewAccount'])->name('account.open'); 
Route::get('switch-account/{id}', [AccountController::class , 'switchAccount'])->name('account.switch'); 

Route::prefix('trades/')->name('trade.')->group(function(){
       
     Route::get('webtrader' , [TradeController::class , 'webtrader'])->name('webtrader');
     Route::get('copytrader' , [TradeController::class , 'copyTrader'])->name('copytrader');
     Route::get('history' , [TradeController::class , 'history'])->name('history');
     Route::get('close/{id}' , [TradeController::class , 'closeTrade'])->name('close');
     Route::post('new' , [TradeController::class , 'newTrade'])->name('new');
        
});

Route::prefix('funds/')->name('funds.')->group(function(){
       
     Route::get('deposits' , [FundsController::class , 'deposits'])->name('deposits');
     Route::get('deposits/cancel', 'FundsController@cancelDeposit')->name('deposits.cancel');
     Route::post('checkdeposit', 'FundsController@checkdeposit')->name('deposit.check');
     Route::get('deposit/confirm/{id?}', 'FundsController@confirmDeposit')->name('deposit.confirm');
     Route::get('/cancel/view/{id?}', 'FundsController@cancelTransactionView')->name('cancel.view');
     Route::get('/cancel/{id?}', 'FundsController@cancelTransactionMade')->name('transaction.cancel');
     Route::get('deposit/{id}' , [FundsController::class , 'deposit'])->name('deposit');
     
     Route::get('withdrawals' , [FundsController::class , 'withdrawals'])->name('withdrawals');
     Route::post('startwithdrawal', 'FundsController@startWithdrawal')->name('withdrawal.start');
     Route::post('checkwithdrawal', 'FundsController@checkWithdrawal')->name('withdrawal.check');
     Route::post('withdrawal/confirm/{id?}', 'FundsController@confirmWithdrawal')->name('withdrawal.confirm');
     Route::get('withdrawal/{id?}', 'FundsController@withdrawal')->name('withdrawal');
     Route::post('savedeposit', 'FundsController@saveDeposit')->name('deposit.save');
     
     Route::get('history' , [FundsController::class , 'history'])->name('history');
        
});

Route::prefix('profile/')->name('profile.')->group(function(){
       
     Route::get('account' , [ProfileController::class , 'index'])->name('account');
     Route::post('edit' , [ProfileController::class , 'editProfile'])->name('edit');
     Route::post('address' , [ProfileController::class , 'editAddress'])->name('address');
     
     Route::get('security' , [ProfileController::class , 'security'])->name('security');
     Route::post('updatepass' , [ProfileController::class , 'updatePassword'])->name('password.update');
     Route::post('switch2fa/{token}/{state}' , [ProfileController::class , 'enable2Fa'])->name('2fa.switch');
     Route::get('activity' , [ProfileController::class , 'activity'])->name('activity');
     Route::post('activity/switch' , [ProfileController::class , 'switchActivity'])->name('activity.switch');
     Route::get('kyc' , [ProfileController::class , 'kyc'])->name('kyc');  
     Route::post('kyc/upload' , [ProfileController::class , 'uploadKyc'])->name('kyc.upload');
     
        
});



Route::prefix('plans/')->name('plans.')->group(function(){
       
     Route::get('index/{id}' , [PlansController::class , 'index'])->name('index');
     Route::post('start' , [PlansController::class , 'startPlan'])->name('start');
     Route::get('invest' , [PlansController::class , 'payPlan'])->name('invest');
     Route::post('confirm' , [PlansController::class , 'joinPlan'])->name('confirm');
     Route::get('myplan' , [PlansController::class , 'userPlan'])->name('user');
     Route::get('single/{id}' , [PlansController::class , 'singlePlan'])->name('single');
     Route::get('upgrade' , [PlansController::class , 'upgradePlan'])->name('upgrade');
        
});

Route::get('trade-stocks/{id}', [TradeController::class , 'tradeStock'])->name('trade.stock');
Route::post('buy-stocks', [TradeController::class , 'buyStock'])->name('stock.buy');
Route::post('reinvest-stocks', [TradeController::class , 'reInvestStock'])->name('stock.reinvest');
Route::get('create-webtrader/{id}', [TradeController::class , 'startTrading'])->name('webtrader.create');
Route::post('create-webtrader/{id}', [TradeController::class , 'addWebTrader'])->name('webtrader.save');
Route::get('webtrader/{id}', [TradeController::class , 'webtrader'])->name('trade.webtrader');


Route::prefix('support/')->name('support.')->group(function(){
       
     Route::get('video' , [SupportController::class , 'video'])->name('video');
     Route::get('contact' , [SupportController::class , 'contact'])->name('contact');
        
});
